@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------*/
 /*--- Callgrind                                                    ---*/
-/*---                                                    ct_bbcc.c ---*/
+/*---                                                       bbcc.c ---*/
 /*--------------------------------------------------------------------*/
 
 /*
@@ -29,6 +29,8 @@
 #include "global.h"
 #include "costs.h"
 
+#include <pub_tool_threadstate.h>
+
 /*------------------------------------------------------------*/
 /*--- BBCC operations                                      ---*/
 /*------------------------------------------------------------*/
@@ -38,36 +40,36 @@
 /* BBCC table (key is BB/Context), per thread, resizable */
 bbcc_hash current_bbccs;
 
-void SK_(init_bbcc_hash)(bbcc_hash* bbccs)
+void CLG_(init_bbcc_hash)(bbcc_hash* bbccs)
 {
    Int i;
 
-   CT_ASSERT(bbccs != 0);
+   CLG_ASSERT(bbccs != 0);
 
    bbccs->size    = N_BBCC_INITIAL_ENTRIES;
    bbccs->entries = 0;
-   bbccs->table = (BBCC**) VG_(malloc)(bbccs->size * sizeof(BBCC*));
+   bbccs->table = (BBCC**) CLG_MALLOC(bbccs->size * sizeof(BBCC*));
 
    for (i = 0; i < bbccs->size; i++) bbccs->table[i] = NULL;
 }
 
-void SK_(copy_current_bbcc_hash)(bbcc_hash* dst)
+void CLG_(copy_current_bbcc_hash)(bbcc_hash* dst)
 {
-  CT_ASSERT(dst != 0);
+  CLG_ASSERT(dst != 0);
 
   dst->size    = current_bbccs.size;
   dst->entries = current_bbccs.entries;
   dst->table   = current_bbccs.table;
 }
 
-bbcc_hash* SK_(get_current_bbcc_hash)()
+bbcc_hash* CLG_(get_current_bbcc_hash)()
 {
   return &current_bbccs;
 }
 
-void SK_(set_current_bbcc_hash)(bbcc_hash* h)
+void CLG_(set_current_bbcc_hash)(bbcc_hash* h)
 {
-  CT_ASSERT(h != 0);
+  CLG_ASSERT(h != 0);
 
   current_bbccs.size    = h->size;
   current_bbccs.entries = h->entries;
@@ -77,35 +79,36 @@ void SK_(set_current_bbcc_hash)(bbcc_hash* h)
 /*
  * Zero all costs of a BBCC
  */
-void SK_(zero_bbcc)(BBCC* bbcc)
+void CLG_(zero_bbcc)(BBCC* bbcc)
 {
   Int i;
   jCC* jcc;
 
-  CT_ASSERT(bbcc->cxt != 0);
-  CT_DEBUG(1, "  zero_bbcc: BB 0x%x, Cxt %d "
+  CLG_ASSERT(bbcc->cxt != 0);
+  CLG_DEBUG(1, "  zero_bbcc: BB %p, Cxt %d "
 	   "(fn '%s', rec %d)\n", 
 	   bb_addr(bbcc->bb),
 	   bbcc->cxt->base_number + bbcc->rec_index,
 	   bbcc->cxt->fn[0]->name,
 	   bbcc->rec_index);
 
-  if ((bbcc->exe_counter ==0) &&
+  if ((bbcc->ecounter_sum ==0) &&
       (bbcc->ret_counter ==0)) return;
 
   for(i=0;i<bbcc->bb->cost_count;i++)
     bbcc->cost[i] = 0;
-   
-  for(jcc=bbcc->jcc_list; jcc; jcc=jcc->next_from)
-    SK_(init_cost)( SK_(sets).full, jcc->cost );
-
-  bbcc->exe_counter = 0;
+  for(i=0;i <= bbcc->bb->cjmp_count;i++) {
+    bbcc->jmp[i].ecounter = 0;
+    for(jcc=bbcc->jmp[i].jcc_list; jcc; jcc=jcc->next_from)
+	CLG_(init_cost)( CLG_(sets).full, jcc->cost );
+  }
+  bbcc->ecounter_sum = 0;
   bbcc->ret_counter = 0;
 }
 
 
 
-void SK_(forall_bbccs)(void (*func)(BBCC*))
+void CLG_(forall_bbccs)(void (*func)(BBCC*))
 {
   BBCC *bbcc, *bbcc2;
   int i, j;
@@ -114,7 +117,7 @@ void SK_(forall_bbccs)(void (*func)(BBCC*))
     if ((bbcc=current_bbccs.table[i]) == NULL) continue;
     while (bbcc) {
       /* every bbcc should have a rec_array */
-      CT_ASSERT(bbcc->rec_array != 0);
+      CLG_ASSERT(bbcc->rec_array != 0);
 
       for(j=0;j<bbcc->cxt->fn[0]->separate_recursions;j++) {
 	if ((bbcc2 = bbcc->rec_array[j]) == 0) continue;
@@ -138,10 +141,10 @@ void SK_(forall_bbccs)(void (*func)(BBCC*))
  */
 
 static __inline__
-UInt hash_bbcc(BB* bb, Context* cxt, UInt size)
+UInt bbcc_hash_idx(BB* bb, Context* cxt, UInt size)
 {
-   CT_ASSERT(bb != 0);
-   CT_ASSERT(cxt != 0);
+   CLG_ASSERT(bb != 0);
+   CLG_ASSERT(cxt != 0);
 
    return ((Addr)bb + (Addr)cxt) % size;
 }
@@ -153,33 +156,33 @@ static
 BBCC* lookup_bbcc(BB* bb, Context* cxt)
 {
    BBCC* bbcc = bb->last_bbcc;
-   UInt  hash;
+   UInt  idx;
 
    /* check LRU */
    if (bbcc->cxt == cxt) {
-       if (!SK_(clo).separate_threads) {
+       if (!CLG_(clo).separate_threads) {
 	   /* if we don't dump threads separate, tid doesn't have to match */
 	   return bbcc;
        }
-       if (bbcc->tid == SK_(current_tid)) return bbcc;
+       if (bbcc->tid == CLG_(current_tid)) return bbcc;
    }
 
-   SK_(stat).bbcc_lru_misses++;
+   CLG_(stat).bbcc_lru_misses++;
 
-   hash = hash_bbcc(bb, cxt, current_bbccs.size);
-   bbcc = current_bbccs.table[hash];
+   idx = bbcc_hash_idx(bb, cxt, current_bbccs.size);
+   bbcc = current_bbccs.table[idx];
    while (bbcc &&
 	  (bb      != bbcc->bb ||
 	   cxt     != bbcc->cxt)) {
        bbcc = bbcc->next;
    }
    
-   CT_DEBUG(2,"  lookup_bbcc(BB 0x%x, Cxt %d, fn '%s'): 0x%p (tid %d)\n",
+   CLG_DEBUG(2,"  lookup_bbcc(BB %p, Cxt %d, fn '%s'): %p (tid %d)\n",
 	    bb_addr(bb), cxt->base_number, cxt->fn[0]->name, 
 	    bbcc, bbcc ? bbcc->tid : 0);
 
-   CT_DEBUGIF(2)
-     if (bbcc) SK_(print_bbcc)(-2,bbcc,False);
+   CLG_DEBUGIF(2)
+     if (bbcc) CLG_(print_bbcc)(-2,bbcc,False);
 
    return bbcc;
 }
@@ -190,11 +193,11 @@ static void resize_bbcc_hash()
 {
     Int i, new_size, conflicts1 = 0, conflicts2 = 0;
     BBCC** new_table;
-    UInt new_hash;
+    UInt new_idx;
     BBCC *curr_BBCC, *next_BBCC;
 
     new_size = 2*current_bbccs.size+3;
-    new_table = (BBCC**) VG_(malloc)(new_size * sizeof(BBCC*));
+    new_table = (BBCC**) CLG_MALLOC(new_size * sizeof(BBCC*));
  
     if (!new_table) return;
  
@@ -208,12 +211,12 @@ static void resize_bbcc_hash()
 	while (NULL != curr_BBCC) {
 	    next_BBCC = curr_BBCC->next;
 
-	    new_hash = hash_bbcc(curr_BBCC->bb,
-				 curr_BBCC->cxt,
-				 new_size);
+	    new_idx = bbcc_hash_idx(curr_BBCC->bb,
+				    curr_BBCC->cxt,
+				    new_size);
 
-	    curr_BBCC->next = new_table[new_hash];
-	    new_table[new_hash] = curr_BBCC;
+	    curr_BBCC->next = new_table[new_idx];
+	    new_table[new_idx] = curr_BBCC;
 	    if (curr_BBCC->next) {
 		conflicts1++;
 		if (curr_BBCC->next->next)
@@ -227,13 +230,13 @@ static void resize_bbcc_hash()
     VG_(free)(current_bbccs.table);
 
 
-    CT_DEBUG(0,"Resize BBCC Hash: %d => %d (entries %d, conflicts %d/%d)\n",
+    CLG_DEBUG(0,"Resize BBCC Hash: %d => %d (entries %d, conflicts %d/%d)\n",
 	     current_bbccs.size, new_size,
 	     current_bbccs.entries, conflicts1, conflicts2);
 
     current_bbccs.size = new_size;
     current_bbccs.table = new_table;
-    SK_(stat).bbcc_hash_resizes++;
+    CLG_(stat).bbcc_hash_resizes++;
 }
 
 
@@ -243,11 +246,11 @@ BBCC** new_recursion(int size)
     BBCC** bbccs;
     int i;
     
-    bbccs = (BBCC**) VG_(malloc)(sizeof(BBCC*) * size);
+    bbccs = (BBCC**) CLG_MALLOC(sizeof(BBCC*) * size);
     for(i=0;i<size;i++)
 	bbccs[i] = 0;
 
-    CT_DEBUG(3,"  new_recursion(size %d): 0x%p\n", size, bbccs);
+    CLG_DEBUG(3,"  new_recursion(size %d): %p\n", size, bbccs);
     
     return bbccs;
 }
@@ -265,28 +268,34 @@ BBCC* new_bbcc(BB* bb)
    BBCC* new;
    Int i;
 
-   new = (BBCC*)VG_(malloc)(sizeof(BBCC));
+   /* We need cjmp_count+1 JmpData structs:
+    * the last is for the unconditional jump/call/ret at end of BB
+    */
+   new = (BBCC*)CLG_MALLOC(sizeof(BBCC) +
+			   (bb->cjmp_count+1) * sizeof(JmpData));
    new->bb  = bb;
-   new->tid = SK_(current_tid);
+   new->tid = CLG_(current_tid);
 
-   new->exe_counter = 0;
    new->ret_counter = 0;
    new->skipped = 0;
-   new->cost = SK_(get_costarray)(bb->cost_count);
+   new->cost = CLG_(get_costarray)(bb->cost_count);
    for(i=0;i<bb->cost_count;i++)
      new->cost[i] = 0;
+   for(i=0; i<=bb->cjmp_count; i++) {
+       new->jmp[i].ecounter = 0;
+       new->jmp[i].jcc_list = 0;
+   }
+   new->ecounter_sum = 0;
 
    /* Init pointer caches (LRU) */
    new->lru_next_bbcc = 0;
    new->lru_from_jcc  = 0;
    new->lru_to_jcc  = 0;
    
-   new->jcc_list  = 0;
+   CLG_(stat).distinct_bbccs++;
 
-   SK_(stat).distinct_bbccs++;
-
-   CT_DEBUG(3, "  new_bbcc(BB 0x%x): %p (now %d)\n", 
-	    bb_addr(bb), new, SK_(stat).distinct_bbccs);
+   CLG_DEBUG(3, "  new_bbcc(BB %p): %p (now %d)\n", 
+	    bb_addr(bb), new, CLG_(stat).distinct_bbccs);
 
    return new;
 }
@@ -306,11 +315,11 @@ BBCC* new_bbcc(BB* bb)
 static
 void insert_bbcc_into_hash(BBCC* bbcc)
 {
-    UInt hash;
+    UInt idx;
     
-    CT_ASSERT(bbcc->cxt != 0);
+    CLG_ASSERT(bbcc->cxt != 0);
 
-    CT_DEBUG(3,"+ insert_bbcc_into_hash(BB 0x%x, fn '%s')\n",
+    CLG_DEBUG(3,"+ insert_bbcc_into_hash(BB %p, fn '%s')\n",
 	     bb_addr(bbcc->bb), bbcc->cxt->fn[0]->name);
 
     /* check fill degree of hash and resize if needed (>90%) */
@@ -318,11 +327,11 @@ void insert_bbcc_into_hash(BBCC* bbcc)
     if (100 * current_bbccs.entries / current_bbccs.size > 90)
 	resize_bbcc_hash();
 
-    hash = hash_bbcc(bbcc->bb, bbcc->cxt, current_bbccs.size);
-    bbcc->next = current_bbccs.table[hash];
-    current_bbccs.table[hash] = bbcc;
+    idx = bbcc_hash_idx(bbcc->bb, bbcc->cxt, current_bbccs.size);
+    bbcc->next = current_bbccs.table[idx];
+    current_bbccs.table[idx] = bbcc;
 
-    CT_DEBUG(3,"- insert_bbcc_into_hash: %d entries\n",
+    CLG_DEBUG(3,"- insert_bbcc_into_hash: %d entries\n",
 	     current_bbccs.entries);
 }
 
@@ -359,7 +368,7 @@ static BBCC* clone_bbcc(BBCC* orig, Context* cxt, Int rec_index)
 {
     BBCC*      new;
 
-    CT_DEBUG(3,"+ clone_bbcc(BB 0x%x, rec %d, fn %s)\n",
+    CLG_DEBUG(3,"+ clone_bbcc(BB %p, rec %d, fn %s)\n",
 	     bb_addr(orig->bb), rec_index, cxt->fn[0]->name);
 
     new  = new_bbcc(orig->bb);
@@ -367,7 +376,7 @@ static BBCC* clone_bbcc(BBCC* orig, Context* cxt, Int rec_index)
     if (rec_index == 0) {
 
       /* hash insertion is only allowed if tid or cxt is different */
-      CT_ASSERT((orig->tid != SK_(current_tid)) ||
+      CLG_ASSERT((orig->tid != CLG_(current_tid)) ||
 		(orig->cxt != cxt));
 
       new->rec_index = 0;
@@ -378,13 +387,13 @@ static BBCC* clone_bbcc(BBCC* orig, Context* cxt, Int rec_index)
       insert_bbcc_into_hash(new);
     }
     else {
-      if (SK_(clo).separate_threads)
-	CT_ASSERT(orig->tid == SK_(current_tid));
+      if (CLG_(clo).separate_threads)
+	CLG_ASSERT(orig->tid == CLG_(current_tid));
 
-      CT_ASSERT(orig->cxt == cxt);
-      CT_ASSERT(orig->rec_array);
-      CT_ASSERT(cxt->fn[0]->separate_recursions > rec_index);
-      CT_ASSERT(orig->rec_array[rec_index] ==0);
+      CLG_ASSERT(orig->cxt == cxt);
+      CLG_ASSERT(orig->rec_array);
+      CLG_ASSERT(cxt->fn[0]->separate_recursions > rec_index);
+      CLG_ASSERT(orig->rec_array[rec_index] ==0);
 
       /* new BBCC will only have differing recursion level */
       new->rec_index = rec_index;
@@ -398,17 +407,17 @@ static BBCC* clone_bbcc(BBCC* orig, Context* cxt, Int rec_index)
     orig->bb->bbcc_list = new;
 
 
-    CT_DEBUGIF(3)
-      SK_(print_bbcc)(-2, new, False);
+    CLG_DEBUGIF(3)
+      CLG_(print_bbcc)(-2, new, False);
 
-    CT_DEBUG(2,"- clone_BBCC(0x%p, %d) for BB 0x%x\n"
+    CLG_DEBUG(2,"- clone_BBCC(%p, %d) for BB %p\n"
 		"   orig %s\n"
 		"   new  %s\n",
 	     orig, rec_index, bb_addr(orig->bb),
 	     mangled_cxt(orig->cxt, orig->rec_index),
 	     mangled_cxt(new->cxt, new->rec_index));
 
-    SK_(stat).bbcc_clones++;
+    CLG_(stat).bbcc_clones++;
  
     return new;
 };
@@ -420,13 +429,13 @@ static BBCC* clone_bbcc(BBCC* orig, Context* cxt, Int rec_index)
  * Also sets BB_seen_before by reference.
  *
  */ 
-BBCC* SK_(get_bbcc)(BB* bb)
+BBCC* CLG_(get_bbcc)(BB* bb)
 {
    BBCC* bbcc;
 
    VGP_PUSHCC(VgpCacheGetBBCC);
 
-   CT_DEBUG(3, "+ get_bbcc(BB 0x%x)\n", bb_addr(bb));
+   CLG_DEBUG(3, "+ get_bbcc(BB %p)\n", bb_addr(bb));
 
    bbcc = bb->bbcc_list;
 
@@ -442,11 +451,11 @@ BBCC* SK_(get_bbcc)(BB* bb)
      bb->bbcc_list = bbcc;
      bb->last_bbcc = bbcc;
 
-     CT_DEBUGIF(3)
-       SK_(print_bbcc)(-2, bbcc, False);
+     CLG_DEBUGIF(3)
+       CLG_(print_bbcc)(-2, bbcc, False);
    }
 
-   CT_DEBUG(3, "- get_bbcc(BB 0x%x): BBCC 0x%p\n",
+   CLG_DEBUG(3, "- get_bbcc(BB %p): BBCC %p\n",
 		bb_addr(bb), bbcc);
 
    VGP_POPCC(VgpCacheGetBBCC);
@@ -458,7 +467,7 @@ BBCC* SK_(get_bbcc)(BB* bb)
  * When leaving a function, a underflow can happen when
  * Callgrind's tracing was switched on in the middle of
  * a run, i.e. when Callgrind was not able to trace the
- * Call.
+ * call instruction.
  * This function tries to reconstruct the original call.
  * As we know the return address (the address following
  * the CALL instruction), we can detect the function
@@ -477,61 +486,61 @@ static void handleUnderflow(BB* bb)
   int fn_number, *pactive;
   call_entry* call_entry_up;
 
-  CT_DEBUG(1,"  Callstack underflow !\n");
+  CLG_DEBUG(1,"  Callstack underflow !\n");
 
   /* we emulate an old call from the function we return to
    * by using (<return address> -1) */
-  source_bb = SK_(get_bb)(bb_addr(bb)-1, 0, &seen_before);
-  source_bbcc = SK_(get_bbcc)(source_bb);
+  source_bb = CLG_(get_bb)(bb_addr(bb)-1, 0, &seen_before);
+  source_bbcc = CLG_(get_bbcc)(source_bb);
 
   /* seen_before can be true if RET from a signal handler */
   if (!seen_before) {
-    source_bbcc->exe_counter = SK_(current_state).collect ? 1 : 0;
+    source_bbcc->ecounter_sum = CLG_(current_state).collect ? 1 : 0;
   }
-  else if (SK_(current_state).collect)
-    source_bbcc->exe_counter++;
+  else if (CLG_(current_state).collect)
+    source_bbcc->ecounter_sum++;
   
   /* Force a new top context, will be set active by push_cxt() */
-  SK_(current_fn_stack).top--;
-  SK_(current_state).cxt = 0;
-  caller = SK_(get_fn_node)(bb);
-  SK_(push_cxt)( caller );
+  CLG_(current_fn_stack).top--;
+  CLG_(current_state).cxt = 0;
+  caller = CLG_(get_fn_node)(bb);
+  CLG_(push_cxt)( caller );
 
   if (!seen_before) {
     /* set rec array for source BBCC: this is at rec level 1 */
     source_bbcc->rec_array = new_recursion(caller->separate_recursions);
     source_bbcc->rec_array[0] = source_bbcc;
 
-    CT_ASSERT(source_bbcc->cxt == 0);
-    source_bbcc->cxt = SK_(current_state).cxt;
+    CLG_ASSERT(source_bbcc->cxt == 0);
+    source_bbcc->cxt = CLG_(current_state).cxt;
     insert_bbcc_into_hash(source_bbcc);
   }
-  CT_ASSERT(SK_(current_state).bbcc);
+  CLG_ASSERT(CLG_(current_state).bbcc);
 
   /* correct active counts */
-  fn_number = SK_(current_state).bbcc->cxt->fn[0]->number;
-  pactive = SK_(get_fn_entry)(fn_number);
+  fn_number = CLG_(current_state).bbcc->cxt->fn[0]->number;
+  pactive = CLG_(get_fn_entry)(fn_number);
   (*pactive)--;
 
   /* This assertion is not correct for reentrant
    * signal handlers */
-  /* CT_ASSERT(*pactive == 0); */
+  /* CLG_ASSERT(*pactive == 0); */
 
-  SK_(current_state).nonskipped = 0; /* we didn't skip this function */
+  CLG_(current_state).nonskipped = 0; /* we didn't skip this function */
   /* back to current context */
-  SK_(push_cxt)( SK_(current_state).bbcc->cxt->fn[0] );
-  SK_(push_call_stack)(source_bbcc, SK_(current_state).bbcc,
+  CLG_(push_cxt)( CLG_(current_state).bbcc->cxt->fn[0] );
+  CLG_(push_call_stack)(source_bbcc, 0, CLG_(current_state).bbcc,
 		       (Addr)-1, False);
   call_entry_up = 
-    &(SK_(current_call_stack).entry[SK_(current_call_stack).sp -1]);
+    &(CLG_(current_call_stack).entry[CLG_(current_call_stack).sp -1]);
   jcc = call_entry_up->jcc;
   /* assume this call is lasting since last dump or
    * for a signal handler since it's call */
-  if (SK_(current_state).sig == 0)
-    SK_(copy_cost)( SK_(sets).full, call_entry_up->enter_cost,
-		    SK_(get_current_thread)()->lastdump_cost );
+  if (CLG_(current_state).sig == 0)
+    CLG_(copy_cost)( CLG_(sets).full, call_entry_up->enter_cost,
+		    CLG_(get_current_thread)()->lastdump_cost );
   else
-    SK_(zero_cost)( SK_(sets).full, call_entry_up->enter_cost );
+    CLG_(zero_cost)( CLG_(sets).full, call_entry_up->enter_cost );
 }
 
 
@@ -540,130 +549,227 @@ static void handleUnderflow(BB* bb)
  * pointer to costs for current thread/context/recursion level
  */
 
-__attribute__ ((regparm (1)))
-void SK_(setup_bbcc)(BB* bb)
+CLG_REGPARM(1)
+void CLG_(setup_bbcc)(BB* bb)
 {
   BBCC *bbcc, *last_bbcc;
   Bool  call_emulation = False, delayed_push = False, skip = False;
-  Addr esp;
+  Addr sp;
   BB* last_bb;
+  ThreadId tid;
+  Int jmpkind, passed = 0, csp;
+  Bool ret_without_call = False;
+  Int popcount_on_return = 1;
 
   VGP_PUSHCC(VgpCacheSetup);
 
-  CT_DEBUG(3,"+ setup_bbcc(BB 0x%x)\n", bb_addr(bb));
+  CLG_DEBUG(3,"+ setup_bbcc(BB %p)\n", bb_addr(bb));
 
-  esp = VG_(get_stack_pointer)();
-  last_bbcc = SK_(current_state).bbcc;
+  /* This is needed because thread switches can not reliable be tracked
+   * with callback CLG_(run_thread) only: we have otherwise no way to get
+   * the thread ID after a signal handler returns.
+   * This could be removed again if that bug is fixed in Valgrind.
+   * This is in the hot path but hopefully not to costly.
+   */
+  tid = VG_(get_running_tid)();
+#if 1
+  CLG_(switch_thread)(tid);
+#else
+  CLG_ASSERT(VG_(get_running_tid)() == CLG_(current_tid));
+#endif
+
+  sp = VG_(get_SP)(tid);
+  last_bbcc = CLG_(current_state).bbcc;
   last_bb = last_bbcc ? last_bbcc->bb : 0;
 
+  if (last_bb) {
+      passed = CLG_(current_state).jmps_passed;
+      if (passed == last_bb->cjmp_count) {
+	  jmpkind = last_bb->jmpkind;
+
+	  /* VEX always gives a Boring jump kind also when passed trough */
+	  if ((jmpkind == Ijk_Boring) &&
+	      (last_bb->offset + last_bb->instr_len == bb->offset))
+	      jmpkind = JmpNone;
+      }
+      else
+	  jmpkind = JmpCond;
+
+      /* if we are in a function which is skipped in the call graph, we
+       * do not increment the exe counter to produce cost (if simulation off),
+       * which would lead to dumping this BB to be skipped
+       */
+      if (CLG_(current_state).collect && !CLG_(current_state).nonskipped) {
+	  last_bbcc->ecounter_sum++;
+	  last_bbcc->jmp[passed].ecounter++;
+	  if (!CLG_(clo).simulate_cache) {
+	      /* update Ir cost */
+	      int instr_count = last_bb->jmp[passed].instr+1;
+	      CLG_(current_state).cost[CLG_(sets).off_sim_Ir] += instr_count;
+	  }
+      }
+
+      CLG_DEBUGIF(4) {
+	  CLG_(print_execstate)(-2, &CLG_(current_state) );
+	  CLG_(print_bbcc_cost)(-2, last_bbcc);
+      }
+  }
+  else {
+      jmpkind = JmpNone;
+  }
+
   /* Manipulate JmpKind if needed, only using BB specific info */
 
-  if (( SK_(current_state).jmpkind != JmpRet) &&
-      ( SK_(current_state).jmpkind != JmpCall) && last_bb) {
+  csp = CLG_(current_call_stack).sp;
+
+  /* A return not matching the top call in our callstack is a jump */
+  if ( (jmpkind == Ijk_Ret) && (csp >0)) {
+      Int csp_up = csp-1;      
+      call_entry* top_ce = &(CLG_(current_call_stack).entry[csp_up]);
+
+      /* We have a real return if
+       * - the stack pointer (SP) left the current stack frame, or
+       * - SP has the same value as when reaching the current function
+       *   and the address of this BB is the return address of last call
+       *   (we even allow to leave multiple frames if the SP stays the
+       *    same and we find a matching return address)
+       * The latter condition is needed because on PPC, SP can stay
+       * the same over CALL=b(c)l / RET=b(c)lr boundaries
+       */
+      if (sp < top_ce->sp) popcount_on_return = 0;
+      else if (top_ce->sp == sp) {
+	  while(1) {
+	      if (top_ce->ret_addr == bb_addr(bb)) break;
+	      if (csp_up>0) {
+		  csp_up--;
+		  top_ce = &(CLG_(current_call_stack).entry[csp_up]);
+		  if (top_ce->sp == sp) {
+		      popcount_on_return++;
+		      continue; 
+		  }
+	      }
+	      popcount_on_return = 0;
+	      break;
+	  }
+      }
+      if (popcount_on_return == 0) {
+	  jmpkind = Ijk_Boring;
+	  ret_without_call = True;
+      }
+  }
+
+  /* Should this jump be converted to call or pop/call ? */
+  if (( jmpkind != Ijk_Ret) &&
+      ( jmpkind != Ijk_Call) && last_bb) {
 
     /* We simulate a JMP/Cont to be a CALL if
      * - jump is in another ELF object or section kind
      * - jump is to first instruction of a function (tail recursion)
      */
-    if (bb->is_entry ||
+    if (ret_without_call ||
+	/* This is for detection of optimized tail recursion.
+	 * On PPC, this is only detected as call when going to another
+	 * function. The problem is that on PPC it can go wrong
+	 * more easily (no stack frame setup needed)
+	 */
+#if defined(VGA_ppc32)
+	(bb->is_entry && (last_bb->fn != bb->fn)) ||
+#else
+	bb->is_entry ||
+#endif
 	(last_bb->sect_kind != bb->sect_kind) ||
 	(last_bb->obj->number != bb->obj->number)) {
-      
-      SK_(current_state).jmpkind = JmpCall;
-      call_emulation = True;
-      
-      CT_DEBUG(1,"     JMP: %s to %s!\n",
-	       last_bb->obj->name, bb->obj->name);
+
+	CLG_DEBUG(1,"     JMP: %s[%s] to %s[%s]%s!\n",
+		  last_bb->fn->name, last_bb->obj->name,
+		  bb->fn->name, bb->obj->name,
+		  ret_without_call?" (RET w/o CALL)":"");
+
+	if (CLG_(get_fn_node)(last_bb)->pop_on_jump && (csp>0)) {
+
+	    call_entry* top_ce = &(CLG_(current_call_stack).entry[csp-1]);
+	    
+	    if (top_ce->jcc) {
+
+		CLG_DEBUG(1,"     Pop on Jump!\n");
+
+		/* change source for delayed push */
+		CLG_(current_state).bbcc = top_ce->jcc->from;
+		sp = top_ce->sp;
+		CLG_(pop_call_stack)();
+	    }
+	    else {
+		CLG_ASSERT(CLG_(current_state).nonskipped != 0);
+	    }
+	}
+
+	jmpkind = Ijk_Call;
+	call_emulation = True;
     }
   }
 
-  if (SK_(current_state).jmpkind == JmpCall)
-    skip = SK_(get_fn_node)(bb)->skip;
+  if (jmpkind == Ijk_Call)
+    skip = CLG_(get_fn_node)(bb)->skip;
 
-  CT_DEBUGIF(1) {
-    VG_(printf)("%s %08x -> %08x, ESP %08x\n",
-		(SK_(current_state).jmpkind == JmpNone)  ? "CONT" :
-		(SK_(current_state).jmpkind == JmpCond)  ? "JCND" :
-		(SK_(current_state).jmpkind == JmpRet)   ? "RET " :
-		(SK_(current_state).jmpkind == JmpCall)  ? "CALL" :
-		(SK_(current_state).jmpkind == JmpBoring)? "JMP " :
-		(SK_(current_state).jmpkind == JmpSyscall)? "SYSC" : "CREQ",
-		last_bb ? bb_jmpaddr(last_bb) : 0,
-		bb_addr(bb), esp);
+  CLG_DEBUGIF(1) {
+      if (jmpkind == JmpCond)
+	  VG_(printf)("Conditional");
+      else if (jmpkind == JmpNone)
+	  VG_(printf)("None");
+      else
+	  ppIRJumpKind( jmpkind );
+
+      VG_(printf)(" %08x -> %08x, SP %08x\n",
+		  last_bb ? bb_jmpaddr(last_bb) : 0,
+		  bb_addr(bb), sp);
   }
 
   /* Handle CALL/RET and update context to get correct BBCC */
   
-  if (SK_(current_state).jmpkind == JmpRet) {
+  if (jmpkind == Ijk_Ret) {
     
-    if ((SK_(current_call_stack).sp == 0) || 
-	((SK_(current_fn_stack).top > SK_(current_fn_stack).bottom) &&
-	 ( *(SK_(current_fn_stack).top-1)==0)) ) {
+    if ((csp == 0) || 
+	((CLG_(current_fn_stack).top > CLG_(current_fn_stack).bottom) &&
+	 ( *(CLG_(current_fn_stack).top-1)==0)) ) {
 
       /* On an empty call stack or at a signal separation marker,
        * a RETURN generates an call stack underflow.
        */	
       handleUnderflow(bb);
-      SK_(pop_call_stack)();
+      CLG_(pop_call_stack)();
     }
     else {
-      if (SK_(unwind_call_stack)(esp)==0) {
-	
-	call_entry* call_entry_up = 
-	  &(SK_(current_call_stack).entry[SK_(current_call_stack).sp-1]);
-	
-	/* Return without Call, i.e. a Jump.
-	 * If this is a jump into another function,
-	 * pretend a Return and a Call happening.
-	 * TODO: Check for jump to same function
-	 */
-	CT_DEBUG(1,"     RET without call!\n");
-	
-	/* change source for delayed push */
-	if (call_entry_up->jcc) {
-	  SK_(current_state).bbcc = call_entry_up->jcc->from;
-	  esp = call_entry_up->esp;
-	  SK_(pop_call_stack)();
-	}
-	else {
-	  CT_ASSERT(SK_(current_state).nonskipped != 0);
-	}
-	
-	skip = SK_(get_fn_node)(bb)->skip;
-	delayed_push = True;
-	
-      }
+	CLG_ASSERT(popcount_on_return >0);
+	CLG_(unwind_call_stack)(sp, popcount_on_return);
     }
   }
   else {
-    SK_(unwind_call_stack)(esp);
+    CLG_(unwind_call_stack)(sp, 0);
     
-    if (SK_(current_state).jmpkind == JmpCall) {
+    if (jmpkind == Ijk_Call) {
       delayed_push = True;
 
-      if (call_emulation && SK_(current_call_stack).sp>0) {
-	call_entry* call_entry_up = 
-	  &(SK_(current_call_stack).entry[SK_(current_call_stack).sp-1]);
-	
-	esp = call_entry_up->esp;
-      }
+      csp = CLG_(current_call_stack).sp;
+      if (call_emulation && csp>0)
+	sp = CLG_(current_call_stack).entry[csp-1].sp;	
+
     }
   }
   
   /* Change new context if needed, taking delayed_push into account */
-  if ((delayed_push && !skip) || (SK_(current_state).cxt == 0)) {
-    SK_(push_cxt)(SK_(get_fn_node)(bb));
+  if ((delayed_push && !skip) || (CLG_(current_state).cxt == 0)) {
+    CLG_(push_cxt)(CLG_(get_fn_node)(bb));
   }
-  CT_ASSERT(SK_(current_fn_stack).top > SK_(current_fn_stack).bottom);
+  CLG_ASSERT(CLG_(current_fn_stack).top > CLG_(current_fn_stack).bottom);
   
   /* If there is a fresh instrumented BBCC, assign current context */
-  bbcc = SK_(get_bbcc)(bb);
+  bbcc = CLG_(get_bbcc)(bb);
   if (bbcc->cxt == 0) {
-    CT_ASSERT(bbcc->rec_array == 0);
+    CLG_ASSERT(bbcc->rec_array == 0);
       
-    bbcc->cxt = SK_(current_state).cxt;
+    bbcc->cxt = CLG_(current_state).cxt;
     bbcc->rec_array = 
-      new_recursion((*SK_(current_fn_stack).top)->separate_recursions);
+      new_recursion((*CLG_(current_fn_stack).top)->separate_recursions);
     bbcc->rec_array[0] = bbcc;
       
     insert_bbcc_into_hash(bbcc);
@@ -677,16 +783,16 @@ void SK_(setup_bbcc)(BB* bb)
       bbcc = last_bbcc->lru_next_bbcc;
       if (bbcc &&
 	  ((bbcc->bb != bb) ||
-	   (bbcc->cxt != SK_(current_state).cxt)))
+	   (bbcc->cxt != CLG_(current_state).cxt)))
 	bbcc = 0;
     }
     else
       bbcc = 0;
 
     if (!bbcc)
-      bbcc = lookup_bbcc(bb, SK_(current_state).cxt);
+      bbcc = lookup_bbcc(bb, CLG_(current_state).cxt);
     if (!bbcc)
-      bbcc = clone_bbcc(bb->bbcc_list, SK_(current_state).cxt, 0);
+      bbcc = clone_bbcc(bb->bbcc_list, CLG_(current_state).cxt, 0);
     
     bb->last_bbcc = bbcc;
   }
@@ -695,18 +801,18 @@ void SK_(setup_bbcc)(BB* bb)
   if (last_bbcc)
     last_bbcc->lru_next_bbcc = bbcc;
 
-  if ((*SK_(current_fn_stack).top)->separate_recursions >1) {
+  if ((*CLG_(current_fn_stack).top)->separate_recursions >1) {
     UInt level, idx;
-    fn_node* top = *(SK_(current_fn_stack).top);
+    fn_node* top = *(CLG_(current_fn_stack).top);
 
-    level = *SK_(get_fn_entry)(top->number);
+    level = *CLG_(get_fn_entry)(top->number);
 
     if (delayed_push && !skip) {
-      if (SK_(clo).skip_direct_recursion) {
+      if (CLG_(clo).skip_direct_recursion) {
 	/* do not increment rec. level if called from
 	 * same function */
-	if (!SK_(current_state).bbcc || 
-	    (SK_(current_state).bbcc->cxt->fn[0] != bbcc->cxt->fn[0]))
+	if (!CLG_(current_state).bbcc || 
+	    (CLG_(current_state).bbcc->cxt->fn[0] != bbcc->cxt->fn[0]))
 	  level++;
       }
       else level++;
@@ -717,70 +823,68 @@ void SK_(setup_bbcc)(BB* bb)
     if (level == 0) {
       /* can only happen if instrumentation just was switched on */
       level = 1;
-      *SK_(get_fn_entry)(top->number) = 1;
+      *CLG_(get_fn_entry)(top->number) = 1;
     }
 
     idx = level -1;
     if (bbcc->rec_array[idx])
       bbcc = bbcc->rec_array[idx];
     else
-      bbcc = clone_bbcc(bbcc, SK_(current_state).cxt, idx);
+      bbcc = clone_bbcc(bbcc, CLG_(current_state).cxt, idx);
 
-    CT_ASSERT(bbcc->rec_array[bbcc->rec_index] == bbcc);
+    CLG_ASSERT(bbcc->rec_array[bbcc->rec_index] == bbcc);
   }
 
   if (delayed_push) {
-    if (!skip && SK_(current_state).nonskipped) {
+    if (!skip && CLG_(current_state).nonskipped) {
       /* a call from skipped to nonskipped */
-      SK_(current_state).bbcc = SK_(current_state).nonskipped;
+      CLG_(current_state).bbcc = CLG_(current_state).nonskipped;
     }
-    SK_(push_call_stack)(SK_(current_state).bbcc,
-			 bbcc, esp, skip);
+    CLG_(push_call_stack)(CLG_(current_state).bbcc, passed,
+			 bbcc, sp, skip);
   }
-  
-  if (SK_(current_state).jmpkind == JmpCond ||
-      SK_(current_state).jmpkind == JmpBoring) {
+
+  if (CLG_(clo).collect_jumps &&
+      ((jmpkind == JmpCond) || (jmpkind == Ijk_Boring))) {
     
     /* Handle conditional jumps followed, i.e. trace arcs
      * This uses JCC structures, too */
     
-    jCC* jcc = SK_(get_jcc)(last_bbcc, bbcc);
-    CT_ASSERT(jcc != 0);
+    jCC* jcc = CLG_(get_jcc)(last_bbcc, passed, bbcc);
+    CLG_ASSERT(jcc != 0);
     // Change from default, and check if already changed
-    if (jcc->jmpkind == JmpCall)
-      jcc->jmpkind = SK_(current_state).jmpkind;
-    else
-      CT_ASSERT(jcc->jmpkind == SK_(current_state).jmpkind);
+    if (jcc->jmpkind == Ijk_Call)
+      jcc->jmpkind = jmpkind;
+    else {
+	// FIXME: Why can this fail?
+	// CLG_ASSERT(jcc->jmpkind == jmpkind);
+    }
     
     jcc->call_counter++;
-    if (SK_(current_state).jmpkind == JmpCond)
-      SK_(stat).jcnd_counter++;
+    if (jmpkind == JmpCond)
+      CLG_(stat).jcnd_counter++;
     else
-      SK_(stat).jump_counter++;
+      CLG_(stat).jump_counter++;
   }
   
-  SK_(current_state).jmpkind = JmpNone;
-  SK_(current_state).bbcc = bbcc;
+  CLG_(current_state).bbcc = bbcc;
   
-  CT_DEBUGIF(1) {
+  CLG_DEBUGIF(1) {
     VG_(printf)("     ");
-    SK_(print_bbcc_fn)(bbcc);
+    CLG_(print_bbcc_fn)(bbcc);
     VG_(printf)("\n");
   }
   
-  CT_DEBUG(3,"- setup_bbcc (BB %p): Cost %p (Len %d), Instrs %d (Len %d)\n",
+  CLG_DEBUG(3,"- setup_bbcc (BB %p): Cost %p (Len %d), Instrs %d (Len %d)\n",
 	   bb_addr(bb), bbcc->cost, bb->cost_count, 
 	   bb->instr_count, bb->instr_len);
-  CT_DEBUGIF(3)
-    SK_(print_cxt)(-8, SK_(current_state).cxt, bbcc->rec_index);
-  CT_DEBUG(3,"\n");
+  CLG_DEBUGIF(3)
+    CLG_(print_cxt)(-8, CLG_(current_state).cxt, bbcc->rec_index);
+  CLG_DEBUG(3,"\n");
   
-  if (SK_(current_state).collect && !SK_(current_state).nonskipped)
-    bbcc->exe_counter++;
-    
-  (*SK_(cachesim).after_bbsetup)();
+  (*CLG_(cachesim).after_bbsetup)();
 
-  SK_(stat).bb_executions++;
+  CLG_(stat).bb_executions++;
 
   VGP_POPCC(VgpCacheSetup);
 }
