@@ -2,7 +2,7 @@
    This file is part of Callgrind, a Valgrind skin for call graph
    profiling programs.
 
-   Copyright (C) 2002-2004, Josef Weidendorfer (Josef.Weidendorfer@gmx.de)
+   Copyright (C) 2002-2005, Josef Weidendorfer (Josef.Weidendorfer@gmx.de)
 
    This skin is derived from and contains lot of code from Cachegrind
    Copyright (C) 2002 Nicholas Nethercote (njn25@cam.ac.uk)
@@ -25,6 +25,8 @@
    The GNU General Public License is contained in the file COPYING.
 */
 
+#include "config.h" // for VG_PREFIX
+
 #include "global.h"
 
 
@@ -49,11 +51,12 @@ struct _fn_config {
 
     Int skip;    /* Handle CALL to this function as JMP (= Skip)? */
     Int group;   /* don't change caller dependency inside group !=0 */
+    Int pop_on_jump; 
 
     Int separate_callers;    /* separate logging dependent on caller  */
     Int separate_recursions; /* separate logging of rec. levels       */
 
-#if CT_ENABLE_DEBUG
+#if CLG_ENABLE_DEBUG
     Int verbosity; /* Change debug verbosity level while in function */
 #endif
 };
@@ -95,18 +98,19 @@ static config_node* fn_configs = 0;
 static __inline__ 
 fn_config* new_fnc()
 {
-   fn_config* new = VG_(malloc)(sizeof(fn_config));
+   fn_config* new = (fn_config*) CLG_MALLOC(sizeof(fn_config));
 
    new->dump_before  = CONFIG_DEFAULT;
    new->dump_after   = CONFIG_DEFAULT;
    new->zero_before  = CONFIG_DEFAULT;
    new->toggle_collect = CONFIG_DEFAULT;
    new->skip         = CONFIG_DEFAULT;
+   new->pop_on_jump  = CONFIG_DEFAULT;
    new->group        = CONFIG_DEFAULT;
    new->separate_callers    = CONFIG_DEFAULT;
    new->separate_recursions = CONFIG_DEFAULT;
 
-#if CT_ENABLE_DEBUG
+#if CLG_ENABLE_DEBUG
    new->verbosity    = CONFIG_DEFAULT;
 #endif
 
@@ -117,7 +121,7 @@ fn_config* new_fnc()
 static config_node* new_config(Char* name, int length)
 {
     int i;
-    config_node* node = VG_(malloc)(sizeof(config_node) + length);
+    config_node* node = (config_node*) CLG_MALLOC(sizeof(config_node) + length);
 
     for(i=0;i<length;i++) {
       if (name[i] == 0) break;
@@ -133,7 +137,7 @@ static config_node* new_config(Char* name, int length)
     node->wild_char = 0;
     node->wild_star = 0;
 
-    CT_DEBUG(3, "   new_config('%s', len %d)\n", node->name, length);
+    CLG_DEBUG(3, "   new_config('%s', len %d)\n", node->name, length);
 
     return node;
 }
@@ -157,7 +161,7 @@ static fn_config* get_fnc2(config_node* node, Char* name)
   config_node *new_sub, *n, *nprev;
   int offset, len;
 
-  CT_DEBUG(3, "  get_fnc2(%p, '%s')\n", node, name);
+  CLG_DEBUG(3, "  get_fnc2(%p, '%s')\n", node, name);
 
   if (name[0] == 0) {
     if (!node->config) node->config = new_fnc();
@@ -292,13 +296,13 @@ static fn_config* get_fnc(Char* name)
 {
   fn_config* fnc;
 
-  CT_DEBUG(3, " +get_fnc(%s)\n", name);
+  CLG_DEBUG(3, " +get_fnc(%s)\n", name);
   if (fn_configs == 0)
     fn_configs = new_config(name, 0);
   fnc =  get_fnc2(fn_configs, name);
 
-  CT_DEBUGIF(3) {
-    CT_DEBUG(3, " -get_fnc(%s):\n", name);
+  CLG_DEBUGIF(3) {
+    CLG_DEBUG(3, " -get_fnc(%s):\n", name);
     print_config_node(3, fn_configs);
   }
   return fnc;
@@ -323,6 +327,9 @@ static void update_fn_config1(fn_node* fn, fn_config* fnc)
     if (fnc->skip != CONFIG_DEFAULT)
 	fn->skip = (fnc->skip == CONFIG_TRUE);
 
+    if (fnc->pop_on_jump != CONFIG_DEFAULT)
+	fn->pop_on_jump = (fnc->pop_on_jump == CONFIG_TRUE);
+
     if (fnc->group != CONFIG_DEFAULT)
 	fn->group = fnc->group;
 
@@ -332,7 +339,7 @@ static void update_fn_config1(fn_node* fn, fn_config* fnc)
     if (fnc->separate_recursions != CONFIG_DEFAULT)
 	fn->separate_recursions = fnc->separate_recursions;
 
-#if CT_ENABLE_DEBUG
+#if CLG_ENABLE_DEBUG
     if (fnc->verbosity != CONFIG_DEFAULT)
 	fn->verbosity = fnc->verbosity;
 #endif
@@ -342,14 +349,14 @@ static void update_fn_config1(fn_node* fn, fn_config* fnc)
  * looking for a match to <name>. For every matching leaf,
  * <fn> is updated with the pattern config.
  */
-static void update_fn_config2(fn_node* fn, char* name, config_node* node)
+static void update_fn_config2(fn_node* fn, Char* name, config_node* node)
 {
     config_node* n;
 
-    CT_DEBUG(3, "  update_fn_config2('%s', node '%s'): \n",
+    CLG_DEBUG(3, "  update_fn_config2('%s', node '%s'): \n",
 	     name, node->name);
     if ((*name == 0) && node->config) {
-      CT_DEBUG(3, "Found!\n");
+      CLG_DEBUG(3, "Found!\n");
       update_fn_config1(fn, node->config);
       return;
     }
@@ -374,9 +381,9 @@ static void update_fn_config2(fn_node* fn, char* name, config_node* node)
 }
 
 /* Update function config according to configs of name prefixes */
-void SK_(update_fn_config)(fn_node* fn)
+void CLG_(update_fn_config)(fn_node* fn)
 {
-    CT_DEBUG(3, "  update_fn_config('%s')\n", fn->name);
+    CLG_DEBUG(3, "  update_fn_config('%s')\n", fn->name);
     if (fn_configs)
       update_fn_config2(fn, fn->name, fn_configs);
 }
@@ -397,67 +404,103 @@ static Char* getUInt(Char* s, UInt* pn)
     return s;
 }
 
-Bool SK_(process_cmd_line_option)(Char* arg)
+__attribute__((unused))
+static UWord getUWord(Char* s)
+{
+    UWord n = 0;
+    Bool isHex = False;
+
+    if ((s[0] == '0') && (s[1] == 'x')) {
+	isHex = True;
+	s += 2;
+    }
+
+    if (!isHex) {
+	while((*s >='0') && (*s <='9')) {
+	    n = 10*n + (*s-'0');
+	    s++;
+	}
+    }
+    else {
+	while(1) {
+	    if ((*s >='0') && (*s <='9')) {
+		n = 16*n + (*s-'0');
+		s++;
+		continue;
+	    }
+	    if ((*s >='a') && (*s <='f')) {
+		n = 16*n + (*s-'a'+10);
+		s++;
+		continue;
+	    }
+	    if ((*s >='A') && (*s <='F')) {
+		n = 16*n + (*s-'A'+10);
+		s++;
+		continue;
+	    }
+	    break;
+	}
+    }
+
+    return n;
+}
+
+Bool CLG_(process_cmd_line_option)(Char* arg)
 {
    if (0 == VG_(strcmp)(arg, "--skip-plt=yes"))
-       SK_(clo).skip_plt = True;
+       CLG_(clo).skip_plt = True;
    else if (0 == VG_(strcmp)(arg, "--skip-plt=no"))
-       SK_(clo).skip_plt = False;
+       CLG_(clo).skip_plt = False;
 
    else if (0 == VG_(strcmp)(arg, "--collect-jumps=yes"))
-       SK_(clo).collect_jumps = True;
+       CLG_(clo).collect_jumps = True;
    else if (0 == VG_(strcmp)(arg, "--collect-jumps=no"))
-       SK_(clo).collect_jumps = False;
+       CLG_(clo).collect_jumps = False;
    /* compatibility alias, deprecated option */
    else if (0 == VG_(strcmp)(arg, "--trace-jump=yes"))
-       SK_(clo).collect_jumps = True;
+       CLG_(clo).collect_jumps = True;
    else if (0 == VG_(strcmp)(arg, "--trace-jump=no"))
-       SK_(clo).collect_jumps = False;
+       CLG_(clo).collect_jumps = False;
 
    else if (0 == VG_(strcmp)(arg, "--combine-dumps=yes"))
-       SK_(clo).combine_dumps = True;
+       CLG_(clo).combine_dumps = True;
    else if (0 == VG_(strcmp)(arg, "--combine-dumps=no"))
-       SK_(clo).combine_dumps = False;
+       CLG_(clo).combine_dumps = False;
 
    else if (0 == VG_(strcmp)(arg, "--collect-data=yes"))
-       SK_(clo).collect_data = True;
+       CLG_(clo).collect_data = True;
    else if (0 == VG_(strcmp)(arg, "--collect-data=no"))
-       SK_(clo).collect_data = False;
+       CLG_(clo).collect_data = False;
 
    else if (0 == VG_(strcmp)(arg, "--collect-atstart=yes"))
-       SK_(clo).collect_atstart = True;
+       CLG_(clo).collect_atstart = True;
    else if (0 == VG_(strcmp)(arg, "--collect-atstart=no"))
-       SK_(clo).collect_atstart = False;
+       CLG_(clo).collect_atstart = False;
 
    else if (0 == VG_(strcmp)(arg, "--instr-atstart=yes"))
-       SK_(clo).instrument_atstart = True;
+       CLG_(clo).instrument_atstart = True;
    else if (0 == VG_(strcmp)(arg, "--instr-atstart=no"))
-       SK_(clo).instrument_atstart = False;
+       CLG_(clo).instrument_atstart = False;
 
    else if (0 == VG_(strcmp)(arg, "--separate-threads=yes"))
-       SK_(clo).separate_threads = True;
+       CLG_(clo).separate_threads = True;
    else if (0 == VG_(strcmp)(arg, "--separate-threads=no"))
-       SK_(clo).separate_threads = False;
-   /* compatibility alias, deprecated option */
-   else if (0 == VG_(strcmp)(arg, "--dump-threads=yes"))
-       SK_(clo).separate_threads = True;
-   else if (0 == VG_(strcmp)(arg, "--dump-threads=no"))
-       SK_(clo).separate_threads = False;
+       CLG_(clo).separate_threads = False;
 
    else if (0 == VG_(strcmp)(arg, "--compress-strings=yes"))
-       SK_(clo).compress_strings = True;
+       CLG_(clo).compress_strings = True;
    else if (0 == VG_(strcmp)(arg, "--compress-strings=no"))
-       SK_(clo).compress_strings = False;
+       CLG_(clo).compress_strings = False;
 
    else if (0 == VG_(strcmp)(arg, "--compress-mangled=yes"))
-       SK_(clo).compress_mangled = True;
+       CLG_(clo).compress_mangled = True;
    else if (0 == VG_(strcmp)(arg, "--compress-mangled=no"))
-       SK_(clo).compress_mangled = False;
+       CLG_(clo).compress_mangled = False;
 
    else if (0 == VG_(strcmp)(arg, "--compress-pos=yes"))
-       SK_(clo).compress_pos = True;
+       CLG_(clo).compress_pos = True;
    else if (0 == VG_(strcmp)(arg, "--compress-pos=no"))
-       SK_(clo).compress_pos = False;
+       CLG_(clo).compress_pos = False;
 
    else if (0 == VG_(strncmp)(arg, "--fn-skip=", 10)) {
        fn_config* fnc = get_fnc(arg+10);
@@ -483,18 +526,24 @@ Bool SK_(process_cmd_line_option)(Char* arg)
        fn_config* fnc = get_fnc(arg+17);
        fnc->toggle_collect = CONFIG_TRUE;
        /* defaults to initial collection off */
-       SK_(clo).collect_atstart = False;
+       CLG_(clo).collect_atstart = False;
    }
 
    else if (0 == VG_(strncmp)(arg, "--separate-recs=", 16))
-        SK_(clo).separate_recursions = (Int)VG_(atoll)(&arg[16]);
+        CLG_(clo).separate_recursions = (Int)VG_(atoll)(&arg[16]);
 
-#if CT_ENABLE_DEBUG
+   /* workaround to find runtime_resolve (needs special handling) */
+   else if (0 == VG_(strncmp)(arg, "--pop-on-jump=", 14)) {
+       fn_config* fnc = get_fnc(arg+14);
+       fnc->pop_on_jump = CONFIG_TRUE;
+   }
+
+#if CLG_ENABLE_DEBUG
    else if (0 == VG_(strncmp)(arg, "--ct-verbose=", 13))
-        SK_(clo).verbose = (Int)VG_(atoll)(&arg[13]);
+        CLG_(clo).verbose = (Int)VG_(atoll)(&arg[13]);
 
    else if (0 == VG_(strncmp)(arg, "--ct-vstart=", 12))
-        SK_(clo).verbose_start = (ULong)VG_(atoll)(&arg[12]);
+        CLG_(clo).verbose_start = (ULong)VG_(atoll)(&arg[12]);
 
    else if (0 == VG_(strncmp)(arg, "--ct-verbose", 12)) {
        UInt n;
@@ -508,9 +557,9 @@ Bool SK_(process_cmd_line_option)(Char* arg)
 
    else if (0 == VG_(strncmp)(arg, "--separate-callers=", 19)) {
      if (0 == VG_(strcmp)(arg+19, "auto"))
-       SK_(clo).separate_callers = CONFIG_AUTO;
+       CLG_(clo).separate_callers = CONFIG_AUTO;
      else
-       SK_(clo).separate_callers = (Int)VG_(atoll)(&arg[19]);
+       CLG_(clo).separate_callers = (Int)VG_(atoll)(&arg[19]);
    }
 
    else if (0 == VG_(strncmp)(arg, "--fn-group", 10)) {
@@ -541,67 +590,79 @@ Bool SK_(process_cmd_line_option)(Char* arg)
    }
 
    else if (0 == VG_(strncmp)(arg, "--base=", 7))
-       SK_(clo).filename_base = VG_(strdup)(arg+7);
+       CLG_(clo).filename_base = VG_(strdup)(arg+7);
 
    else if (0 == VG_(strcmp)(arg, "--mangle-names=yes"))
-       SK_(clo).mangle_names = True;
+       CLG_(clo).mangle_names = True;
    else if (0 == VG_(strcmp)(arg, "--mangle-names=no"))
-       SK_(clo).mangle_names = False;
+       CLG_(clo).mangle_names = False;
 
    else if (0 == VG_(strcmp)(arg, "--skip-direct-rec=yes"))
-       SK_(clo).skip_direct_recursion = True;
+       CLG_(clo).skip_direct_recursion = True;
    else if (0 == VG_(strcmp)(arg, "--skip-direct-rec=no"))
-       SK_(clo).skip_direct_recursion = False;
+       CLG_(clo).skip_direct_recursion = False;
 
    else if (0 == VG_(strcmp)(arg, "--dump-bbs=yes"))
-       SK_(clo).dump_bbs = True;
+       CLG_(clo).dump_bbs = True;
    else if (0 == VG_(strcmp)(arg, "--dump-bbs=no"))
-       SK_(clo).dump_bbs = False;
+       CLG_(clo).dump_bbs = False;
 
    else if (0 == VG_(strcmp)(arg, "--dump-line=yes"))
-       SK_(clo).dump_line = True;
+       CLG_(clo).dump_line = True;
    else if (0 == VG_(strcmp)(arg, "--dump-line=no"))
-       SK_(clo).dump_line = False;
+       CLG_(clo).dump_line = False;
 
    else if (0 == VG_(strcmp)(arg, "--dump-instr=yes"))
-       SK_(clo).dump_instr = True;
+       CLG_(clo).dump_instr = True;
    else if (0 == VG_(strcmp)(arg, "--dump-instr=no"))
-       SK_(clo).dump_instr = False;
+       CLG_(clo).dump_instr = False;
 
    else if (0 == VG_(strcmp)(arg, "--dump-bb=yes"))
-       SK_(clo).dump_bb = True;
+       CLG_(clo).dump_bb = True;
    else if (0 == VG_(strcmp)(arg, "--dump-bb=no"))
-       SK_(clo).dump_bb = False;
+       CLG_(clo).dump_bb = False;
 
    else if (0 == VG_(strcmp)(arg, "--dump-objs=yes")) {
-       SK_(clo).dump_objs = True;
-       SK_(clo).dump_line = True;
-       SK_(clo).dump_instr = True;
+       CLG_(clo).dump_objs = True;
+       CLG_(clo).dump_line = True;
+       CLG_(clo).dump_instr = True;
    }
    else if (0 == VG_(strcmp)(arg, "--dump-objs=no"))
-       SK_(clo).dump_objs = False;
+       CLG_(clo).dump_objs = False;
 
    else if (0 == VG_(strncmp)(arg, "--dump-every-bb=", 16))
-        SK_(clo).dump_every_bb = (Int)VG_(atoll)(&arg[16]);
+        CLG_(clo).dump_every_bb = (Int)VG_(atoll)(&arg[16]);
 
 
    else if (0 == VG_(strcmp)(arg, "--collect-alloc=yes"))
-       SK_(clo).collect_alloc = True;
+       CLG_(clo).collect_alloc = True;
    else if (0 == VG_(strcmp)(arg, "--collect-alloc=no"))
-       SK_(clo).collect_alloc = False;
+       CLG_(clo).collect_alloc = False;
 
    else if (0 == VG_(strcmp)(arg, "--collect-systime=yes"))
-       SK_(clo).collect_systime = True;
+       CLG_(clo).collect_systime = True;
    else if (0 == VG_(strcmp)(arg, "--collect-systime=no"))
-       SK_(clo).collect_systime = False;
+       CLG_(clo).collect_systime = False;
 
-   else
-     return (*SK_(cachesim).parse_opt)(arg);
+   else if (0 == VG_(strcmp)(arg, "--simulate-cache=yes"))
+       CLG_(clo).simulate_cache = True;
+   else if (0 == VG_(strcmp)(arg, "--simulate-cache=no"))
+       CLG_(clo).simulate_cache = False;
+
+   else {
+       Bool isCachesimOption = (*CLG_(cachesim).parse_opt)(arg);
+
+       /* cache simulator is used if a simulator option is given */
+       if (isCachesimOption)
+	   CLG_(clo).simulate_cache = True;
+
+       return isCachesimOption;
+   }
 
    return True;
 }
 
-void SK_(print_usage)(void)
+void CLG_(print_usage)(void)
 {
    VG_(printf)(
 "\n   dump creation options:\n"
@@ -611,7 +672,7 @@ void SK_(print_usage)(void)
 "    --compress-strings=no|yes Compress strings in profile dump? [yes]\n"
 "    --compress-pos=no|yes     Compress positions in profile dump? [yes]\n"
 "    --combine-dumps=no|yes    Concat all dumps into same file [no]\n"
-#if CT_EXPERIMENTAL
+#if CLG_EXPERIMENTAL
 "    --compress-events=no|yes  Compress events in profile dump? [no]\n"
 "    --dump-bb=no|yes          Dump basic block address of costs? [no]\n"
 "    --dump-bbs=no|yes         Dump basic block info? [no]\n"
@@ -624,16 +685,20 @@ void SK_(print_usage)(void)
 "    --dump-before=<func>      Dump when entering function\n"
 "    --zero-before=<func>      Zero all costs when entering function\n"
 "    --dump-after=<func>       Dump when leaving function\n"
+#if CLG_EXPERIMENTAL
 "    --dump-objs=no|yes        Dump static object information [no]\n"
+#endif
 
 "\n   data collection options:\n"
 "    --instr-atstart=no|yes    Do instrumentation at callgrind start [yes]\n"
 "    --collect-atstart=no|yes  Collect at process/thread start [yes]\n"
 "    --toggle-collect=<func>   Toggle collection on enter/leave function\n"
 "    --collect-jumps=no|yes    Collect jumps? [no]\n"
+#if CLG_EXPERIMENTAL
 "    --collect-alloc=no|yes    Collect memory allocation info? [no]\n"
+#endif
 "    --collect-systime=no|yes  Collect system call time info? [no]\n"
-#if CT_DATA
+#if CLG_DATA
 "    --collect-data=no|yes     Collect data relation of events [no]\n"
 #endif
 
@@ -646,23 +711,23 @@ void SK_(print_usage)(void)
 "    --separate-callers<n>=<f> Separate <n> callers for function <f>\n"
 "    --skip-direct-rec=no|yes  Ignore direct recursions? [yes]\n"
 "    --fn-skip=<function>      Ignore calls to/from function?\n"
-#if CT_EXPERIMENTAL
+#if CLG_EXPERIMENTAL
 "    --fn-group<no>=<func>     Put function into separation group <no>\n"
 #endif
     );
 
-   (*SK_(cachesim).print_opts)();
+   (*CLG_(cachesim).print_opts)();
 
    VG_(printf)("\n"
 	       "  For full callgrind documentation, see\n"
-	       "  "VG_LIBDIR"/../share/doc/valgrind/ct_main.html.\n\n");
+	       "  "VG_PREFIX"/share/doc/callgrind/html/callgrind.html\n\n");
 }
 
-void SK_(print_debug_usage)(void)
+void CLG_(print_debug_usage)(void)
 {
     VG_(printf)(
 
-#if CT_ENABLE_DEBUG
+#if CLG_ENABLE_DEBUG
 "    --ct-verbose=<level>       Verbosity of standard debug output [0]\n"
 "    --ct-vstart=<BB number>    Only be verbose after basic block [0]\n"
 "    --ct-verbose<level>=<func> Verbosity while in <func>\n"
@@ -674,42 +739,44 @@ void SK_(print_debug_usage)(void)
 }
 
 
-void SK_(set_clo_defaults)(void)
+void CLG_(set_clo_defaults)(void)
 {
   /* Default values for command line arguments */
 
   /* dump options */
-  SK_(clo).filename_base    = 0;
-  SK_(clo).combine_dumps    = False;
-  SK_(clo).compress_strings = True;
-  SK_(clo).compress_mangled = False;
-  SK_(clo).compress_events  = False;
-  SK_(clo).compress_pos     = True;
-  SK_(clo).mangle_names     = True;
-  SK_(clo).dump_line        = True;
-  SK_(clo).dump_instr       = False;
-  SK_(clo).dump_bb          = False;
-  SK_(clo).dump_bbs         = False;
+  CLG_(clo).filename_base    = 0;
+  CLG_(clo).combine_dumps    = False;
+  CLG_(clo).compress_strings = True;
+  CLG_(clo).compress_mangled = False;
+  CLG_(clo).compress_events  = False;
+  CLG_(clo).compress_pos     = True;
+  CLG_(clo).mangle_names     = True;
+  CLG_(clo).dump_line        = True;
+  CLG_(clo).dump_instr       = False;
+  CLG_(clo).dump_bb          = False;
+  CLG_(clo).dump_bbs         = False;
 
-  SK_(clo).dump_every_bb    = 0;
+  CLG_(clo).dump_every_bb    = 0;
 
   /* Collection */
-  SK_(clo).instrument_atstart = True;
-  SK_(clo).separate_threads = False;
-  SK_(clo).collect_data     = False;
-  SK_(clo).collect_atstart  = True;
-  SK_(clo).collect_jumps    = False;
-  SK_(clo).collect_alloc    = False;
-  SK_(clo).collect_systime  = False;
+  CLG_(clo).separate_threads = False;
+  CLG_(clo).collect_data     = False;
+  CLG_(clo).collect_atstart  = True;
+  CLG_(clo).collect_jumps    = False;
+  CLG_(clo).collect_alloc    = False;
+  CLG_(clo).collect_systime  = False;
 
-  SK_(clo).skip_plt         = True;
-  SK_(clo).separate_callers = 0;
-  SK_(clo).separate_recursions = 2;
-  SK_(clo).skip_direct_recursion = False;
+  CLG_(clo).skip_plt         = True;
+  CLG_(clo).separate_callers = 0;
+  CLG_(clo).separate_recursions = 2;
+  CLG_(clo).skip_direct_recursion = False;
 
+  /* Instrumentation */
+  CLG_(clo).instrument_atstart = True;
+  CLG_(clo).simulate_cache = False;
 
-#if CT_ENABLE_DEBUG
-  SK_(clo).verbose = 0;
-  SK_(clo).verbose_start = 0;
+#if CLG_ENABLE_DEBUG
+  CLG_(clo).verbose = 0;
+  CLG_(clo).verbose_start = 0;
 #endif
 }
