@@ -29,8 +29,11 @@
  * Functions related to interactive commands via "callgrind.cmd"
  */
 
+#include "config.h"
 #include "global.h"
 
+#include <pub_tool_threadstate.h> // VG_N_THREADS
+
 static Char outbuf[FILENAME_LEN + FN_NAME_LEN + OBJ_NAME_LEN];
 
 static Char* command_file = 0;
@@ -41,15 +44,16 @@ static Char* dump_base = 0;
 
 static Bool command_inited = False;
 
-void SK_(init_command)(Char* dir, Char* dumps)
+void CLG_(init_command)(Char* dir, Char* dumps)
 {
   Int fd, size;
+  SysRes res;
 
   dump_base = dumps;
 
   size = VG_(strlen)(dir) + VG_(strlen)(DEFAULT_COMMANDNAME) +10;
-  command_file = VG_(malloc)(size);
-  CT_ASSERT(command_file != 0);
+  command_file = (char*) CLG_MALLOC(size);
+  CLG_ASSERT(command_file != 0);
   VG_(sprintf)(command_file, "%s/%s.%d",
 	       dir, DEFAULT_COMMANDNAME, VG_(getpid)());
 
@@ -58,38 +62,40 @@ void SK_(init_command)(Char* dir, Char* dumps)
    * different callgrind instances from same base directory.
    * Should be removed sometimes in the future (29.10.03)
    */
-  command_file2 = VG_(malloc)(size);
-  CT_ASSERT(command_file2 != 0);
+  command_file2 = (char*) CLG_MALLOC(size);
+  CLG_ASSERT(command_file2 != 0);
   VG_(sprintf)(command_file2, "%s/%s",
 	       dir, DEFAULT_COMMANDNAME);
 
   size = VG_(strlen)(dir) + VG_(strlen)(DEFAULT_RESULTNAME) +10;
-  result_file = VG_(malloc)(size);
-  CT_ASSERT(result_file != 0);
+  result_file = (char*) CLG_MALLOC(size);
+  CLG_ASSERT(result_file != 0);
   VG_(sprintf)(result_file, "%s/%s.%d",
 	       dir, DEFAULT_RESULTNAME, VG_(getpid)());
 
-  info_file = VG_(malloc)(VG_(strlen)(DEFAULT_INFONAME) + 10);
-  CT_ASSERT(info_file != 0);
+  info_file = (char*) CLG_MALLOC(VG_(strlen)(DEFAULT_INFONAME) + 10);
+  CLG_ASSERT(info_file != 0);
   VG_(sprintf)(info_file, "%s.%d", DEFAULT_INFONAME, VG_(getpid)());
 
-  CT_DEBUG(1, "  dump file base: '%s'\n", dump_base);
-  CT_DEBUG(1, "  command file:   '%s'\n", command_file);
-  CT_DEBUG(1, "  result file:    '%s'\n", result_file);
-  CT_DEBUG(1, "  info file:      '%s'\n", info_file);
+  CLG_DEBUG(1, "  dump file base: '%s'\n", dump_base);
+  CLG_DEBUG(1, "  command file:   '%s'\n", command_file);
+  CLG_DEBUG(1, "  result file:    '%s'\n", result_file);
+  CLG_DEBUG(1, "  info file:      '%s'\n", info_file);
 
   /* create info file to indicate that we are running */ 
-  fd = VG_(open)(info_file, VKI_O_WRONLY|VKI_O_TRUNC, 0);
-  if (fd<0) { 
-    fd = VG_(open)(info_file, VKI_O_CREAT|VKI_O_WRONLY,
+  res = VG_(open)(info_file, VKI_O_WRONLY|VKI_O_TRUNC, 0);
+  if (res.isError) { 
+    res = VG_(open)(info_file, VKI_O_CREAT|VKI_O_WRONLY,
 		   VKI_S_IRUSR|VKI_S_IWUSR);
-    if (fd <0) {
+    if (res.isError) {
       VG_(message)(Vg_DebugMsg, 
 		   "warning: can't write info file '%s'", info_file);
       info_file = 0;
       fd = -1;
     }
   }
+  if (!res.isError)
+      fd = (Int) res.val;
   if (fd>=0) {
     Char buf[512];
     Int i;
@@ -98,7 +104,13 @@ void SK_(init_command)(Char* dir, Char* dumps)
 		 "# This file is generated by Callgrind-" VERSION ".\n"
 		 "# It is used to enable controlling the supervision of\n"
 		 "#  '%s'\n"
-		 "# by external tools.\n\n", VG_(client_argv[0]));
+		 "# by external tools.\n\n",
+#if VG_CORE_INTERFACE_VERSION < 9
+		 VG_(client_argv[0])
+#else
+		 VG_(args_the_exename)
+#endif
+	);
     VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
     
     VG_(sprintf)(buf, "version: " VERSION "\n");
@@ -118,10 +130,21 @@ void SK_(init_command)(Char* dir, Char* dumps)
     
     VG_(strcpy)(buf, "cmd:");
     VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
+#if VG_CORE_INTERFACE_VERSION < 9
     for (i = 0; i < VG_(client_argc); i++) {
-      VG_(sprintf)(buf, " %s", VG_(client_argv[i]));
-      VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
+	if (!VG_(client_argv[i])) continue;
+	VG_(sprintf)(buf, " %s", VG_(client_argv[i]));
+	VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
     }
+#else
+    VG_(sprintf)(buf, " %s", VG_(args_the_exename));
+    VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
+    for (i = 0; i < VG_(args_for_client).used; i++) {
+	if (!VG_(args_for_client).strs[i]) continue;
+	VG_(sprintf)(buf, " %s", VG_(args_for_client).strs[i]);
+	VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
+    }
+#endif
     VG_(write)(fd, "\n", 1);
     VG_(close)(fd);
   }
@@ -129,7 +152,7 @@ void SK_(init_command)(Char* dir, Char* dumps)
   command_inited = True;
 }
 
-void SK_(finish_command)()
+void CLG_(finish_command)()
 {
   /* unlink info file */
   if (info_file) VG_(unlink)(info_file);
@@ -138,17 +161,20 @@ void SK_(finish_command)()
 
 static Int createRes(Int fd)
 {
+    SysRes res;
+
     if (fd > -2) return fd;
 
     /* fd == -2: No error, but we need to create the file */
-    fd = VG_(open)(result_file,
+    res = VG_(open)(result_file,
 		   VKI_O_CREAT|VKI_O_WRONLY|VKI_O_TRUNC,
 		   VKI_S_IRUSR|VKI_S_IWUSR);
 
     /* VG_(open) can return any negative number on error. Remap errors to -1,
      * to not confuse it with our special value -2
      */
-    if (fd<0) fd = -1;
+    if (res.isError) fd = -1;
+    else fd = (Int) res.val;
 
     return fd;
 }
@@ -176,10 +202,21 @@ static Int dump_info(Int fd)
     /* "cmd:" line */
     VG_(strcpy)(buf, "cmd:");
     VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
+#if VG_CORE_INTERFACE_VERSION < 9
     for (i = 0; i < VG_(client_argc); i++) {
+	if (!VG_(client_argv[i])) continue;
 	VG_(sprintf)(buf, " %s", VG_(client_argv[i]));
 	VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
     }
+#else
+    VG_(sprintf)(buf, " %s", VG_(args_the_exename));
+    VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
+    for (i = 0; i < VG_(args_for_client).used; i++) {
+	if (!VG_(args_for_client).strs[i]) continue;
+	VG_(sprintf)(buf, " %s", VG_(args_for_client).strs[i]);
+	VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
+    }
+#endif
 
     return fd;
 }
@@ -192,29 +229,29 @@ Int dump_fd;
 void static dump_state_of_thread(thread_info* ti)
 {
     Char* buf = outbuf;
-    int t = SK_(current_tid);
+    int t = CLG_(current_tid);
     Int p, i;
     static FullCost sum = 0, tmp = 0;
     BBCC *from, *to;
     call_entry* ce;
 
     p = VG_(sprintf)(buf, "events-%d: ", t);
-    SK_(init_cost_lz)( SK_(sets).full, &sum );
-    SK_(copy_cost_lz)( SK_(sets).full, &tmp, ti->lastdump_cost );
-    SK_(add_diff_cost)( SK_(sets).full, sum,
+    CLG_(init_cost_lz)( CLG_(sets).full, &sum );
+    CLG_(copy_cost_lz)( CLG_(sets).full, &tmp, ti->lastdump_cost );
+    CLG_(add_diff_cost)( CLG_(sets).full, sum,
 			ti->lastdump_cost,
 			ti->states.entry[0]->cost);
-    SK_(copy_cost)( SK_(sets).full, ti->lastdump_cost, tmp );
-    p += SK_(sprint_mappingcost)(buf + p, SK_(dumpmap), sum);
+    CLG_(copy_cost)( CLG_(sets).full, ti->lastdump_cost, tmp );
+    p += CLG_(sprint_mappingcost)(buf + p, CLG_(dumpmap), sum);
     p += VG_(sprintf)(buf+p, "\n");
     VG_(write)(dump_fd, (void*)buf, p);
 
     p = VG_(sprintf)(buf, "frames-%d: %d\n", t,
-		     SK_(current_call_stack).sp);
+		     CLG_(current_call_stack).sp);
     VG_(write)(dump_fd, (void*)buf, p);
     ce = 0;
-    for(i = 0; i < SK_(current_call_stack).sp; i++) {
-      ce = SK_(get_call_entry)(i);
+    for(i = 0; i < CLG_(current_call_stack).sp; i++) {
+      ce = CLG_(get_call_entry)(i);
       /* if this frame is skipped, we don't have counters */
       if (!ce->jcc) continue;
       
@@ -228,14 +265,14 @@ void static dump_state_of_thread(thread_info* ti)
       VG_(write)(dump_fd, (void*)buf, p);
       
       /* FIXME: EventSets! */
-      SK_(copy_cost)( SK_(sets).full, sum, ce->jcc->cost );
-      SK_(copy_cost)( SK_(sets).full, tmp, ce->enter_cost );
-      SK_(add_diff_cost)( SK_(sets).full, sum,
-			  ce->enter_cost, SK_(current_state).cost );
-      SK_(copy_cost)( SK_(sets).full, ce->enter_cost, tmp );
+      CLG_(copy_cost)( CLG_(sets).full, sum, ce->jcc->cost );
+      CLG_(copy_cost)( CLG_(sets).full, tmp, ce->enter_cost );
+      CLG_(add_diff_cost)( CLG_(sets).full, sum,
+			  ce->enter_cost, CLG_(current_state).cost );
+      CLG_(copy_cost)( CLG_(sets).full, ce->enter_cost, tmp );
       
       p = VG_(sprintf)(buf, "events-%d-%d: ",t, i);
-      p += SK_(sprint_mappingcost)(buf + p, SK_(dumpmap), sum );
+      p += CLG_(sprint_mappingcost)(buf + p, CLG_(dumpmap), sum );
       p += VG_(sprintf)(buf+p, "\n");
       VG_(write)(dump_fd, (void*)buf, p);
     }
@@ -253,46 +290,46 @@ static Int dump_state(Int fd)
     Char* buf = outbuf;
     thread_info** th;
     int t, p;
-    Int orig_tid = SK_(current_tid);
+    Int orig_tid = CLG_(current_tid);
 
     if ( (fd = createRes(fd)) <0) return fd;
 
     VG_(sprintf)(buf, "instrumentation: %s\n",
-		 SK_(instrument_state) ? "on":"off");
+		 CLG_(instrument_state) ? "on":"off");
     VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
 
-    if (!SK_(instrument_state)) return fd;
+    if (!CLG_(instrument_state)) return fd;
 
-    VG_(sprintf)(buf, "executed-bbs: %llu\n", SK_(stat).bb_executions);
+    VG_(sprintf)(buf, "executed-bbs: %llu\n", CLG_(stat).bb_executions);
     VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
 
-    VG_(sprintf)(buf, "executed-calls: %llu\n", SK_(stat).call_counter);
+    VG_(sprintf)(buf, "executed-calls: %llu\n", CLG_(stat).call_counter);
     VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
 
-    VG_(sprintf)(buf, "distinct-bbs: %d\n", SK_(stat).distinct_bbs);
+    VG_(sprintf)(buf, "distinct-bbs: %d\n", CLG_(stat).distinct_bbs);
     VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
 
-    VG_(sprintf)(buf, "distinct-calls: %d\n", SK_(stat).distinct_jccs);
+    VG_(sprintf)(buf, "distinct-calls: %d\n", CLG_(stat).distinct_jccs);
     VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
 
-    VG_(sprintf)(buf, "distinct-functions: %d\n", SK_(stat).distinct_fns);
+    VG_(sprintf)(buf, "distinct-functions: %d\n", CLG_(stat).distinct_fns);
     VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
 
-    VG_(sprintf)(buf, "distinct-contexts: %d\n", SK_(stat).distinct_contexts);
+    VG_(sprintf)(buf, "distinct-contexts: %d\n", CLG_(stat).distinct_contexts);
     VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
 
     /* "events:" line. Given here because it will be dynamic in the future */
     p = VG_(sprintf)(buf, "events: ");
-    SK_(sprint_eventmapping)(buf+p, SK_(dumpmap));
+    CLG_(sprint_eventmapping)(buf+p, CLG_(dumpmap));
     VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
     VG_(write)(fd, "\n", 1);
 		
     /* "part:" line (number of last part. Is 0 at start */
-    VG_(sprintf)(buf, "\npart: %d\n", SK_(get_dump_counter)());
+    VG_(sprintf)(buf, "\npart: %d\n", CLG_(get_dump_counter)());
     VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
 		
     /* threads */
-    th = SK_(get_threads)();
+    th = CLG_(get_threads)();
     p = VG_(sprintf)(buf, "threads:");
     for(t=1;t<VG_N_THREADS;t++) {
 	if (!th[t]) continue;
@@ -306,12 +343,12 @@ static Int dump_state(Int fd)
 
     /* current event counters */
     dump_fd = fd;
-    SK_(forall_threads)(dump_state_of_thread);
+    CLG_(forall_threads)(dump_state_of_thread);
 
     return fd;
 }
 
-void SK_(check_command)()
+void CLG_(check_command)()
 {
     /* check for dumps needed */
     static Char buf[512];
@@ -319,15 +356,18 @@ void SK_(check_command)()
     Char *cmdPos = 0, *cmdNextLine = 0;
     Int fd, bytesRead = 0, do_kill = 0;
     static Char* cfile = 0;
+    SysRes res;
 
     if (!command_inited) return;
 
     /* toggle between 2 command files, with/without ".pid" postfix */
     cfile = ((cfile == command_file) || (cfile == 0)) ? 
       command_file2 : command_file;
-
-    fd = VG_(open)(cfile, VKI_O_RDONLY,0);
-    if (fd>=0) {
+    
+    
+    res = VG_(open)(cfile, VKI_O_RDONLY,0);
+    if (!res.isError) {
+	fd = (Int) res.val;
 	bytesRead = VG_(read)(fd,cmdBuffer,500);
 	cmdBuffer[500] = 0; /* no command overrun please */
 	VG_(close)(fd);
@@ -365,11 +405,11 @@ void SK_(check_command)()
 	  while((*cmdPos == ' ') || (*cmdPos == '\t')) cmdPos++;
 	  
 	  switch(*cmdPos) {
-#if CT_ENABLE_DEBUG
+#if CLG_ENABLE_DEBUG
 	    /* verbosity */
 	  case 'V':
 	  case 'v':
-	    SK_(clo).verbose = value;
+	    CLG_(clo).verbose = value;
 	    break;
 #endif
 	  default:
@@ -390,7 +430,7 @@ void SK_(check_command)()
 	  switch(*cmdPos) {
 	  case 'I':
 	  case 'i':
-	    SK_(set_instrument_state)("Command", value);
+	    CLG_(set_instrument_state)("Command", value);
 	    break;
 
 	  default:
@@ -413,12 +453,12 @@ void SK_(check_command)()
 	    VG_(sprintf)(buf, "Dump Command:%s", cmdPos);
 	  else
 	    VG_(sprintf)(buf, "Dump Command");
-	  SK_(dump_profile)(buf, False);
+	  CLG_(dump_profile)(buf, False);
 	  break;
 	    
 	case 'Z':
 	case 'z':
-	    SK_(zero_all_cost)(False);
+	    CLG_(zero_all_cost)(False);
 	    break;
 
 	case 'K':
@@ -443,16 +483,16 @@ void SK_(check_command)()
 	    if ( (fd = createRes(fd)) <0) break;
 
 	    VG_(sprintf)(buf, "\ndesc: Option: --skip-plt=%s\n",
-			 SK_(clo).skip_plt ? "yes" : "no");
+			 CLG_(clo).skip_plt ? "yes" : "no");
 	    VG_(write)(fd, (void*)buf, VG_(strlen)(buf));	    
 	    VG_(sprintf)(buf, "desc: Option: --collect-jumps=%s\n",
-			 SK_(clo).collect_jumps ? "yes" : "no");
+			 CLG_(clo).collect_jumps ? "yes" : "no");
 	    VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
 	    VG_(sprintf)(buf, "desc: Option: --separate-recs=%d\n",
-			 SK_(clo).separate_recursions);
+			 CLG_(clo).separate_recursions);
 	    VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
 	    VG_(sprintf)(buf, "desc: Option: --separate-callers=%d\n",
-			 SK_(clo).separate_callers);
+			 CLG_(clo).separate_callers);
 	    VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
 
 	    break;
@@ -471,7 +511,7 @@ void SK_(check_command)()
     if (do_kill) {
       VG_(message)(Vg_UserMsg,
 		   "Killed because of command from %s", cfile);
-      SK_(fini)(0);
+      CLG_(fini)(0);
       VG_(exit)(1);
     }
 }
