@@ -36,51 +36,51 @@
 #define N_FNSTACK_INITIAL_ENTRIES 500
 #define N_CXT_INITIAL_ENTRIES 2537
 
-fn_stack SK_(current_fn_stack);
+fn_stack CLG_(current_fn_stack);
 
-void SK_(init_fn_stack)(fn_stack* s)
+void CLG_(init_fn_stack)(fn_stack* s)
 {
-  CT_ASSERT(s != 0);
+  CLG_ASSERT(s != 0);
 
   s->size   = N_FNSTACK_INITIAL_ENTRIES;   
-  s->bottom = (fn_node**) VG_(malloc)(s->size * sizeof(fn_node*));
+  s->bottom = (fn_node**) CLG_MALLOC(s->size * sizeof(fn_node*));
   s->top    = s->bottom;
   s->bottom[0] = 0;
 }
 
-void SK_(copy_current_fn_stack)(fn_stack* dst)
+void CLG_(copy_current_fn_stack)(fn_stack* dst)
 {
-  CT_ASSERT(dst != 0);
+  CLG_ASSERT(dst != 0);
 
-  dst->size   = SK_(current_fn_stack).size;
-  dst->bottom = SK_(current_fn_stack).bottom;
-  dst->top    = SK_(current_fn_stack).top;
+  dst->size   = CLG_(current_fn_stack).size;
+  dst->bottom = CLG_(current_fn_stack).bottom;
+  dst->top    = CLG_(current_fn_stack).top;
 }
 
-void SK_(set_current_fn_stack)(fn_stack* s)
+void CLG_(set_current_fn_stack)(fn_stack* s)
 {
-  CT_ASSERT(s != 0);
+  CLG_ASSERT(s != 0);
 
-  SK_(current_fn_stack).size   = s->size;
-  SK_(current_fn_stack).bottom = s->bottom;
-  SK_(current_fn_stack).top    = s->top;
+  CLG_(current_fn_stack).size   = s->size;
+  CLG_(current_fn_stack).bottom = s->bottom;
+  CLG_(current_fn_stack).top    = s->top;
 }
 
 static cxt_hash cxts;
 
-void SK_(init_cxt_table)()
+void CLG_(init_cxt_table)()
 {
    Int i;
    
    cxts.size    = N_CXT_INITIAL_ENTRIES;
    cxts.entries = 0;
-   cxts.table   = (Context**) VG_(malloc)(cxts.size * sizeof(Context*));
+   cxts.table   = (Context**) CLG_MALLOC(cxts.size * sizeof(Context*));
 
    for (i = 0; i < cxts.size; i++)
      cxts.table[i] = 0;
 }
 
-cxt_hash* SK_(get_cxt_hash)()
+cxt_hash* CLG_(get_cxt_hash)()
 {
   return &cxts;
 }
@@ -88,12 +88,12 @@ cxt_hash* SK_(get_cxt_hash)()
 /* double size of cxt table  */
 static void resize_cxt_table()
 {
-    Int i, new_size, conflicts1 = 0, conflicts2 = 0;
+    UInt i, new_size, conflicts1 = 0, conflicts2 = 0;
     Context **new_table, *curr, *next;
-    UInt new_hash;
+    UInt new_idx;
 
     new_size  = 2* cxts.size +3;
-    new_table = (Context**) VG_(malloc)(new_size * sizeof(Context*));
+    new_table = (Context**) CLG_MALLOC(new_size * sizeof(Context*));
 
     if (!new_table) return;
 
@@ -107,10 +107,10 @@ static void resize_cxt_table()
         while (NULL != curr) {
             next = curr->next;
 
-            new_hash = curr->hash % new_size;
+            new_idx = (UInt) (curr->hash % new_size);
 
-            curr->next = new_table[new_hash];
-            new_table[new_hash] = curr;
+            curr->next = new_table[new_idx];
+            new_table[new_idx] = curr;
             if (curr->next) {
                 conflicts1++;
                 if (curr->next->next)
@@ -124,22 +124,22 @@ static void resize_cxt_table()
     VG_(free)(cxts.table);
 
 
-    CT_DEBUG(0, "Resize Context Hash: %d => %d (entries %d, conflicts %d/%d)\n",
+    CLG_DEBUG(0, "Resize Context Hash: %d => %d (entries %d, conflicts %d/%d)\n",
              cxts.size, new_size,
              cxts.entries, conflicts1, conflicts2);
 
     cxts.size  = new_size;
     cxts.table = new_table;
-    SK_(stat).cxt_hash_resizes++;
+    CLG_(stat).cxt_hash_resizes++;
 }
 
 __inline__
-static UInt cxt_hash_val(fn_node** fn, UInt size)
+static UWord cxt_hash_val(fn_node** fn, UInt size)
 {
-    UInt hash = 0;
+    UWord hash = 0;
     UInt count = size;
     while(*fn != 0) {
-        hash = (hash<<7) + (hash>>25) + (UInt)(*fn);
+        hash = (hash<<7) + (hash>>25) + (UWord)(*fn);
         fn--;
         count--;
         if (count==0) break;
@@ -148,7 +148,7 @@ static UInt cxt_hash_val(fn_node** fn, UInt size)
 }
 
 __inline__
-static Bool is_cxt(UInt hash, fn_node** fn, Context* cxt)
+static Bool is_cxt(UWord hash, fn_node** fn, Context* cxt)
 {
     int count;
     fn_node** cxt_fn;
@@ -172,11 +172,12 @@ static Bool is_cxt(UInt hash, fn_node** fn, Context* cxt)
 static Context* new_cxt(fn_node** fn)
 {
     Context* new;
-    UInt idx, hash, offset;
+    UInt idx, offset;
+    UWord hash;
     int size, recs;
     fn_node* top_fn;
 
-    CT_ASSERT(fn);
+    CLG_ASSERT(fn);
     top_fn = *fn;
     if (top_fn == 0) return 0;
 
@@ -189,12 +190,14 @@ static Context* new_cxt(fn_node** fn)
     if (10 * cxts.entries / cxts.size > 8)
         resize_cxt_table();
 
-    new = VG_(malloc)(sizeof(Context)+sizeof(fn_node*)*size);
+    new = (Context*) CLG_MALLOC(sizeof(Context)+sizeof(fn_node*)*size);
 
+    // hash value calculation similar to cxt_hash_val(), but additionally
+    // copying function pointers in one run
     hash = 0;
     offset = 0;
     while(*fn != 0) {
-        hash = (hash<<7) + (hash>>25) + (UInt)(*fn);
+        hash = (hash<<7) + (hash>>25) + (UWord)(*fn);
         new->fn[offset] = *fn;
         offset++;
         fn--;
@@ -203,21 +206,21 @@ static Context* new_cxt(fn_node** fn)
     if (offset < size) size = offset;
 
     new->size        = size;
-    new->base_number = SK_(stat).context_counter;
+    new->base_number = CLG_(stat).context_counter;
     new->hash        = hash;
 
-    SK_(stat).context_counter += recs;
-    SK_(stat).distinct_contexts++;
+    CLG_(stat).context_counter += recs;
+    CLG_(stat).distinct_contexts++;
 
     /* insert into Context hash table */
-    idx = hash % cxts.size;
+    idx = (UInt) (hash % cxts.size);
     new->next = cxts.table[idx];
     cxts.table[idx] = new;
 
-#if CT_ENABLE_DEBUG
-    CT_DEBUGIF(3) {
+#if CLG_ENABLE_DEBUG
+    CLG_DEBUGIF(3) {
       VG_(printf)("  new_cxt ox%p: ", new);
-      SK_(print_cxt)(12, new, 0);
+      CLG_(print_cxt)(12, new, 0);
     }
 #endif
 
@@ -225,29 +228,30 @@ static Context* new_cxt(fn_node** fn)
 }
 
 /* get the Context structure for current context */
-Context* SK_(get_cxt)(fn_node** fn)
+Context* CLG_(get_cxt)(fn_node** fn)
 {
     Context* cxt;
-    UInt hash, size, idx;
+    UInt size, idx;
+    UWord hash;
 
-    CT_ASSERT(fn != 0);
+    CLG_ASSERT(fn != 0);
     if (*fn == 0) return 0;
     size = (*fn)->separate_callers+1;
     if (size<=0) { size = -size+1; }
 
-    CT_DEBUG(5, "+ get_cxt(fn '%s'): size %d\n",
+    CLG_DEBUG(5, "+ get_cxt(fn '%s'): size %d\n",
                 (*fn)->name, size);
 
     hash = cxt_hash_val(fn, size);
 
     if ( ((cxt = (*fn)->last_cxt) != 0) && is_cxt(hash, fn, cxt)) {
-        CT_DEBUG(5, "- get_cxt: 0x%p\n", cxt);
+        CLG_DEBUG(5, "- get_cxt: %p\n", cxt);
         return cxt;
     }
 
-    SK_(stat).cxt_lru_misses++;
+    CLG_(stat).cxt_lru_misses++;
 
-    idx = hash % cxts.size;
+    idx = (UInt) (hash % cxts.size);
     cxt = cxts.table[idx];
 
     while(cxt) {
@@ -260,7 +264,7 @@ Context* SK_(get_cxt)(fn_node** fn)
 
     (*fn)->last_cxt = cxt;
 
-    CT_DEBUG(5, "- get_cxt: %p\n", cxt);
+    CLG_DEBUG(5, "- get_cxt: %p\n", cxt);
 
     return cxt;
 }
@@ -270,55 +274,55 @@ Context* SK_(get_cxt)(fn_node** fn)
  * Change execution context by calling a new function from current context
  *
  */
-void SK_(push_cxt)(fn_node* fn)
+void CLG_(push_cxt)(fn_node* fn)
 {
-  call_stack* cs = &SK_(current_call_stack);
+  call_stack* cs = &CLG_(current_call_stack);
   Int fn_entries;
 
   /* save old context on stack (even if not changed at all!) */
-  CT_ASSERT(cs->sp < cs->size);
-  CT_ASSERT(cs->entry[cs->sp].cxt == 0);
-  cs->entry[cs->sp].cxt = SK_(current_state).cxt;
-  cs->entry[cs->sp].fn_sp = SK_(current_fn_stack).top - SK_(current_fn_stack).bottom;
+  CLG_ASSERT(cs->sp < cs->size);
+  CLG_ASSERT(cs->entry[cs->sp].cxt == 0);
+  cs->entry[cs->sp].cxt = CLG_(current_state).cxt;
+  cs->entry[cs->sp].fn_sp = CLG_(current_fn_stack).top - CLG_(current_fn_stack).bottom;
 
-  if (*(SK_(current_fn_stack).top) == fn) return;
+  if (*(CLG_(current_fn_stack).top) == fn) return;
   if (fn && (fn->group>0) &&
-      ((*(SK_(current_fn_stack).top))->group == fn->group)) return;
+      ((*(CLG_(current_fn_stack).top))->group == fn->group)) return;
 
   /* resizing needed ? */
-  fn_entries = SK_(current_fn_stack).top - SK_(current_fn_stack).bottom;
-  if (fn_entries == SK_(current_fn_stack).size-1) {
-    int new_size = SK_(current_fn_stack).size *2;
-    fn_node** new = VG_(malloc)(new_size * sizeof(fn_node*));
+  fn_entries = CLG_(current_fn_stack).top - CLG_(current_fn_stack).bottom;
+  if (fn_entries == CLG_(current_fn_stack).size-1) {
+    int new_size = CLG_(current_fn_stack).size *2;
+    fn_node** new = (fn_node**) CLG_MALLOC(new_size * sizeof(fn_node*));
     int i;
-    for(i=0;i<SK_(current_fn_stack).size;i++)
-      new[i] = SK_(current_fn_stack).bottom[i];
-    VG_(free)(SK_(current_fn_stack).bottom);
-    SK_(current_fn_stack).top = new + fn_entries;
-    SK_(current_fn_stack).bottom = new;
+    for(i=0;i<CLG_(current_fn_stack).size;i++)
+      new[i] = CLG_(current_fn_stack).bottom[i];
+    VG_(free)(CLG_(current_fn_stack).bottom);
+    CLG_(current_fn_stack).top = new + fn_entries;
+    CLG_(current_fn_stack).bottom = new;
 
-    CT_DEBUG(0, "Resize Context Stack: %d => %d (pushing '%s')\n", 
-	     SK_(current_fn_stack).size, new_size,
+    CLG_DEBUG(0, "Resize Context Stack: %d => %d (pushing '%s')\n", 
+	     CLG_(current_fn_stack).size, new_size,
 	     fn ? fn->name : (Char*)"0x0");
 
-    SK_(current_fn_stack).size = new_size;
+    CLG_(current_fn_stack).size = new_size;
   }
 
-  if (*(SK_(current_fn_stack).top) == 0) {
-    Int *pactive;
+  if (*(CLG_(current_fn_stack).top) == 0) {
+    UInt *pactive;
 
     /* this is first function: increment its active count */
-    CT_ASSERT(fn != 0);
-    pactive = SK_(get_fn_entry)(fn->number);
+    CLG_ASSERT(fn != 0);
+    pactive = CLG_(get_fn_entry)(fn->number);
     (*pactive)++;
   }
 
-  SK_(current_fn_stack).top++;
-  *(SK_(current_fn_stack).top) = fn;
-  SK_(current_state).cxt = SK_(get_cxt)(SK_(current_fn_stack).top);
+  CLG_(current_fn_stack).top++;
+  *(CLG_(current_fn_stack).top) = fn;
+  CLG_(current_state).cxt = CLG_(get_cxt)(CLG_(current_fn_stack).top);
 
-  CT_DEBUG(5, "  push_cxt(fn '%s'): %d\n", 
+  CLG_DEBUG(5, "  push_cxt(fn '%s'): %d\n", 
 	   fn ? fn->name : (Char*)"0x0",
-	   SK_(current_fn_stack).top - SK_(current_fn_stack).bottom);
+	   CLG_(current_fn_stack).top - CLG_(current_fn_stack).bottom);
 }
 			       
