@@ -7,10 +7,12 @@
 #ifndef CT_COSTS
 #define CT_COSTS
 
-#include "vg_skin.h"
+#include "pub_tool_basics.h"
 
-extern UInt SK_(costarray_entries);
-extern UInt SK_(costarray_chunks);
+#define CLG_(str) VGAPPEND(vgCallgrind_,str)
+
+extern UInt CLG_(costarray_entries);
+extern UInt CLG_(costarray_chunks);
 
 /* Array of 64bit costs. This is separated from other structs
  * to support a dynamic number of costs for a cost item.
@@ -26,8 +28,8 @@ struct _CostChunk {
 
 /* Allocate a number of 64bit cost values.
  * Typically used from ct_events.c */
-ULong* SK_(get_costarray)(Int size);
-void SK_(free_costarrays)();
+ULong* CLG_(get_costarray)(Int size);
+void CLG_(free_costarrays)();
 
 
 #endif /* CT_COSTS */
