@@ -41,8 +41,8 @@ typedef struct _ObjectList ObjectList;
 
 struct _SubEntry {
   DataType* type;
-  UInt offset;
-  UInt size;    /* Array if greater than type->size */
+  OffT offset;
+  SizeT size;    /* Array if greater than type->size */
 };
 
 /* How a type was deduced:
@@ -60,15 +60,16 @@ struct _SubEntry {
 struct _DataType {
   // this is an entry of a rhash
   rhash_entry* next;
-  UInt hash;
+  UWord hash;
 
-  char* name;
+  Char* name;
   UInt number;
 
   ULong read_cost, write_cost;
-  UInt allocCount, freeCount, allocSize, freeSize;
+  UInt allocCount, freeCount;
+  SizeT allocSize, freeSize;
 
-  UInt size;
+  SizeT size;
   UInt subentries, capacity;
   SubEntry e[1];
 };
@@ -76,7 +77,8 @@ struct _DataType {
 struct _DataObject {
   DataType* type;
   Char* name;
-  UInt addr, size;
+  Addr addr;
+  SizeT size;
 
   ULong* cost; /* allocated lazy */
 };
@@ -88,9 +90,10 @@ struct _DataObject {
 
 UInt type_counter = 0;
 
-static UInt str_hash(const UChar *s)
+static UWord str_hash(void* data)
 {
-    int hash_value = 0;
+    UChar* s = ((DataObject*)data)->name;
+    UWord hash_value = 0;
     for ( ; *s; s++)
         hash_value = (253 * hash_value + *s);
     return hash_value;
@@ -106,8 +109,8 @@ Bool is_type(rhash_entry* e, void* data)
 rhash_entry* new_type(void* data)
 {
   DataObject* o = (DataObject*) data;
-  DataType* new = VG_(malloc)(sizeof(DataType));
-  new->hash   = str_hash(o->name);
+  DataType* new = (DataType*) CLG_MALLOC(sizeof(DataType));
+
   new->name   = VG_(strdup)(o->name);
   new->number = type_counter++;
   new->read_cost = 0;
@@ -117,7 +120,7 @@ rhash_entry* new_type(void* data)
   new->allocSize = 0;
   new->freeSize = 0;
 
-  CT_DEBUG(3," new_type(%s)\n", new->name);
+  CLG_DEBUG(3," new_type(%s)\n", new->name);
 
   return (rhash_entry*) new;
 }
@@ -127,19 +130,20 @@ rhash type_hash = {
   .name = "TypeHash",
   .size = 577,
   .new_entry = new_type,
-  .has_key = is_type
+  .has_key = is_type,
+  .hash_value = str_hash
 };
 
 static
 void init_type_table()
 {
-  SK_(rhash_init)(&type_hash);
+  CLG_(rhash_init)(&type_hash);
 }
   
 static
 DataType* get_type(DataObject* o)
 {
-  return (DataType*) SK_(rhash_get)(&type_hash, str_hash(o->name), o);
+  return (DataType*) CLG_(rhash_get)(&type_hash, o);
 }
 
 static
@@ -147,7 +151,7 @@ void print_type(rhash_entry* e)
 {
   DataType* t = (DataType*) e;
  
-  CT_DEBUG(0,"T %5d /%4d Objs %6d /%5d %8llu R %8llu W %s\n",
+  CLG_DEBUG(0,"T %5d /%4d Objs %6d /%5d %8llu R %8llu W %s\n",
 	   t->allocCount, t->allocCount - t->freeCount,
 	   t->allocSize, t->allocSize - t->freeSize,
 	   t->read_cost, t->write_cost, t->name);
@@ -156,14 +160,14 @@ void print_type(rhash_entry* e)
 static
 void print_type_table()
 {
-  CT_DEBUG(0,"Entries of Type Table (%d):\n", type_hash.entries);
-  SK_(rhash_forall)(&type_hash, print_type);
+  CLG_DEBUG(0,"Entries of Type Table (%d):\n", type_hash.entries);
+  CLG_(rhash_forall)(&type_hash, print_type);
 }
 
 static
-DataObject* new_object(char* name, UInt addr, UInt size)
+DataObject* new_object(Char* name, UInt addr, UInt size)
 {
-  DataObject* o = (DataObject*) VG_(malloc)(sizeof(DataObject));
+  DataObject* o = (DataObject*) CLG_MALLOC(sizeof(DataObject));
 
   o->name  = VG_(strdup)(name);
   o->addr  = addr;
@@ -171,7 +175,7 @@ DataObject* new_object(char* name, UInt addr, UInt size)
   o->cost  = 0;
   o->type  = get_type(o);
 
-  CT_DEBUG(3," new_object(%s): addr %p, size %d\n", o->name, o->addr, o->size);
+  CLG_DEBUG(3," new_object(%s): addr %p, size %d\n", o->name, o->addr, o->size);
 
   return o;
 }
@@ -193,9 +197,9 @@ DataObject* new_object(char* name, UInt addr, UInt size)
 
 typedef struct _TablePos TablePos;
 struct _TablePos {
-  UInt addr;
-  UInt pos1, pos2, pos3;
-  UInt *table2, *table3;
+  UWord addr;
+  UWord pos1, pos2, pos3;
+  UWord *table2, *table3;
 };
 
 /* 2 lower bits of each table entry encode the type.
@@ -219,7 +223,7 @@ struct _TablePos {
 
 #define TABLE_LEVEL3_SIZE 1024
 
-UInt  data_table[TABLE_LEVEL1_SIZE];
+UWord data_table[TABLE_LEVEL1_SIZE];
 
 void init_page_table()
 {
@@ -230,11 +234,11 @@ void init_page_table()
 }
 
 static
-UInt* new_l2table()
+UWord* new_l2table()
 {
   Int i;
-  UInt* t = (UInt*) VG_(malloc)(sizeof(UInt)*TABLE_LEVEL2_SIZE);
-  CT_ASSERT(( ((UInt)t)&3) == 0);
+  UWord* t = (UWord*) CLG_MALLOC(sizeof(UWord)*TABLE_LEVEL2_SIZE);
+  CLG_ASSERT(( ((UWord)t)&3) == 0);
 
   for(i=0;i<TABLE_LEVEL2_SIZE;i++)
     t[i] = ENTRY_INVALID;
@@ -243,11 +247,11 @@ UInt* new_l2table()
 }
 
 static
-UInt* new_l3table()
+UWord* new_l3table()
 {
   Int i;
-  UInt* t = (UInt*) VG_(malloc)(sizeof(UInt)*TABLE_LEVEL3_SIZE);
-  CT_ASSERT( (((UInt)t)&3) == 0);
+  UWord* t = (UWord*) CLG_MALLOC(sizeof(UWord)*TABLE_LEVEL3_SIZE);
+  CLG_ASSERT( (((UWord)t)&3) == 0);
 
   for(i=0;i<TABLE_LEVEL3_SIZE;i++)
     t[i] = ENTRY_INVALID;
@@ -257,7 +261,7 @@ UInt* new_l3table()
 
 
 
-static UInt tablePosAddr(UInt pos1, UInt pos2, UInt pos3)
+static UWord tablePosAddr(UWord pos1, UWord pos2, UWord pos3)
 {
   return (pos1 << TABLE_LEVEL1_SHIFT) |
     (pos2 << TABLE_LEVEL2_SHIFT) | pos3;
@@ -269,7 +273,7 @@ static UInt tablePosAddr(UInt pos1, UInt pos2, UInt pos3)
  *
  * Returns depth.
  */
-static int minTableStartPos(UInt addr, TablePos* tp)
+static int minTableStartPos(UWord addr, TablePos* tp)
 {
   int d;
 
@@ -283,13 +287,13 @@ static int minTableStartPos(UInt addr, TablePos* tp)
     (tp->pos2 > 0) ? 2 : 
     (tp->pos1 > 0) ? 1 : 0;
 
-  CT_DEBUG(5," minTableStartPos(addr %p): pos %d/%d/%d, depth %d\n",
+  CLG_DEBUG(5," minTableStartPos(addr %p): pos %d/%d/%d, depth %d\n",
 	   addr, tp->pos1, tp->pos2, tp->pos3, d);
 
   return d;
 }
 
-static int minTableEndPos(UInt addr, TablePos* tp)
+static int minTableEndPos(UWord addr, TablePos* tp)
 {
   int d;
 
@@ -303,7 +307,7 @@ static int minTableEndPos(UInt addr, TablePos* tp)
     (tp->pos2 < TABLE_LEVEL2_SIZE-1) ? 2 : 
     (tp->pos1 < TABLE_LEVEL1_SIZE-1) ? 1 : 0;
 
-  CT_DEBUG(5," minTableEndPos(addr %p): pos %d/%d/%d, depth %d\n",
+  CLG_DEBUG(5," minTableEndPos(addr %p): pos %d/%d/%d, depth %d\n",
 	   addr, tp->pos1, tp->pos2, tp->pos3, d);
 
   return d;
@@ -316,35 +320,35 @@ static int minTableEndPos(UInt addr, TablePos* tp)
  */
 static int attachSubtables(int depth, TablePos* tp)
 {
-  UInt entry1, entry2, *table2, *table3;
+  UWord entry1, entry2, *table2, *table3;
 
-  CT_DEBUG(5," attachSubtables(depth %d, pos %d/%d/%d)\n",
+  CLG_DEBUG(5," attachSubtables(depth %d, pos %d/%d/%d)\n",
 	   depth, tp->pos1, tp->pos2, tp->pos3);
 
   entry1 = data_table[tp->pos1];
   if ((entry1 & ENTRY_TYPE_MASK) == ENTRY_OBJECT) {
     tp->table2 = tp->table3 = 0;
 
-    CT_DEBUG(5,"  => 0 [Object at L1]\n");
+    CLG_DEBUG(5,"  => 0 [Object at L1]\n");
     return 0;
   }
   if ((entry1 & ENTRY_TYPE_MASK) == ENTRY_INVALID) {
     if (depth <= 0) {
       tp->table2 = tp->table3 = 0;
 
-      CT_DEBUG(5,"  => 0\n");
+      CLG_DEBUG(5,"  => 0\n");
       return 0;
     }
     table2 = new_l2table();
-    entry1 = (UInt)table2 | ENTRY_SUBTABLE;
+    entry1 = (UWord)table2 | ENTRY_SUBTABLE;
     data_table[tp->pos1] = entry1;
 
-    CT_DEBUG(5,"   new L2 subtable at %d: %p\n",
+    CLG_DEBUG(5,"   new L2 subtable at %d: %p\n",
 	     tp->pos1, table2, entry1);
   }
   else {
-    table2 = (UInt*)(entry1 & ~3);
-    CT_DEBUG(5,"   L2 subtable at %d: %p\n", tp->pos1, table2);
+    table2 = (UWord*)(entry1 & ~3);
+    CLG_DEBUG(5,"   L2 subtable at %d: %p\n", tp->pos1, table2);
   }
   tp->table2 = table2;
 
@@ -352,25 +356,25 @@ static int attachSubtables(int depth, TablePos* tp)
   if ((entry2 & ENTRY_TYPE_MASK) == ENTRY_OBJECT) {
     tp->table3 = 0;
 
-    CT_DEBUG(5,"  => 1 [Object at L2]\n");
+    CLG_DEBUG(5,"  => 1 [Object at L2]\n");
     return 1;
   }
   if ((entry2 & ENTRY_TYPE_MASK) == ENTRY_INVALID) {
     if (depth == 1) {
       tp->table3 = 0;
 
-      CT_DEBUG(5,"  => 1\n");
+      CLG_DEBUG(5,"  => 1\n");
       return 1;
     }
     table3 = new_l3table();
-    entry2 = (UInt)table3 | ENTRY_SUBTABLE;
+    entry2 = (UWord)table3 | ENTRY_SUBTABLE;
     table2[tp->pos2] = entry2;
-    CT_DEBUG(5,"   new L3 subtable at %d: %p\n",
+    CLG_DEBUG(5,"   new L3 subtable at %d: %p\n",
 	     tp->pos2, table3, entry2);
   }
   else {
-    table3 = (UInt*)(entry2 & ~3);
-    CT_DEBUG(5,"   L3 subtable at %d: %p\n", tp->pos2, table3);
+    table3 = (UWord*)(entry2 & ~3);
+    CLG_DEBUG(5,"   L3 subtable at %d: %p\n", tp->pos2, table3);
   }
   tp->table3 = table3;
 
@@ -378,14 +382,14 @@ static int attachSubtables(int depth, TablePos* tp)
 }
 
 static
-void compactTable2(UInt* table, int idx)
+void compactTable2(UWord* table, int idx)
 { 
   int i;
-  UInt* subtable;
+  UWord* subtable;
   Bool isEmpty = True;
  
   if ((table[idx] & ENTRY_TYPE_MASK) != ENTRY_SUBTABLE) return;
-  subtable = (UInt*)(table[idx] & ~3);
+  subtable = (UWord*)(table[idx] & ~3);
   for(i=0;i<TABLE_LEVEL3_SIZE;i++) {
     if (subtable[i] != ENTRY_INVALID)
       isEmpty = False;
@@ -395,18 +399,18 @@ void compactTable2(UInt* table, int idx)
   table[idx] = ENTRY_INVALID;
   VG_(free)(subtable);
 
-  CT_DEBUG(5,"   compactTable2(./%d): deleted subtable\n", idx);
+  CLG_DEBUG(5,"   compactTable2(./%d): deleted subtable\n", idx);
 }
 
 static
-void compactTable1(UInt* table, int idx)
+void compactTable1(UWord* table, int idx)
 { 
   int i;
-  UInt* subtable;
+  UWord* subtable;
   Bool isEmpty = True;
  
   if ((table[idx] & ENTRY_TYPE_MASK) != ENTRY_SUBTABLE) return;
-  subtable = (UInt*)(table[idx] & ~3);
+  subtable = (UWord*)(table[idx] & ~3);
   for(i=0;i<TABLE_LEVEL2_SIZE;i++) {
     compactTable2(subtable,i);
     if (subtable[i] != ENTRY_INVALID)
@@ -417,19 +421,19 @@ void compactTable1(UInt* table, int idx)
   table[idx] = ENTRY_INVALID;
   VG_(free)(subtable);
 
-  CT_DEBUG(5,"  compactTable1(%d): deleted subtable\n", idx);
+  CLG_DEBUG(5,"  compactTable1(%d): deleted subtable\n", idx);
 }
     
 
 /* fill all entries between start and end */
 static
-void changeEntry(UInt from, UInt to, TablePos* start, TablePos* end)
+void changeEntry(UWord from, UWord to, TablePos* start, TablePos* end)
 {
   int i;
-  int start_pos1, end_pos1;
-  int start_pos2, end_pos2;
+  UWord start_pos1, end_pos1;
+  UWord start_pos2, end_pos2;
 
-  CT_DEBUG(5," changeEntry(%p -> %p, %d/%d/%d - %d/%d/%d)\n",
+  CLG_DEBUG(5," changeEntry(%p -> %p, %d/%d/%d - %d/%d/%d)\n",
 	   from, to,
 	   start->pos1, start->pos2, start->pos3,
 	   end->pos1, end->pos2, end->pos3);
@@ -437,7 +441,7 @@ void changeEntry(UInt from, UInt to, TablePos* start, TablePos* end)
   if (start->pos1 == end->pos1) {
     if (start->pos2 == end->pos2) {
       for(i = start->pos3; i<=end->pos3; i++) {
-	CT_ASSERT(start->table3[i] == from);
+	CLG_ASSERT(start->table3[i] == from);
 	start->table3[i] = to;
       }
     }
@@ -446,20 +450,20 @@ void changeEntry(UInt from, UInt to, TablePos* start, TablePos* end)
       if (start->table3) {
 	start_pos2++;
 	for(i = start->pos3; i<TABLE_LEVEL3_SIZE; i++) {
-	  CT_ASSERT(start->table3[i] == from);
+	  CLG_ASSERT(start->table3[i] == from);
 	  start->table3[i] = to;
 	}
       }
       if (end->table3) {
 	end_pos2--;
 	for(i = 0; i<= end->pos3; i++) {
-	  CT_ASSERT(end->table3[i] == from);
+	  CLG_ASSERT(end->table3[i] == from);
 	  end->table3[i] = to;
 	}
       }
       for(i = start_pos2; i<=end_pos2; i++) {
 	if (from == ENTRY_INVALID) compactTable2(start->table2,i);
-	CT_ASSERT(start->table2[i] == from);
+	CLG_ASSERT(start->table2[i] == from);
 	start->table2[i] = to;
       }
     }
@@ -473,13 +477,13 @@ void changeEntry(UInt from, UInt to, TablePos* start, TablePos* end)
     if (start->table3) {
       start_pos2++;      
       for(i = start->pos3; i<TABLE_LEVEL3_SIZE; i++) {
-	CT_ASSERT(start->table3[i] == from);
+	CLG_ASSERT(start->table3[i] == from);
 	start->table3[i] = to;
       }
     }
     for(i = start_pos2; i<TABLE_LEVEL2_SIZE; i++) {
       if (from == ENTRY_INVALID) compactTable2(start->table2,i);
-      CT_ASSERT(start->table2[i] == from);
+      CLG_ASSERT(start->table2[i] == from);
       start->table2[i] = to;
     }
   }
@@ -489,29 +493,30 @@ void changeEntry(UInt from, UInt to, TablePos* start, TablePos* end)
     if (end->table3) {
       end_pos2--;
       for(i = 0; i<= end->pos3; i++) {
-	CT_ASSERT(end->table3[i] == from);
+	CLG_ASSERT(end->table3[i] == from);
 	end->table3[i] = to;
       }
     }
     for(i = 0; i<=end_pos2; i++) {
       if (from == ENTRY_INVALID) compactTable2(end->table2,i);
-      CT_ASSERT(end->table2[i] == from);
+      CLG_ASSERT(end->table2[i] == from);
       end->table2[i] = to;
     }
   }
   for(i = start_pos1; i<=end_pos1; i++) {
     if (from == ENTRY_INVALID) compactTable1(data_table,i);
-    CT_ASSERT(data_table[i] == from);
+    CLG_ASSERT(data_table[i] == from);
     data_table[i] = to;
   }
 }
 
 /* Sets start and end according to addr and size */
-Bool setRange(UInt addr, UInt size, TablePos* start, TablePos* end)
+Bool setRange(UWord addr, UInt size, TablePos* start, TablePos* end)
 {
-  int startDepth, endDepth, d;
+  UWord startDepth, endDepth;
+  Int d;
 
-  CT_DEBUG(5," setRange(addr %p, size %u)\n", addr, size);
+  CLG_DEBUG(5," setRange(addr %p, size %u)\n", addr, size);
 
   startDepth = minTableStartPos(addr, start);
   endDepth   = minTableEndPos(addr+size-1, end);
@@ -529,7 +534,7 @@ Bool setRange(UInt addr, UInt size, TablePos* start, TablePos* end)
     }
   }
   
-  CT_DEBUG(5,"  > adjusted: depth %d - %d\n", startDepth, endDepth);
+  CLG_DEBUG(5,"  > adjusted: depth %d - %d\n", startDepth, endDepth);
 
   /* attach subtables, creating if needed.
    * return 0 if conflicts exist */
@@ -546,11 +551,11 @@ static
 void print_page_table()
 {
   int pos1, pos2, pos3;
-  UInt entry1, entry2, entry3, *table2, *table3;
+  UWord entry1, entry2, entry3, *table2, *table3;
   DataObject* o;
-  UInt last1, last2, last3;
+  UWord last1, last2, last3;
 
-  CT_DEBUG(0,"Page Table:\n");
+  CLG_DEBUG(0,"Page Table:\n");
 
   last1 = 0;
   for(pos1=0; pos1<TABLE_LEVEL1_SIZE; pos1++) {
@@ -559,23 +564,23 @@ void print_page_table()
     if ((entry1 == last1) &&
 	(entry1 & ENTRY_TYPE_MASK) != ENTRY_SUBTABLE) continue;
     if (last1 != 0)
-      CT_DEBUG(0,"P  ... - (%d)\n", pos1-1);
+      CLG_DEBUG(0,"P  ... - (%d)\n", pos1-1);
     last1 = entry1;
 
     if ((entry1 & ENTRY_TYPE_MASK) == ENTRY_OBJECT) {
       o = (DataObject*)(entry1 & ~3);
-      CT_DEBUG(0,"P  at L1 (%d): (%p/%u) %s\n",
+      CLG_DEBUG(0,"P  at L1 (%d): (%p/%u) %s\n",
 	       pos1, o->addr, o->size, o->name); 
       continue;
     }
     if ((entry1 & ENTRY_TYPE_MASK) == ENTRY_INVALID) {
-      CT_DEBUG(0,"P  at L1 (%d): None\n", pos1);
+      CLG_DEBUG(0,"P  at L1 (%d): None\n", pos1);
       continue;
     }
     last1 = 0;
 
-    table2 = (UInt*)(entry1 & ~3);
-    CT_DEBUG(0,"P  subtable (%p) at L1 %d (%p-%p)\n",
+    table2 = (UWord*)(entry1 & ~3);
+    CLG_DEBUG(0,"P  subtable (%p) at L1 %d (%p-%p)\n",
 	     table2, pos1,
 	     tablePosAddr(pos1,0,0),
 	     tablePosAddr(pos1,TABLE_LEVEL2_SIZE-1, TABLE_LEVEL3_SIZE-1));
@@ -587,23 +592,23 @@ void print_page_table()
       if ((entry2 == last2) &&
 	  (entry2 & ENTRY_TYPE_MASK) != ENTRY_SUBTABLE) continue;
       if (last2 != 0)
-	CT_DEBUG(0,"P   ... - (%d/%d)\n", pos1, pos2-1);
+	CLG_DEBUG(0,"P   ... - (%d/%d)\n", pos1, pos2-1);
       last2 = entry2;
 
       if ((entry2 & ENTRY_TYPE_MASK) == ENTRY_OBJECT) {
 	o = (DataObject*)(entry2 & ~3);
-	CT_DEBUG(0,"P   at L2 (%d/%d): (%p/%u) %s\n",
+	CLG_DEBUG(0,"P   at L2 (%d/%d): (%p/%u) %s\n",
 		 pos1, pos2, o->addr, o->size, o->name); 
 	continue;
       }
       if ((entry2 & ENTRY_TYPE_MASK) == ENTRY_INVALID) {
-	CT_DEBUG(3,"P   at L2 (%d/%d): None\n", pos1, pos2);
+	CLG_DEBUG(3,"P   at L2 (%d/%d): None\n", pos1, pos2);
 	continue;
       }
       last2 = 0;
 
-      table3 = (UInt*)(entry2 & ~3);
-      CT_DEBUG(0,"P   subtable (%p) at L2 %d/%d (%p-%p)\n", 
+      table3 = (UWord*)(entry2 & ~3);
+      CLG_DEBUG(0,"P   subtable (%p) at L2 %d/%d (%p-%p)\n", 
 	       table3, pos1, pos2,
 	       tablePosAddr(pos1,pos2,0),
 	       tablePosAddr(pos1,pos2, TABLE_LEVEL3_SIZE-1));
@@ -614,64 +619,64 @@ void print_page_table()
 
 	if (entry3 == last3) continue;
 	if (last3 != 0)
-	  CT_DEBUG(0,"P    ... - (%d/%d/%d)\n", pos1, pos2, pos3-1);
+	  CLG_DEBUG(0,"P    ... - (%d/%d/%d)\n", pos1, pos2, pos3-1);
 	last3 = entry3;
 
 	if ((entry3 & ENTRY_TYPE_MASK) == ENTRY_OBJECT) {
 	  o = (DataObject*)(entry3 & ~3);
-	  CT_DEBUG(0,"P    at L3 (%d/%d/%d): (%p/%u) %s\n",
+	  CLG_DEBUG(0,"P    at L3 (%d/%d/%d): (%p/%u) %s\n",
 		   pos1, pos2, pos3, o->addr, o->size, o->name); 
 	}
 	else
-	  CT_DEBUG(0,"P    at L3 (%d/%d/%d): None\n", pos1, pos2, pos3);
+	  CLG_DEBUG(0,"P    at L3 (%d/%d/%d): None\n", pos1, pos2, pos3);
       }
     }
   }
 }
 
-DataObject* get_object(UInt addr)
+DataObject* get_object(UWord addr)
 {
-  int pos1, pos2, pos3;
-  UInt entry, *table2, *table3;
+  UWord pos1, pos2, pos3;
+  UWord entry, *table2, *table3;
   DataObject* o;
 
   pos1 = addr >> TABLE_LEVEL1_SHIFT;
   entry = data_table[pos1];
   if ((entry & ENTRY_TYPE_MASK) == ENTRY_OBJECT) {
     o = (DataObject*)(entry & ~3);
-    CT_DEBUG(5," get_object(%p): at L1 (%d)\n  => (%p/%d) %s\n",
+    CLG_DEBUG(5," get_object(%p): at L1 (%d)\n  => (%p/%d) %s\n",
 	     addr, pos1, o->addr, o->size, o->name); 
     return o;
   }
   if ((entry & ENTRY_TYPE_MASK) == ENTRY_INVALID) {
-    CT_DEBUG(5," get_object(%p): None at L1 (%d)\n", addr, pos1); 
+    CLG_DEBUG(5," get_object(%p): None at L1 (%d)\n", addr, pos1); 
     return 0;
   }
-  table2 = (UInt*)(entry & ~3);
+  table2 = (UWord*)(entry & ~3);
   
   pos2 = (addr >> TABLE_LEVEL2_SHIFT) & (TABLE_LEVEL2_SIZE-1);
   entry = table2[pos2];
   if ((entry & ENTRY_TYPE_MASK) == ENTRY_OBJECT) {
     o = (DataObject*)(entry & ~3);
-    CT_DEBUG(5," get_object(%p): at L2 (%d/%d)\n  => (%p/%d) %s\n",
+    CLG_DEBUG(5," get_object(%p): at L2 (%d/%d)\n  => (%p/%d) %s\n",
 	     addr, pos1, pos2, o->addr, o->size, o->name); 
     return o;
   }
   if ((entry & ENTRY_TYPE_MASK) == ENTRY_INVALID) {
-    CT_DEBUG(5," get_object(%p): None at L2 (%d/%d)\n", addr, pos1, pos2); 
+    CLG_DEBUG(5," get_object(%p): None at L2 (%d/%d)\n", addr, pos1, pos2); 
     return 0;
   }
-  table3 = (UInt*)(entry & ~3);
+  table3 = (UWord*)(entry & ~3);
 
   pos3 = addr & (TABLE_LEVEL3_SIZE-1);
   entry = table3[pos3];
   if ((entry & ENTRY_TYPE_MASK) == ENTRY_OBJECT) {
     o = (DataObject*)(entry & ~3);
-    CT_DEBUG(5," get_object(%p): at L3 (%d/%d/%d)\n  => (%p/%u) %s\n",
+    CLG_DEBUG(5," get_object(%p): at L3 (%d/%d/%d)\n  => (%p/%u) %s\n",
 	     addr, pos1, pos2, pos3, o->addr, o->size, o->name); 
     return o;
   }
-  CT_DEBUG(5," get_object(%p): None at L3 (%d/%d/%d)\n",
+  CLG_DEBUG(5," get_object(%p): None at L3 (%d/%d/%d)\n",
 	   addr, pos1, pos2, pos3); 
   return 0;
 }
@@ -682,12 +687,12 @@ void remove_object(DataObject* o)
   TablePos start, end;
   Bool noConflict;
 
-  CT_DEBUG(5," remove_object(%s, addr %p, size %u)\n", 
+  CLG_DEBUG(5," remove_object(%s, addr %p, size %u)\n", 
 	   o->name, o->addr, o->size);
 
   noConflict = setRange(o->addr,o->size,&start,&end);
-  CT_ASSERT(noConflict == True);
-  changeEntry( ((UInt)o) | ENTRY_OBJECT, ENTRY_INVALID, &start, &end);
+  CLG_ASSERT(noConflict == True);
+  changeEntry( ((UWord)o) | ENTRY_OBJECT, ENTRY_INVALID, &start, &end);
 }
 
 
@@ -702,17 +707,17 @@ void add_object(DataObject* o)
 
   DataObject* o2 = get_object(o->addr);
   if (o2) {
-    CT_DEBUG(0,"WARNING: removing old '%s' (%p/%d)\n         for new '%s' (%p/%d)\n",
+    CLG_DEBUG(0,"WARNING: removing old '%s' (%p/%d)\n         for new '%s' (%p/%d)\n",
 	     o2->name, o2->addr, o2->size,
 	     o->name, o->addr, o->size);
     remove_object(o2);
   }
 
-  CT_DEBUG(5," add_object(%s, addr %p, size %u)\n",
+  CLG_DEBUG(5," add_object(%s, addr %p, size %u)\n",
 	   o->name, o->addr, o->size);
   noConflict = setRange(o->addr,o->size,&start,&end);
-  if (!noConflict) { print_page_table(); CT_ASSERT(0);  }
-  changeEntry( ENTRY_INVALID, ((UInt)o) | ENTRY_OBJECT, &start, &end);
+  if (!noConflict) { print_page_table(); CLG_ASSERT(0);  }
+  changeEntry( ENTRY_INVALID, ((UWord)o) | ENTRY_OBJECT, &start, &end);
 }
 
 
@@ -723,7 +728,7 @@ void add_object(DataObject* o)
 
 DataType* aType;
 
-void SK_(init_data)()
+void CLG_(init_data)()
 {
   init_page_table();
   init_type_table();
@@ -735,36 +740,36 @@ void SK_(init_data)()
 /* Need to get the size and subelements first time
  * from debug info.
  */
-void SK_(handle_constructor)(UInt addr, Char* fullname)
+void CLG_(handle_constructor)(Addr addr, Char* fullname)
 {
-  VG_(message)(Vg_DebugMsg, "Constructor %s for %x",
+  VG_(message)(Vg_DebugMsg, "Constructor %s for %p",
 	       fullname, addr);
 }
 
-void SK_(handle_destructor)(UInt addr, Char* fullname)
+void CLG_(handle_destructor)(Addr addr, Char* fullname)
 {
-  VG_(message)(Vg_DebugMsg, "Destructor %s for %x",
+  VG_(message)(Vg_DebugMsg, "Destructor %s for %p",
 	       fullname, addr);
 }
 
 Char obj_name[BUF_LEN];
 int allocated = 0, freed = 0;
 
-void SK_(handle_malloc)(UInt addr, UInt size)
+void CLG_(handle_malloc)(Addr addr, SizeT size)
 {
   DataObject* o;
-  fn_node** pfn = SK_(current_fn_stack).top;
+  fn_node** pfn = CLG_(current_fn_stack).top;
   int opos = 0, d=0;
 
   if (addr == 0) {
-    CT_DEBUG(3," Failed allocation (addr %x, size %d)\n",
+    CLG_DEBUG(3," Failed allocation (addr %p, size %p)\n",
 	     addr, size);
     return;
   }
 
-  CT_DEBUG(3," Allocated (%x-%x, size %d)\n", addr, addr+size-1, size);
+  CLG_DEBUG(3," Allocated (%p-%p, size %p)\n", addr, addr+size-1, size);
   while(*pfn) {
-    CT_DEBUG(3,"  from %s\n", (*pfn)->name);
+    CLG_DEBUG(3,"  from %s\n", (*pfn)->name);
     if ((opos<BUF_LEN) && (++d<4))
       opos += VG_(sprintf)(obj_name+opos, "'%s", (*pfn)->name);
     pfn--;
@@ -776,23 +781,23 @@ void SK_(handle_malloc)(UInt addr, UInt size)
   o->type->allocSize += size;
 }
 
-Int SK_(handle_realloc)(UInt newaddr, UInt oldaddr, UInt newsize)
+Int CLG_(handle_realloc)(Addr newaddr, Addr oldaddr, SizeT newsize)
 {
   DataObject* o;
   Int oldsize = 0;
 
   if (newaddr == 0) {
-    CT_DEBUG(3," Failed reallocation (newaddr %x, oldaddr %x, newsize %d)\n",
+    CLG_DEBUG(3," Failed reallocation (newaddr %p, oldaddr %p, newsize %p)\n",
 	     newaddr, oldaddr, newsize);
     return 0;
   }
 
-  CT_DEBUG(3," Reallocated (old %x, new %x-%x, size %d)\n",
+  CLG_DEBUG(3," Reallocated (old %p, new %p-%p, size %p)\n",
 	   oldaddr, newaddr, newaddr+newsize-1, newsize);
 
   o = get_object(oldaddr);
   if (o && (o->addr == oldaddr)) {
-    CT_DEBUG(3,"  old size %d, obj %s\n", o->size, o->name);
+    CLG_DEBUG(3,"  old size %p, obj %s\n", o->size, o->name);
     oldsize = o->size;
     remove_object(o);
     o->addr = newaddr;
@@ -802,21 +807,21 @@ Int SK_(handle_realloc)(UInt newaddr, UInt oldaddr, UInt newsize)
     o->type->allocSize += newsize-oldsize;
   }
   else
-    CT_DEBUG(3,"  nothing found to realloc.\n");
+    CLG_DEBUG(3,"  nothing found to realloc.\n");
 
   return oldsize;
 }
 
 /* returns size of freed allocation */
-Int SK_(handle_free)(UInt addr)
+Int CLG_(handle_free)(Addr addr)
 {
-  fn_node** pfn = SK_(current_fn_stack).top-1;
+  fn_node** pfn = CLG_(current_fn_stack).top-1;
   DataObject* o;
   Int res = 0;
 
-  CT_DEBUG(3," Free %x\n", addr);
+  CLG_DEBUG(3," Free %p\n", addr);
   while(*pfn) {
-    CT_DEBUG(3,"  from %s\n", (*pfn)->name);
+    CLG_DEBUG(3,"  from %s\n", (*pfn)->name);
     pfn--;
   }
 
@@ -832,7 +837,7 @@ Int SK_(handle_free)(UInt addr)
     VG_(free)(o);
   }
   else
-    CT_DEBUG(3,"  nothing found to free.\n");
+    CLG_DEBUG(3,"  nothing found to free.\n");
 
   return res;
 }
@@ -842,7 +847,7 @@ ULong* getcost(Addr addr, Context* cxt)
   return 0;
 }
 
-void SK_(handle_read)(UInt addr, UInt size, int sim)
+void CLG_(handle_read)(Addr addr, UInt size, int sim)
 {
   DataObject* o = get_object(addr);
   DataType* t = o ? o->type : aType;
@@ -850,13 +855,13 @@ void SK_(handle_read)(UInt addr, UInt size, int sim)
   t->read_cost++;
 
   if (o) {
-    CT_DEBUG(3,"Read  %p: Off %d in %s (%p/%d)\n",
+    CLG_DEBUG(3,"Read  %p: Off %d in %s (%p/%d)\n",
 	     addr, addr - o->addr, o->name, o->addr, o->size);
   }
-  CT_DEBUG(3,"Read (%p), reads of %s: %llu\n", addr, t->name, t->read_cost);
+  CLG_DEBUG(3,"Read (%p), reads of %s: %llu\n", addr, t->name, t->read_cost);
 }
 
-void SK_(handle_write)(UInt addr, UInt size, int sim)
+void CLG_(handle_write)(Addr addr, UInt size, int sim)
 {
   DataObject* o = get_object(addr);
   DataType* t = o ? o->type : aType;
@@ -864,95 +869,95 @@ void SK_(handle_write)(UInt addr, UInt size, int sim)
   t->write_cost++;
 
   if (o) {
-    CT_DEBUG(3,"Write %p: Off %d in %s (%p/%d)\n",
+    CLG_DEBUG(3,"Write %p: Off %d in %s (%p/%d)\n",
 	     addr, addr - o->addr, o->name, o->addr, o->size);
   }
-  CT_DEBUG(3,"Write (%p), writes of %s: %llu\n", addr, t->name, t->write_cost);
+  CLG_DEBUG(3,"Write (%p), writes of %s: %llu\n", addr, t->name, t->write_cost);
 }
 
-void SK_(finish_data)()
+void CLG_(finish_data)()
 {
   print_type_table();
   //print_page_table();
 }
 
-void SK_(my_new_mem_startup)(Addr a, SizeT len, Bool rr, Bool ww, Bool xx)
+void CLG_(my_new_mem_startup)(Addr a, SizeT len, Bool rr, Bool ww, Bool xx)
 {
   //DataObject* o;
   static char buf[64];
 
-  if (!SK_(clo).collect_data) return;
+  if (!CLG_(clo).collect_data) return;
 
-  VG_(sprintf)(buf,"Seg[%08p-%08p/%c%c%c]", a, a+len-1,
+  VG_(sprintf)(buf,"Seg[%p-%p/%c%c%c]", a, a+len-1,
 	       rr ? 'r':'-', ww ? 'w':'-', xx ? 'x':'-');
 
-  CT_DEBUG(0,"new_mem_startup: %s\n", buf);
+  CLG_DEBUG(0,"new_mem_startup: %s\n", buf);
   
   //o = new_object(buf, a, len);
   //add_object(o);
 }
 
-void SK_(my_new_mem_stack_signal)(Addr a, SizeT len)
+void CLG_(my_new_mem_stack_signal)(Addr a, SizeT len)
 {
   DataObject* o;
   static char buf[64];
 
-  if (!SK_(clo).collect_data) return;
+  if (!CLG_(clo).collect_data) return;
 
-  VG_(sprintf)(buf,"Stack[%08p-%08p]", a, a+len-1);
+  VG_(sprintf)(buf,"Stack[%p-%p]", a, a+len-1);
 
-  CT_DEBUG(0,"new_mem_stack_signal: %s\n", buf);
+  CLG_DEBUG(0,"new_mem_stack_signal: %s\n", buf);
   
   o = new_object(buf, a, len);
   add_object(o);
 }
 
-void SK_(my_new_mem_brk)(Addr a, SizeT len)
+void CLG_(my_new_mem_brk)(Addr a, SizeT len)
 {
   DataObject* o;
   static char buf[64];
 
-  if (!SK_(clo).collect_data) return;
+  if (!CLG_(clo).collect_data) return;
 
-  VG_(sprintf)(buf,"Brk[%08p-%08p]", a, a+len-1);
+  VG_(sprintf)(buf,"Brk[%p-%p]", a, a+len-1);
 
-  CT_DEBUG(0,"new_mem_brk: %s\n", buf);
+  CLG_DEBUG(0,"new_mem_brk: %s\n", buf);
   
   o = new_object(buf, a, len);
   add_object(o);
 }
 
-void SK_(my_new_mem_mmap)(Addr a, SizeT len, Bool rr, Bool ww, Bool xx)
+void CLG_(my_new_mem_mmap)(Addr a, SizeT len, Bool rr, Bool ww, Bool xx)
 {
   //DataObject* o;
   static char buf[64];
 
-  if (!SK_(clo).collect_data) return;
+  if (!CLG_(clo).collect_data) return;
 
-  VG_(sprintf)(buf,"MMap[%08p-%08p/%c%c%c]", a, a+len-1,
+  VG_(sprintf)(buf,"MMap[%p-%p/%c%c%c]", a, a+len-1,
 	       rr ? 'r':'-', ww ? 'w':'-', xx ? 'x':'-');
 
-  CT_DEBUG(0,"new_mem_mmap: %s\n", buf);
+  CLG_DEBUG(0,"new_mem_mmap: %s\n", buf);
   
   //o = new_object(buf, a, len);
   //add_object(o);
 }
 
-void SK_(new_data_sym)(Addr a, SizeT len, SegInfo* si, Char* name)
+void CLG_(new_data_sym)(Addr a, SizeT len, SegInfo* si, Char* name)
 {
   DataObject* o;
   static char buf[256];
 
-  VG_(sprintf)(buf,"%s:%s [%08p/%d]", 
-	       VG_(seg_filename)(si), name, a, len);
+  VG_(sprintf)(buf,"%s:%s [%p-%p]", 
+	       VG_(seginfo_filename)(si), name, a, a+len-1);
 
-  CT_DEBUG(0,"new_data_sym: %s\n", buf);
+  CLG_DEBUG(0,"new_data_sym: %s\n", buf);
 
   o = new_object(buf, a, len);
   add_object(o);
 }
 
-void SK_(free_seg_info)(SegInfo* si)
+void CLG_(free_seg_info)(SegInfo* si)
 {
-  CT_DEBUG(0,"free_seg_info: %s\n", VG_(seg_filename)(si));
+  CLG_DEBUG(0,"free_seg_info: %s\n", VG_(seginfo_filename)(si));
 }
