@@ -30,7 +30,7 @@
 
 /* If debugging mode of, dummy functions are provided (see below)
  */
-#if CT_ENABLE_DEBUG
+#if CLG_ENABLE_DEBUG
 
 /*------------------------------------------------------------*/
 /*--- Debug output helpers                                 ---*/
@@ -44,14 +44,14 @@ static void print_indent(int s)
     VG_(printf)(sp+40-s);
 }
 
-void SK_(print_bb)(int s, BB* bb)
+void CLG_(print_bb)(int s, BB* bb)
 {
     if (s<0) {
 	s = -s;
 	print_indent(s);
     }
 
-    VG_(printf)("BB 0x%08x (Obj '%s')", bb_addr(bb), bb->obj->name);
+    VG_(printf)("BB %p (Obj '%s')", bb_addr(bb), bb->obj->name);
 }
 
 static
@@ -72,7 +72,7 @@ void print_mangled_cxt(Context* cxt, int rec_index)
 
 
 
-void SK_(print_cxt)(int s, Context* cxt, int rec_index)
+void CLG_(print_cxt)(int s, Context* cxt, int rec_index)
 {
   if (s<0) {
     s = -s;
@@ -80,8 +80,8 @@ void SK_(print_cxt)(int s, Context* cxt, int rec_index)
   }
   
   if (cxt) {
-    Int *pactive = SK_(get_fn_entry)(cxt->fn[0]->number);
-    CT_ASSERT(rec_index < cxt->fn[0]->separate_recursions);
+    UInt *pactive = CLG_(get_fn_entry)(cxt->fn[0]->number);
+    CLG_ASSERT(rec_index < cxt->fn[0]->separate_recursions);
     
     VG_(printf)("Cxt %d" ,cxt->base_number + rec_index);
     if (*pactive>0)
@@ -94,8 +94,25 @@ void SK_(print_cxt)(int s, Context* cxt, int rec_index)
     VG_(printf)("(no context)\n");
 }
 
+void CLG_(print_execstate)(int s, exec_state* es)
+{
+  if (s<0) {
+    s = -s;
+    print_indent(s);
+  }
+  
+  if (!es) {
+    VG_(printf)("ExecState 0x0\n");
+    return;
+  }
 
-void SK_(print_bbcc)(int s, BBCC* bbcc, Bool jumpaddr)
+  VG_(printf)("ExecState [Sig %d, collect %s, nonskipped %p]: jmps_passed %d\n",
+	      es->sig, es->collect?"yes":"no",
+	      es->nonskipped, es->jmps_passed);
+}
+
+
+void CLG_(print_bbcc)(int s, BBCC* bbcc, Bool jumpaddr)
 {
   BB* bb;
 
@@ -110,20 +127,22 @@ void SK_(print_bbcc)(int s, BBCC* bbcc, Bool jumpaddr)
   }
  
   bb = bbcc->bb;
-  CT_ASSERT(bb!=0);
- 
+  CLG_ASSERT(bb!=0);
+
+#if 0 
   if (jumpaddr)
-    VG_(printf)("%s +0x%x=0x%x, ",
+    VG_(printf)("%s +%p=%p, ",
 	      bb->obj->name + bb->obj->last_slash_pos,
 	      bb->jmp_offset, bb_jmpaddr(bb));
   else
-    VG_(printf)("%s +0x%x=0x%x, ",
+#endif
+    VG_(printf)("%s +%p=%p, ",
 		bb->obj->name + bb->obj->last_slash_pos,
 		bb->offset, bb_addr(bb));
-  SK_(print_cxt)(s+8, bbcc->cxt, bbcc->rec_index);
+  CLG_(print_cxt)(s+8, bbcc->cxt, bbcc->rec_index);
 }
 
-void SK_(print_eventset)(int s, EventSet* es)
+void CLG_(print_eventset)(int s, EventSet* es)
 {
   int i;
 
@@ -156,7 +175,7 @@ void SK_(print_eventset)(int s, EventSet* es)
 }
 
 
-void SK_(print_cost)(int s, EventSet* es, ULong* c)
+void CLG_(print_cost)(int s, EventSet* es, ULong* c)
 {
   Int i, pos;
 
@@ -198,21 +217,21 @@ void SK_(print_cost)(int s, EventSet* es, ULong* c)
 }
 
 
-void SK_(print_short_jcc)(jCC* jcc)
+void CLG_(print_short_jcc)(jCC* jcc)
 {
     if (jcc)
-	VG_(printf)("%x => %x [%llu/%llu,%llu,%llu]",
+	VG_(printf)("%p => %p [%llu/%llu,%llu,%llu]",
 		    bb_jmpaddr(jcc->from->bb),
 		    bb_addr(jcc->to->bb),
 		    jcc->call_counter,
-		    jcc->cost ? jcc->cost[SK_(sets).off_sim_Ir]:0,
-		    jcc->cost ? jcc->cost[SK_(sets).off_sim_Dr]:0,
-		    jcc->cost ? jcc->cost[SK_(sets).off_sim_Dw]:0);
+		    jcc->cost ? jcc->cost[CLG_(sets).off_sim_Ir]:0,
+		    jcc->cost ? jcc->cost[CLG_(sets).off_sim_Dr]:0,
+		    jcc->cost ? jcc->cost[CLG_(sets).off_sim_Dw]:0);
     else
 	VG_(printf)("[Skipped JCC]");
 }
 
-void SK_(print_jcc)(int s, jCC* jcc)
+void CLG_(print_jcc)(int s, jCC* jcc)
 {
     if (s<0) {
 	s = -s;
@@ -223,19 +242,19 @@ void SK_(print_jcc)(int s, jCC* jcc)
 	VG_(printf)("JCC to skipped function\n");
 	return;
     }
-    VG_(printf)("JCC 0x%x from ", jcc);
-    SK_(print_bbcc)(s+9, jcc->from, True);
+    VG_(printf)("JCC %p from ", jcc);
+    CLG_(print_bbcc)(s+9, jcc->from, True);
     print_indent(s+4);    
     VG_(printf)("to   ");
-    SK_(print_bbcc)(s+9, jcc->to, False);
+    CLG_(print_bbcc)(s+9, jcc->to, False);
     print_indent(s+4);
     VG_(printf)("Calls %llu\n", jcc->call_counter);
     print_indent(s+4);
-    SK_(print_cost)(s+9, SK_(sets).full, jcc->cost);
+    CLG_(print_cost)(s+9, CLG_(sets).full, jcc->cost);
 }
 
 /* dump out the current call stack */
-void SK_(print_stackentry)(int s, int sp)
+void CLG_(print_stackentry)(int s, int sp)
 {
     call_entry* ce;
 
@@ -244,15 +263,15 @@ void SK_(print_stackentry)(int s, int sp)
 	print_indent(s);
     }
 
-    ce = SK_(get_call_entry)(sp);
-    VG_(printf)("[%-2d] ESP 0x%x", sp, ce->esp);
+    ce = CLG_(get_call_entry)(sp);
+    VG_(printf)("[%-2d] SP %p, RA %p", sp, ce->sp, ce->ret_addr);
     if (ce->nonskipped)
-	VG_(printf)(" NonSkipped BB 0x%x / %s",
+	VG_(printf)(" NonSkipped BB %p / %s",
 		    bb_addr(ce->nonskipped->bb),
 		    ce->nonskipped->cxt->fn[0]->name);
     VG_(printf)("\n");
     print_indent(s+5);
-    SK_(print_jcc)(5,ce->jcc);
+    CLG_(print_jcc)(5,ce->jcc);
 }
 
 /* debug output */
@@ -262,12 +281,12 @@ static void print_call_stack()
     int c;
 
     VG_(printf)("Call Stack:\n");
-    for(c=0;c<SK_(current_call_stack).sp;c++)
-      SK_(print_stackentry)(-2, c);
+    for(c=0;c<CLG_(current_call_stack).sp;c++)
+      CLG_(print_stackentry)(-2, c);
 }
 #endif
 
-void SK_(print_bbcc_fn)(BBCC* bbcc)
+void CLG_(print_bbcc_fn)(BBCC* bbcc)
 {
     obj_node* obj;
 
@@ -296,8 +315,57 @@ void SK_(print_bbcc_fn)(BBCC* bbcc)
     }
 }	
 
+void CLG_(print_bbcc_cost)(int s, BBCC* bbcc)
+{
+  BB* bb;
+  Int i, cjmpNo;
+  ULong ecounter;
+
+  if (s<0) {
+    s = -s;
+    print_indent(s);
+  }
+  
+  if (!bbcc) {
+    VG_(printf)("BBCC 0x0\n");
+    return;
+  }
+ 
+  bb = bbcc->bb;
+  CLG_ASSERT(bb!=0);
+    
+  CLG_(print_bbcc)(s, bbcc, False);
+
+  ecounter = bbcc->ecounter_sum;
+
+  print_indent(s+2);
+  VG_(printf)("ECounter: sum %d ", ecounter);
+  for(i=0; i<bb->cjmp_count; i++) {
+      VG_(printf)("[%d]=%d ",
+		  bb->jmp[i].instr, bbcc->jmp[i].ecounter);
+  }
+  VG_(printf)("\n");
+
+  cjmpNo = 0; 
+  for(i=0; i<bb->instr_count; i++) {
+      InstrInfo* ii = &(bb->instr[i]);
+      print_indent(s+2);
+      VG_(printf)("[%2d] IOff %2d ecnt %3d ",
+		  i, ii->instr_offset, ecounter);
+      CLG_(print_cost)(s+5, ii->eventset, bbcc->cost + ii->cost_offset);
+
+      /* update execution counter */
+      if (cjmpNo < bb->cjmp_count)
+	  if (bb->jmp[cjmpNo].instr == i) {
+	      ecounter -= bbcc->jmp[cjmpNo].ecounter;
+	      cjmpNo++;
+	  }
+  }
+}
+
+
 /* dump out an address with source info if available */
-void SK_(print_addr)(Addr addr)
+void CLG_(print_addr)(Addr addr)
 {
     Char fl_buf[FILENAME_LEN];
     Char fn_buf[FN_NAME_LEN];
@@ -310,15 +378,15 @@ void SK_(print_addr)(Addr addr)
 	return;
     }
 
-    SK_(get_debug_info)(addr, fl_buf, fn_buf, &ln, &si);
+    CLG_(get_debug_info)(addr, fl_buf, fn_buf, &ln, &si);
 
     if (VG_(strcmp)(fn_buf,"???")==0)
-	VG_(printf)("%x", addr);
+	VG_(printf)("%p", addr);
     else
-	VG_(printf)("%x %s", addr, fn_buf);
+	VG_(printf)("%p %s", addr, fn_buf);
 
     if (si) {
-      obj_name = VG_(seg_filename)(si);
+      obj_name = VG_(seginfo_filename)(si);
       if (obj_name) {
 	while(obj_name[i]) {
 	  if (obj_name[i]=='/') opos = i+1;
@@ -333,48 +401,53 @@ void SK_(print_addr)(Addr addr)
     	VG_(printf)(" (%s:%u)", fl_buf,ln);
 }
 
-void SK_(print_addr_ln)(Addr addr)
+void CLG_(print_addr_ln)(Addr addr)
 {
-  SK_(print_addr)(addr);
+  CLG_(print_addr)(addr);
   VG_(printf)("\n");
 }
 
 static ULong bb_written = 0;
 
-void SK_(print_bbno)(void)
+void CLG_(print_bbno)(void)
 {
-  if (bb_written != SK_(stat).bb_executions) {
-    bb_written = SK_(stat).bb_executions;
-    VG_(printf)("BB# %llu\n",SK_(stat).bb_executions);
+  if (bb_written != CLG_(stat).bb_executions) {
+    bb_written = CLG_(stat).bb_executions;
+    VG_(printf)("BB# %llu\n",CLG_(stat).bb_executions);
   }
 }
 
-void SK_(print_context)(void)
+void CLG_(print_context)(void)
 {
   BBCC* bbcc;
 
-  CT_DEBUG(0,"In tid %d [%d] ",
-	   SK_(current_tid),  SK_(current_call_stack).sp);
-  bbcc =  SK_(current_state).bbcc;
-  print_mangled_cxt(SK_(current_state).cxt,
+  CLG_DEBUG(0,"In tid %d [%d] ",
+	   CLG_(current_tid),  CLG_(current_call_stack).sp);
+  bbcc =  CLG_(current_state).bbcc;
+  print_mangled_cxt(CLG_(current_state).cxt,
 		    bbcc ? bbcc->rec_index : 0);
   VG_(printf)("\n");
 }
 
+void* CLG_(malloc)(UWord s, char* f)
+{
+    CLG_DEBUG(3, "Malloc(%d) in %s.\n", s, f);
+    return VG_(malloc)(s);
+}
 
-#else /* CT_ENABLE_DEBUG */
+#else /* CLG_ENABLE_DEBUG */
 
-void SK_(print_bbno)(void) {}
-void SK_(print_context)(void) {}
-void SK_(print_jcc)(int s, jCC* jcc) {}
-void SK_(print_bbcc)(int s, BBCC* bbcc, Bool b) {}
-void SK_(print_bbcc_fn)(BBCC* bbcc) {}
-void SK_(print_cost)(int s, EventSet* es, ULong* cost) {}
-void SK_(print_bb)(int s, BB* bb) {}
-void SK_(print_cxt)(int s, Context* cxt, int rec_index) {}
-void SK_(print_short_jcc)(jCC* jcc) {}
-void SK_(print_stackentry)(int s, int sp) {}
-void SK_(print_addr)(Addr addr) {}
-void SK_(print_addr_ln)(Addr addr) {}
+void CLG_(print_bbno)(void) {}
+void CLG_(print_context)(void) {}
+void CLG_(print_jcc)(int s, jCC* jcc) {}
+void CLG_(print_bbcc)(int s, BBCC* bbcc, Bool b) {}
+void CLG_(print_bbcc_fn)(BBCC* bbcc) {}
+void CLG_(print_cost)(int s, EventSet* es, ULong* cost) {}
+void CLG_(print_bb)(int s, BB* bb) {}
+void CLG_(print_cxt)(int s, Context* cxt, int rec_index) {}
+void CLG_(print_short_jcc)(jCC* jcc) {}
+void CLG_(print_stackentry)(int s, int sp) {}
+void CLG_(print_addr)(Addr addr) {}
+void CLG_(print_addr_ln)(Addr addr) {}
 
 #endif
