@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------*/
 /*--- Callgrind                                                    ---*/
-/*---                                                   ct_costs.c ---*/
+/*---                                                     events.c ---*/
 /*--------------------------------------------------------------------*/
 
 /*
@@ -26,16 +26,14 @@
    The GNU General Public License is contained in the file COPYING.
 */
 
-#include "global.h" // debugging macros
-#include "events.h"
-#include "costs.h"
+#include "global.h"
 
 #define MAX_EVENTTYPE 20
 
 static EventType eventtype[MAX_EVENTTYPE];
 static Int eventtype_count = 0;
 
-EventType* SK_(register_eventtype)(char* name)
+EventType* CLG_(register_eventtype)(Char* name)
 {
   EventType* et;
 
@@ -43,7 +41,7 @@ EventType* SK_(register_eventtype)(char* name)
     VG_(printf)("\nMore than %d event types used!\n"
 		"Increase MAX_EVENTTYPE in ct_events.c and recomile this tool!\n",
 		MAX_EVENTTYPE);
-    VG_(skin_panic)("Too many event types requested.");
+    VG_(tool_panic)("Too many event types requested.");
   }
 
   et = &(eventtype[eventtype_count]);
@@ -57,7 +55,7 @@ EventType* SK_(register_eventtype)(char* name)
 }
 
 
-EventType* SK_(get_eventtype)(char* name)
+EventType* CLG_(get_eventtype)(Char* name)
 {
   Int i;
 
@@ -67,7 +65,7 @@ EventType* SK_(get_eventtype)(char* name)
   return 0;
 }
 
-EventType* SK_(get_eventtype_byindex)(Int id)
+EventType* CLG_(get_eventtype_byindex)(Int id)
 {
   if ((id >= 0) && (id < eventtype_count))
     return eventtype+id;
@@ -75,11 +73,11 @@ EventType* SK_(get_eventtype_byindex)(Int id)
 }
 
 /* Allocate space for an event set */
-EventSet* SK_(get_eventset)(Char* n, Int capacity)
+EventSet* CLG_(get_eventset)(Char* n, Int capacity)
 {
   EventSet* es;
 
-  es = (EventSet*) VG_(malloc)(sizeof(EventSet) +
+  es = (EventSet*) CLG_MALLOC(sizeof(EventSet) +
 			       capacity * sizeof(EventSetEntry));
   es->capacity = capacity;
   es->size = 0;
@@ -89,7 +87,7 @@ EventSet* SK_(get_eventset)(Char* n, Int capacity)
 }
 
 /* Incorporate a event type into a set, get start offset */
-Int SK_(add_eventtype)(EventSet* es, EventType* t)
+Int CLG_(add_eventtype)(EventSet* es, EventType* t)
 {
   Int offset = es->size;
   if (es->capacity - offset < 1) return -1;
@@ -102,7 +100,7 @@ Int SK_(add_eventtype)(EventSet* es, EventType* t)
 }
 
 /* Incorporate one event set into another, get start offset */
-Int SK_(add_eventset)(EventSet* dst, EventSet* src)
+Int CLG_(add_eventset)(EventSet* dst, EventSet* src)
 {
   Int offset = dst->size, i;
   if (!src || (src->size == 0)) return offset;
@@ -119,7 +117,7 @@ Int SK_(add_eventset)(EventSet* dst, EventSet* src)
 }
 
 /* Incorporate two event types into a set, with second < first */
-Int SK_(add_dep_event2)(EventSet* es, EventType* e1, EventType* e2)
+Int CLG_(add_dep_event2)(EventSet* es, EventType* e1, EventType* e2)
 {
   Int offset = es->size;
 
@@ -135,7 +133,7 @@ Int SK_(add_dep_event2)(EventSet* es, EventType* e1, EventType* e2)
 }
 
 /* Incorporate 3 event types into a set, with third < second < first */
-Int SK_(add_dep_event3)(EventSet* es,
+Int CLG_(add_dep_event3)(EventSet* es,
 			EventType* e1, EventType* e2, EventType* e3)
 {
   Int offset = es->size;
@@ -153,7 +151,7 @@ Int SK_(add_dep_event3)(EventSet* es,
   return offset;
 }
 
-Int SK_(add_dep_event4)(EventSet* es,
+Int CLG_(add_dep_event4)(EventSet* es,
 			EventType* e1, EventType* e2,
 			EventType* e3, EventType* e4)
 {
@@ -175,7 +173,7 @@ Int SK_(add_dep_event4)(EventSet* es,
 }
 
 /* Returns number of characters written */
-Int SK_(sprint_eventset)(Char* buf, EventSet* es)
+Int CLG_(sprint_eventset)(Char* buf, EventSet* es)
 {
   Int i, pos = 0;
 
@@ -189,13 +187,13 @@ Int SK_(sprint_eventset)(Char* buf, EventSet* es)
 }
 
 /* Get cost array for an event set */
-ULong* SK_(get_eventset_cost)(EventSet* es)
+ULong* CLG_(get_eventset_cost)(EventSet* es)
 {
-  return SK_(get_costarray)(es->capacity);
+  return CLG_(get_costarray)(es->capacity);
 }
 
 /* Set all costs of an event set to zero */
-void SK_(init_cost)(EventSet* es, ULong* cost)
+void CLG_(init_cost)(EventSet* es, ULong* cost)
 {
   Int i;
 
@@ -206,19 +204,19 @@ void SK_(init_cost)(EventSet* es, ULong* cost)
 }
 
 /* Set all costs of an event set to zero */
-void SK_(init_cost_lz)(EventSet* es, ULong** cost)
+void CLG_(init_cost_lz)(EventSet* es, ULong** cost)
 {
   Int i;
 
-  CT_ASSERT(cost != 0);
+  CLG_ASSERT(cost != 0);
   if (!(*cost))
-    *cost = SK_(get_eventset_cost)(es);
+    *cost = CLG_(get_eventset_cost)(es);
 
   for(i=0;i<es->capacity;i++)
     (*cost)[i] = 0;
 }
 
-void SK_(zero_cost)(EventSet* es, ULong* cost)
+void CLG_(zero_cost)(EventSet* es, ULong* cost)
 {
   Int i;
 
@@ -228,7 +226,7 @@ void SK_(zero_cost)(EventSet* es, ULong* cost)
     cost[i] = 0;
 }
   
-Bool SK_(is_zero_cost)(EventSet* es, ULong* cost)
+Bool CLG_(is_zero_cost)(EventSet* es, ULong* cost)
 {
   Int i = 0;
 
@@ -241,12 +239,12 @@ Bool SK_(is_zero_cost)(EventSet* es, ULong* cost)
   return True;
 }
 
-Bool SK_(is_equal_cost)(EventSet* es, ULong* c1, ULong* c2)
+Bool CLG_(is_equal_cost)(EventSet* es, ULong* c1, ULong* c2)
 {
   Int i = 0;
 
-  if (!c1) return SK_(is_zero_cost)(es,c2);
-  if (!c2) return SK_(is_zero_cost)(es,c1);
+  if (!c1) return CLG_(is_zero_cost)(es,c2);
+  if (!c2) return CLG_(is_zero_cost)(es,c1);
 
   while(i<es->size) {
     if (c1[i] != c2[i]) return False;
@@ -258,45 +256,45 @@ Bool SK_(is_equal_cost)(EventSet* es, ULong* c1, ULong* c2)
   return True;
 }
 
-void SK_(copy_cost)(EventSet* es, ULong* dst, ULong* src)
+void CLG_(copy_cost)(EventSet* es, ULong* dst, ULong* src)
 {
   Int i;
 
   if (!src) {
-    SK_(zero_cost)(es, dst);
+    CLG_(zero_cost)(es, dst);
     return;
   }
-  CT_ASSERT(dst != 0);
+  CLG_ASSERT(dst != 0);
   
   for(i=0;i<es->size;i++)
     dst[i] = src[i];
 }
 
-void SK_(copy_cost_lz)(EventSet* es, ULong** pdst, ULong* src)
+void CLG_(copy_cost_lz)(EventSet* es, ULong** pdst, ULong* src)
 {
   Int i;
   ULong* dst;
 
-  CT_ASSERT(pdst != 0);
+  CLG_ASSERT(pdst != 0);
 
   if (!src) {
-    SK_(zero_cost)(es, *pdst);
+    CLG_(zero_cost)(es, *pdst);
     return;
   }
   dst = *pdst;
   if (!dst)
-    dst = *pdst = SK_(get_eventset_cost)(es);
+    dst = *pdst = CLG_(get_eventset_cost)(es);
   
   for(i=0;i<es->size;i++)
     dst[i] = src[i];
 }
 
-void SK_(add_cost)(EventSet* es, ULong* dst, ULong* src)
+void CLG_(add_cost)(EventSet* es, ULong* dst, ULong* src)
 {
   Int i = 0;
 
   if (!src) return;
-  CT_ASSERT(dst != 0);
+  CLG_ASSERT(dst != 0);
 
   while(i<es->size) {
     if (src[i] == 0)
@@ -308,18 +306,18 @@ void SK_(add_cost)(EventSet* es, ULong* dst, ULong* src)
   }
 }
 
-void SK_(add_cost_lz)(EventSet* es, ULong** pdst, ULong* src)
+void CLG_(add_cost_lz)(EventSet* es, ULong** pdst, ULong* src)
 {
   Int i;
   ULong* dst;
 
   if (!src) return;
-  CT_ASSERT(pdst != 0);
+  CLG_ASSERT(pdst != 0);
 
   dst = *pdst;
   if (!dst) {
-    dst = *pdst = SK_(get_eventset_cost)(es);
-    SK_(copy_cost)(es,dst,src);
+    dst = *pdst = CLG_(get_eventset_cost)(es);
+    CLG_(copy_cost)(es,dst,src);
     return;
   }
 
@@ -335,13 +333,13 @@ void SK_(add_cost_lz)(EventSet* es, ULong** pdst, ULong* src)
 }
 
 /* Adds src to dst and zeros src. Returns false if nothing changed */
-Bool SK_(add_and_zero_cost)(EventSet* es, ULong* dst, ULong* src)
+Bool CLG_(add_and_zero_cost)(EventSet* es, ULong* dst, ULong* src)
 {
   Int i = 0, j = 0;
 
-  CT_DEBUGIF(6) {
-    CT_DEBUG(6, "   add_and_zero_cost(%s, dst %p, src %p)\n", es->name, dst, src);
-    SK_(print_cost)(-5, es, src);
+  CLG_DEBUGIF(6) {
+    CLG_DEBUG(6, "   add_and_zero_cost(%s, dst %p, src %p)\n", es->name, dst, src);
+    CLG_(print_cost)(-5, es, src);
   }
 
   if (!es || !src) return False;
@@ -361,7 +359,7 @@ Bool SK_(add_and_zero_cost)(EventSet* es, ULong* dst, ULong* src)
 }
 
 /* Adds src to dst and zeros src. Returns false if nothing changed */
-Bool SK_(add_and_zero_cost_lz)(EventSet* es, ULong** pdst, ULong* src)
+Bool CLG_(add_and_zero_cost_lz)(EventSet* es, ULong** pdst, ULong* src)
 {
   Int i;
   ULong* dst;
@@ -375,12 +373,12 @@ Bool SK_(add_and_zero_cost_lz)(EventSet* es, ULong** pdst, ULong* src)
     i = es->e[i].nextTop;
   }
 
-  CT_ASSERT(pdst != 0);
+  CLG_ASSERT(pdst != 0);
   dst = *pdst;
   if (!dst) {
-    dst = *pdst = SK_(get_eventset_cost)(es);
-    SK_(copy_cost)(es,dst,src);
-    SK_(zero_cost)(es,src);
+    dst = *pdst = CLG_(get_eventset_cost)(es);
+    CLG_(copy_cost)(es,dst,src);
+    CLG_(zero_cost)(es,src);
     return True;
   }
 
@@ -402,7 +400,7 @@ Bool SK_(add_and_zero_cost_lz)(EventSet* es, ULong** pdst, ULong* src)
 
 /* Adds difference of new and old to dst, and set old to new.
  * Returns false if nothing changed */
-Bool SK_(add_diff_cost)(EventSet* es, ULong* dst, ULong* old, ULong* new)
+Bool CLG_(add_diff_cost)(EventSet* es, ULong* dst, ULong* old, ULong* new)
 {
   Int i = 0, j = 0;
 
@@ -422,14 +420,14 @@ Bool SK_(add_diff_cost)(EventSet* es, ULong* dst, ULong* old, ULong* new)
 
 /* Adds difference of new and old to dst, and set old to new.
  * Returns false if nothing changed */
-Bool SK_(add_diff_cost_lz)(EventSet* es, ULong** pdst, 
+Bool CLG_(add_diff_cost_lz)(EventSet* es, ULong** pdst, 
 			    ULong* old, ULong* new)
 {
   Int i;
   ULong* dst;
 
   if (!old && !new) return False;  
-  CT_ASSERT(old && new);
+  CLG_ASSERT(old && new);
 
   i = 0;
   while(1) {
@@ -438,11 +436,11 @@ Bool SK_(add_diff_cost_lz)(EventSet* es, ULong** pdst,
     i = es->e[i].nextTop;
   }
 
-  CT_ASSERT(pdst != 0);
+  CLG_ASSERT(pdst != 0);
   dst = *pdst;
   if (!dst) {
-    dst = *pdst = SK_(get_eventset_cost)(es);
-    SK_(zero_cost)(es,dst);
+    dst = *pdst = CLG_(get_eventset_cost)(es);
+    CLG_(zero_cost)(es,dst);
   }
 
   dst[i] += new[i] - old[i];
@@ -463,7 +461,7 @@ Bool SK_(add_diff_cost_lz)(EventSet* es, ULong** pdst,
 }
 
 /* Returns number of characters written */
-Int SK_(sprint_cost)(Char* buf, EventSet* es, ULong* c)
+Int CLG_(sprint_cost)(Char* buf, EventSet* es, ULong* c)
 {
   Int i, pos, skipped = 0;
 
@@ -495,13 +493,13 @@ Int SK_(sprint_cost)(Char* buf, EventSet* es, ULong* c)
 
 
 /* Allocate space for an event mapping */
-EventMapping* SK_(get_eventmapping)(EventSet* es)
+EventMapping* CLG_(get_eventmapping)(EventSet* es)
 {
   EventMapping* em;
 
-  CT_ASSERT(es != 0);
+  CLG_ASSERT(es != 0);
 
-  em = (EventMapping*) VG_(malloc)(sizeof(EventMapping) +
+  em = (EventMapping*) CLG_MALLOC(sizeof(EventMapping) +
 				   es->capacity * sizeof(Int));
   em->capacity = es->capacity;
   em->size = 0;
@@ -510,11 +508,11 @@ EventMapping* SK_(get_eventmapping)(EventSet* es)
   return em;
 }
 
-void SK_(append_event)(EventMapping* em, char* n)
+void CLG_(append_event)(EventMapping* em, Char* n)
 {
   Int i;
 
-  CT_ASSERT(em != 0);
+  CLG_ASSERT(em != 0);
 
   for(i=0; i<em->set->size; i++)
     if (VG_(strcmp)(n, em->set->e[i].type->name)==0)
@@ -522,7 +520,7 @@ void SK_(append_event)(EventMapping* em, char* n)
   
   if (i == em->set->size) return;
 
-  CT_ASSERT(em->capacity > em->size);
+  CLG_ASSERT(em->capacity > em->size);
 
   em->index[em->size] = i;
   em->size++;
@@ -530,11 +528,11 @@ void SK_(append_event)(EventMapping* em, char* n)
 
 
 /* Returns number of characters written */
-Int SK_(sprint_eventmapping)(Char* buf, EventMapping* em)
+Int CLG_(sprint_eventmapping)(Char* buf, EventMapping* em)
 {
   Int i, pos = 0;
 
-  CT_ASSERT(em != 0);
+  CLG_ASSERT(em != 0);
 
   for(i=0; i< em->size; i++) {
     if (pos>0) buf[pos++] = ' ';
@@ -546,7 +544,7 @@ Int SK_(sprint_eventmapping)(Char* buf, EventMapping* em)
 }
 
 /* Returns number of characters written */
-Int SK_(sprint_mappingcost)(Char* buf, EventMapping* em, ULong* c)
+Int CLG_(sprint_mappingcost)(Char* buf, EventMapping* em, ULong* c)
 {
   Int i, pos, skipped = 0;
 
