@@ -32,6 +32,124 @@
 
 static fn_array current_fn_active;
 
+static Addr runtime_resolve_addr = 0;
+static int  runtime_resolve_length = 0;
+
+/* _ld_runtime_resolve, located in needs special handling:
+ * The jump at end into the resolved function should not be
+ * represented as a call (as usually done in callgrind with jumps),
+ * but as a return + call. Otherwise, the repeated existance of
+ * _ld_runtime_resolve in call chains will lead to huge cycles,
+ * making the profile almost worthless.
+ *
+ * If ld.so is stripped, the symbol will not appear. But as this
+ * function is handcrafted assembler, we search for it...
+ *
+ * Returns 0 if code not found, otherwise start address
+ */
+static void search_runtime_resolve(obj_node* obj)
+{
+    /* We do not check target address of <fixup>, therefore we have >1 ranges.
+     * We use a tuple sequence (offset,length) into the code array for this
+     */
+
+#if defined(VGA_x86)
+    /* Check ranges [0-11], [16-23] */
+    static int  code_offsets[] = { 0, 12, 16, 8, 24, 0 };
+    static unsigned char code[] = {
+	/* 0*/ 0x50, 0x51, 0x52, 0x8b, 0x54, 0x24, 0x10, 0x8b,
+	/* 8*/ 0x44, 0x24, 0x0c, 0xe8, 0x70, 0x01, 0x00, 0x00,
+	/*16*/ 0x5a, 0x59, 0x87, 0x04, 0x24, 0xc2, 0x08, 0x00 };
+#else
+#if defined(VGA_ppc32)
+    static int  code_offsets[] = {0, 65, 68, 64, 132, 0 };
+    static unsigned char code[] = {
+	/* 0*/ 0x94, 0x21, 0xff, 0xc0, 0x90, 0x01, 0x00, 0x0c,
+	/* 8*/ 0x90, 0x61, 0x00, 0x10, 0x90, 0x81, 0x00, 0x14,
+	/*16*/ 0x7d, 0x83, 0x63, 0x78, 0x90, 0xa1, 0x00, 0x18,
+	/*24*/ 0x7d, 0x64, 0x5b, 0x78, 0x90, 0xc1, 0x00, 0x1c,
+	/*32*/ 0x7c, 0x08, 0x02, 0xa6, 0x90, 0xe1, 0x00, 0x20,
+	/*40*/ 0x90, 0x01, 0x00, 0x30, 0x91, 0x01, 0x00, 0x24,
+	/*48*/ 0x7c, 0x00, 0x00, 0x26, 0x91, 0x21, 0x00, 0x28,
+	/*56*/ 0x91, 0x41, 0x00, 0x2c, 0x90, 0x01, 0x00, 0x08,
+	/*64*/ 0x48, 0x00, 0x02, 0x91, 0x7c, 0x69, 0x03, 0xa6, /* at 64: bl aff0 <fixup> */
+	/*72*/ 0x80, 0x01, 0x00, 0x30, 0x81, 0x41, 0x00, 0x2c,
+	/*80*/ 0x81, 0x21, 0x00, 0x28, 0x7c, 0x08, 0x03, 0xa6,
+	/*88*/ 0x81, 0x01, 0x00, 0x24, 0x80, 0x01, 0x00, 0x08,
+	/*96*/ 0x80, 0xe1, 0x00, 0x20, 0x80, 0xc1, 0x00, 0x1c,
+	/*104*/0x7c, 0x0f, 0xf1, 0x20, 0x80, 0xa1, 0x00, 0x18,
+	/*112*/0x80, 0x81, 0x00, 0x14, 0x80, 0x61, 0x00, 0x10,
+	/*120*/0x80, 0x01, 0x00, 0x0c, 0x38, 0x21, 0x00, 0x40,
+	/*128*/0x4e, 0x80, 0x04, 0x20 };
+#else
+#if defined(VGA_amd64)
+    /* x86_64 */
+    static int  code_offsets[] = {0, 62, 66, 44, 110, 0 };
+    static unsigned char code[] = {
+	/* 0*/ 0x48, 0x83, 0xec, 0x38, 0x48, 0x89, 0x04, 0x24,
+	/* 8*/ 0x48, 0x89, 0x4c, 0x24, 0x08, 0x48, 0x89, 0x54, 0x24, 0x10,
+	/*18*/ 0x48, 0x89, 0x74, 0x24, 0x18, 0x48, 0x89, 0x7c, 0x24, 0x20,
+	/*28*/ 0x4c, 0x89, 0x44, 0x24, 0x28, 0x4c, 0x89, 0x4c, 0x24, 0x30,
+	/*38*/ 0x48, 0x8b, 0x74, 0x24, 0x40, 0x49, 0x89, 0xf3,
+	/*46*/ 0x4c, 0x01, 0xde, 0x4c, 0x01, 0xde, 0x48, 0xc1, 0xe6, 0x03,
+	/*56*/ 0x48, 0x8b, 0x7c, 0x24, 0x38, 0xe8, 0xee, 0x01, 0x00, 0x00,
+	/*66*/ 0x49, 0x89, 0xc3, 0x4c, 0x8b, 0x4c, 0x24, 0x30,
+	/*74*/ 0x4c, 0x8b, 0x44, 0x24, 0x28, 0x48, 0x8b, 0x7c, 0x24, 0x20,
+	/*84*/ 0x48, 0x8b, 0x74, 0x24, 0x18, 0x48, 0x8b, 0x54, 0x24, 0x10,
+	/*94*/ 0x48, 0x8b, 0x4c, 0x24, 0x08, 0x48, 0x8b, 0x04, 0x24,
+	/*103*/0x48, 0x83, 0xc4, 0x48, 0x41, 0xff, 0xe3 };
+#else
+    /* Unknown architecture, no check is done */
+    static int  code_offsets[] = {0, 0 };
+    static unsigned char code[] = { 0 };
+#endif
+#endif
+#endif
+    
+    int *range = &(code_offsets[0]), *r = 0;
+    Bool found = False;
+    Addr addr, end;
+
+    /* Only search in libraries with a given name pattern */
+    if ((VG_(strncmp)(obj->name, "/lib/ld", 7) != 0) &&
+	(VG_(strncmp)(obj->name, "/lib64/ld", 9) != 0)) return;
+    
+    CLG_DEBUG(1, "search_rs: Checking %d bytes of [%x %x %x...]\n",
+	      range[1], code[0], code[1], code[2]);
+
+    end = obj->start + obj->size - range[1];
+    addr = obj->start;
+    while(addr < end) {
+	if (VG_(memcmp)( (void*)addr, code, range[1]) == 0) {
+
+	    r = range + 2;
+	    found = True;
+	    while(r[1]) {
+		CLG_DEBUG(1, " [%p] Found! Checking %d bytes of [%x %x %x...]\n",
+			  addr, r[1], code[r[0]], code[r[0]+1], code[r[0]+2]);
+
+		if (VG_(memcmp)( (void*)(addr+r[0]), code+r[0], r[1]) != 0) {
+		    found = False;
+		    break;
+		}
+		r += 2;
+	    }
+	    if (found) break;
+	}
+	addr++;
+    }
+
+    if (!found || (r==0)) return;
+
+    if (VG_(clo_verbosity) > 1)
+	VG_(message)(Vg_DebugMsg, "Code check found runtime_resolve: %s +%p=%p, length %d",
+		     obj->name + obj->last_slash_pos,
+		     addr - obj->start, addr, r[0]);
+
+    runtime_resolve_addr   = addr;
+    runtime_resolve_length = r[0];
+}
+
 /*------------------------------------------------------------*/
 /*--- Object/File/Function hash entry operations           ---*/
 /*------------------------------------------------------------*/
@@ -39,7 +157,7 @@ static fn_array current_fn_active;
 /* Object hash table, fixed */
 static obj_node* obj_table[N_OBJ_ENTRIES];
 
-void SK_(init_obj_table)()
+void CLG_(init_obj_table)()
 {
     Int i;
     for (i = 0; i < N_OBJ_ENTRIES; i++)
@@ -48,7 +166,7 @@ void SK_(init_obj_table)()
 
 #define HASH_CONSTANT   256
 
-static UInt str_hash(const UChar *s, UInt table_size)
+static UInt str_hash(const Char *s, UInt table_size)
 {
     int hash_value = 0;
     for ( ; *s; s++)
@@ -57,47 +175,50 @@ static UInt str_hash(const UChar *s, UInt table_size)
 }
 
 
-static UChar* anonymous_obj = "???";
+static Char* anonymous_obj = "???";
 
 static __inline__ 
 obj_node* new_obj_node(SegInfo* si, obj_node* next)
 {
    Int i;
-   obj_node* new = VG_(malloc)(sizeof(obj_node));
-   new->name  = si ? (UChar*)VG_(strdup)( VG_(seg_filename)(si) )
+   obj_node* new;
+
+   new = (obj_node*) CLG_MALLOC(sizeof(obj_node));
+   new->name  = si ? VG_(strdup)( VG_(seginfo_filename)(si) )
                      : anonymous_obj;
    for (i = 0; i < N_FILE_ENTRIES; i++) {
       new->files[i] = NULL;
    }
-   SK_(stat).distinct_objs ++;
-   new->number  = SK_(stat).distinct_objs;
-   new->start   = si ? VG_(seg_start)(si) : 0;
-   new->size    = si ? VG_(seg_size)(si) : 0;
-   new->offset  = si ? VG_(seg_sym_offset)(si) : 0;
+   CLG_(stat).distinct_objs ++;
+   new->number  = CLG_(stat).distinct_objs;
+   new->start   = si ? VG_(seginfo_start)(si) : 0;
+   new->size    = si ? VG_(seginfo_size)(si) : 0;
+   new->offset  = si ? VG_(seginfo_sym_offset)(si) : 0;
    new->next    = next;
 
-#if CT_ENABLE_DEBUG  
+   // not only used for debug output (see static.c)
    new->last_slash_pos = 0;
    i = 0;
    while(new->name[i]) {
 	if (new->name[i]=='/') new->last_slash_pos = i+1;
 	i++;
    }
-#endif
 
    /* Dumping of static object info is only triggered once */
-   if (SK_(clo).dump_objs) SK_(dump_obj)(new);
+   if (CLG_(clo).dump_objs) CLG_(dump_obj)(new);
+
+   if (runtime_resolve_addr == 0) search_runtime_resolve(new);
    
    return new;
 }
 
-obj_node* SK_(get_obj_node)(SegInfo* si)
+obj_node* CLG_(get_obj_node)(SegInfo* si)
 {
     obj_node*    curr_obj_node;
     UInt         objname_hash;
     const UChar* obj_name;
     
-    obj_name = si ? VG_(seg_filename)(si) : anonymous_obj;
+    obj_name = si ? (Char*) VG_(seginfo_filename)(si) : anonymous_obj;
 
     /* lookup in obj hash */
     objname_hash = str_hash(obj_name, N_OBJ_ENTRIES);
@@ -120,20 +241,20 @@ file_node* new_file_node(Char filename[FILENAME_LEN],
 			 obj_node* obj, file_node* next)
 {
   Int i;
-  file_node* new = VG_(malloc)(sizeof(file_node));
+  file_node* new = (file_node*) CLG_MALLOC(sizeof(file_node));
   new->name  = VG_(strdup)(filename);
   for (i = 0; i < N_FN_ENTRIES; i++) {
     new->fns[i] = NULL;
   }
-  SK_(stat).distinct_files++;
-  new->number  = SK_(stat).distinct_files;
+  CLG_(stat).distinct_files++;
+  new->number  = CLG_(stat).distinct_files;
   new->obj     = obj;
   new->next      = next;
   return new;
 }
 
  
-file_node* SK_(get_file_node)(obj_node* curr_obj_node,
+file_node* CLG_(get_file_node)(obj_node* curr_obj_node,
 			      Char filename[FILENAME_LEN])
 {
     file_node* curr_file_node;
@@ -162,11 +283,11 @@ static __inline__
 fn_node* new_fn_node(Char fnname[FILENAME_LEN],
 		     file_node* file, fn_node* next)
 {
-    fn_node* new = VG_(malloc)(sizeof(fn_node));
+    fn_node* new = (fn_node*) CLG_MALLOC(sizeof(fn_node));
     new->name = VG_(strdup)(fnname);
 
-    SK_(stat).distinct_fns++;
-    new->number   = SK_(stat).distinct_fns;
+    CLG_(stat).distinct_fns++;
+    new->number   = CLG_(stat).distinct_fns;
     new->last_cxt = 0;
     new->pure_cxt = 0;
     new->file     = file;
@@ -177,6 +298,7 @@ fn_node* new_fn_node(Char fnname[FILENAME_LEN],
     new->zero_before  = False;
     new->toggle_collect = False;
     new->skip         = False;
+    new->pop_on_jump  = False;
     new->is_malloc    = False;
     new->is_realloc   = False;
     new->is_free      = False;
@@ -184,14 +306,14 @@ fn_node* new_fn_node(Char fnname[FILENAME_LEN],
     new->is_destructor = False;
 
     new->group        = 0;
-    new->separate_callers    = SK_(clo).separate_callers;
-    new->separate_recursions = SK_(clo).separate_recursions;
+    new->separate_callers    = CLG_(clo).separate_callers;
+    new->separate_recursions = CLG_(clo).separate_recursions;
 
-#if CT_ENABLE_DEBUG
+#if CLG_ENABLE_DEBUG
     new->verbosity    = -1;
 #endif
 
-    if (SK_(stat).distinct_fns >= current_fn_active.size)
+    if (CLG_(stat).distinct_fns >= current_fn_active.size)
 	resize_fn_array();
 
     return new;
@@ -208,7 +330,7 @@ fn_node* get_fn_node_infile(file_node* curr_file_node,
     fn_node* curr_fn_node;
     UInt     fnname_hash;
 
-    CT_ASSERT(curr_file_node != 0);
+    CLG_ASSERT(curr_file_node != 0);
 
     /* lookup in function hash */
     fnname_hash = str_hash(fnname, N_FN_ENTRIES);
@@ -235,61 +357,63 @@ fn_node* get_fn_node_inseg(SegInfo* si,
 			   Char filename[FILENAME_LEN],
 			   Char fnname[FN_NAME_LEN])
 {
-  obj_node  *obj  = SK_(get_obj_node)(si);
-  file_node *file = SK_(get_file_node)(obj, filename);
+  obj_node  *obj  = CLG_(get_obj_node)(si);
+  file_node *file = CLG_(get_file_node)(obj, filename);
   fn_node   *fn   = get_fn_node_infile(file, fnname);
 
   return fn;
 }
 
 
-Bool SK_(get_debug_info)(Addr instr_addr, 
+Bool CLG_(get_debug_info)(Addr instr_addr,
 			 Char filename[FILENAME_LEN],
-			 Char fn_name[FN_NAME_LEN], Int* line_num,
+			 Char fn_name[FN_NAME_LEN], UInt* line_num,
 			 SegInfo** pSegInfo)
 {
   Bool found1, found2, result = True;
-  int line;
+  UInt line;
   
-  CT_DEBUG(6, "  + get_debug_info(0x%08x)\n", instr_addr);
+  CLG_DEBUG(6, "  + get_debug_info(%p)\n", instr_addr);
 
   if (pSegInfo) {
-      *pSegInfo = VG_(get_obj)(instr_addr);
+      *pSegInfo = VG_(find_seginfo)(instr_addr);
 
       // for generated code in anonymous space, pSegInfo is 0
    }
 
-   found1 = VG_(get_filename_linenum)(instr_addr, filename,
-				      FILENAME_LEN, &line);
+   found1 = VG_(get_filename_linenum)(instr_addr,
+				      filename, FILENAME_LEN,
+				      NULL, 0, NULL, // FIXME: add dirnames!
+				      &line);
    found2 = VG_(get_fnname)(instr_addr, 
 			    fn_name, FN_NAME_LEN);
 
    if (!found1 && !found2) {
-     SK_(stat).no_debug_BBs++;
+     CLG_(stat).no_debug_BBs++;
      VG_(strcpy)(filename, "???");
      VG_(strcpy)(fn_name,  "???");
      if (line_num) *line_num=0;
      result = False;
 
    } else if ( found1 &&  found2) {
-     SK_(stat).full_debug_BBs++;
+     CLG_(stat).full_debug_BBs++;
      if (line_num) *line_num=line;
 
    } else if ( found1 && !found2) {
-     SK_(stat).file_line_debug_BBs++;
+     CLG_(stat).file_line_debug_BBs++;
      VG_(strcpy)(fn_name,  "???");
      if (line_num) *line_num=line;
 
    } else  /*(!found1 &&  found2)*/ {
-     SK_(stat).fn_name_debug_BBs++;
+     CLG_(stat).fn_name_debug_BBs++;
      VG_(strcpy)(filename, "???");
      if (line_num) *line_num=0;
    }
 
-   CT_DEBUG(6, "  - get_debug_info(0x%08x): seg %s\n, fn %s\n",
+   CLG_DEBUG(6, "  - get_debug_info(%p): seg '%s', fn %s\n",
 	    instr_addr,
 	    !pSegInfo   ? (const UChar*)"-" :
-	    (*pSegInfo) ? VG_(seg_filename)(*pSegInfo) :
+	    (*pSegInfo) ? VG_(seginfo_filename)(*pSegInfo) :
 	    (const UChar*)"(None)",
 	    fn_name);
 
@@ -338,7 +462,7 @@ static Int is_cppstructor(Char* n)
 		      end_pos-method_pos) == 0);
 
   if (res) {
-    CT_DEBUG(3,"  %s is %sstructor!", n, is_destr ? "De":"Con");
+    CLG_DEBUG(3,"  %s is %sstructor!", n, is_destr ? "De":"Con");
     return is_destr ? 2:1;
   }
   return 0;
@@ -348,28 +472,35 @@ static Int is_cppstructor(Char* n)
 /*
  * Attach function struct to a BB from debug info.
  */
-fn_node* SK_(get_fn_node)(BB* bb)
+fn_node* CLG_(get_fn_node)(BB* bb)
 {
     Char       filename[FILENAME_LEN], fnname[FN_NAME_LEN];
     SegInfo*   si;
-    Int        line_num;
+    UInt       line_num;
     fn_node*   fn;
 
     /* fn from debug info is idempotent for a BB */
     if (bb->fn) return bb->fn;
 
-    CT_DEBUG(3,"+ get_fn_node(BB 0x%x)\n", bb_addr(bb));
+    CLG_DEBUG(3,"+ get_fn_node(BB %p)\n", bb_addr(bb));
 
     /* get function/file name, line number and object of
      * the BB according to debug information
      */
-    SK_(get_debug_info)(bb_addr(bb),
+    CLG_(get_debug_info)(bb_addr(bb),
 			filename, fnname, &line_num, &si);
 
     if (0 == VG_(strcmp)(fnname, "???")) {
+	int p;
+
 	/* Use address as found in library */
-	VG_(sprintf)(fnname, "0x%08x%s", 
-		     bb->offset,
+	if (sizeof(Addr) == 4)
+	    p = VG_(sprintf)(fnname, "%08p", bb->offset);
+	else 	    
+	    // 64bit address
+	    p = VG_(sprintf)(fnname, "%016p", bb->offset);
+
+	VG_(sprintf)(fnname+p, "%s", 
 		     (bb->sect_kind == Vg_SectData) ? " [Data]" :
 		     (bb->sect_kind == Vg_SectBSS)  ? " [BSS]"  :
 		     (bb->sect_kind == Vg_SectGOT)  ? " [GOT]"  :
@@ -386,14 +517,21 @@ fn_node* SK_(get_fn_node)(BB* bb)
      */
     if (0 == VG_(strcmp)(fnname, "vgPlain___libc_freeres_wrapper")
 	&& exit_bb) {
-      SK_(get_debug_info)(bb_addr(exit_bb),
+      CLG_(get_debug_info)(bb_addr(exit_bb),
 			  filename, fnname, &line_num, &si);
 	
-	CT_DEBUG(1, "__libc_freeres_wrapper renamed to _exit\n");
+	CLG_DEBUG(1, "__libc_freeres_wrapper renamed to _exit\n");
     }
     if (0 == VG_(strcmp)(fnname, "_exit") && !exit_bb)
 	exit_bb = bb;
     
+    if (runtime_resolve_addr && 
+	(bb_addr(bb) >= runtime_resolve_addr) &&
+	(bb_addr(bb) < runtime_resolve_addr + runtime_resolve_length)) {
+	/* BB in runtime_resolve found by code check; use this name */
+	VG_(sprintf)(fnname, "_dl_runtime_resolve");
+    }
+
     /* get fn_node struct for this function */
     fn = get_fn_node_inseg( si, filename, fnname);
 
@@ -408,15 +546,24 @@ fn_node* SK_(get_fn_node)(BB* bb)
       fn_node* pure[2];
       pure[0] = 0;
       pure[1] = fn;
-      fn->pure_cxt = SK_(get_cxt)(pure+1);
+      fn->pure_cxt = CLG_(get_cxt)(pure+1);
 
       if (bb->sect_kind == Vg_SectPLT)	
-	fn->skip = SK_(clo).skip_plt;
+	fn->skip = CLG_(clo).skip_plt;
+
+      if (VG_(strcmp)(fn->name, "_dl_runtime_resolve")==0) {
+	  fn->pop_on_jump = True;
+
+	  if (VG_(clo_verbosity) > 1)
+	      VG_(message)(Vg_DebugMsg, "Symbol match: found runtime_resolve: %s +%p=%p",
+		      bb->obj->name + bb->obj->last_slash_pos,
+		      bb->offset, bb_addr(bb));
+      }
 
       fn->is_malloc  = (VG_(strcmp)(fn->name, "malloc")==0);
       fn->is_realloc = (VG_(strcmp)(fn->name, "realloc")==0);
       fn->is_free    = (VG_(strcmp)(fn->name, "free")==0);
-      if (SK_(clo).collect_data) {
+      if (CLG_(clo).collect_data) {
 	Int cppstr = is_cppstructor(fn->name);
 	if (cppstr == 1)
 	  fn->is_constructor = True;
@@ -426,14 +573,14 @@ fn_node* SK_(get_fn_node)(BB* bb)
 
       /* apply config options from function name patterns
        * given on command line */
-      SK_(update_fn_config)(fn);
+      CLG_(update_fn_config)(fn);
     }
 
 
     bb->fn   = fn;
     bb->line = line_num;
 
-    CT_DEBUG(3,"- get_fn_node(BB 0x%x): %s (in %s:%d)\n",
+    CLG_DEBUG(3,"- get_fn_node(BB %p): %s (in %s:%u)\n",
 	     bb_addr(bb), fnname, filename, line_num);
 
     return fn;
@@ -452,47 +599,47 @@ fn_node* SK_(get_fn_node)(BB* bb)
  * levels should be separated.
  */
 
-UInt* SK_(get_fn_entry)(Int n)
+UInt* CLG_(get_fn_entry)(Int n)
 {
-  CT_ASSERT(n < current_fn_active.size);
+  CLG_ASSERT(n < current_fn_active.size);
   return current_fn_active.array + n;
 }
 
-void SK_(init_fn_array)(fn_array* a)
+void CLG_(init_fn_array)(fn_array* a)
 {
   Int i;
 
-  CT_ASSERT(a != 0);
+  CLG_ASSERT(a != 0);
 
   a->size = N_INITIAL_FN_ARRAY_SIZE;
-  if (a->size <= SK_(stat).distinct_fns)
-    a->size = SK_(stat).distinct_fns+1;
+  if (a->size <= CLG_(stat).distinct_fns)
+    a->size = CLG_(stat).distinct_fns+1;
   
-  a->array = (UInt*) VG_(malloc)(a->size * sizeof(UInt));
+  a->array = (UInt*) CLG_MALLOC(a->size * sizeof(UInt));
   for(i=0;i<a->size;i++)
     a->array[i] = 0;
 }
 
-void SK_(copy_current_fn_array)(fn_array* dst)
+void CLG_(copy_current_fn_array)(fn_array* dst)
 {
-  CT_ASSERT(dst != 0);
+  CLG_ASSERT(dst != 0);
 
   dst->size  = current_fn_active.size;
   dst->array = current_fn_active.array;
 }
 
-fn_array* SK_(get_current_fn_array)()
+fn_array* CLG_(get_current_fn_array)()
 {
   return &current_fn_active;
 }
 
-void SK_(set_current_fn_array)(fn_array* a)
+void CLG_(set_current_fn_array)(fn_array* a)
 {
-  CT_ASSERT(a != 0);
+  CLG_ASSERT(a != 0);
 
   current_fn_active.size  = a->size;
   current_fn_active.array = a->array;
-  if (current_fn_active.size <= SK_(stat).distinct_fns)
+  if (current_fn_active.size <= CLG_(stat).distinct_fns)
     resize_fn_array();
 }
 
@@ -506,12 +653,12 @@ static void resize_fn_array()
     Int i, newsize;
 
     newsize = current_fn_active.size;
-    while (newsize <= SK_(stat).distinct_fns) newsize *=2;
+    while (newsize <= CLG_(stat).distinct_fns) newsize *=2;
 
-    CT_DEBUG(0, "Resize fn_active_array: %d => %d\n",
+    CLG_DEBUG(0, "Resize fn_active_array: %d => %d\n",
 	     current_fn_active.size, newsize);
 
-    new = VG_(malloc)(newsize * sizeof(UInt));
+    new = (UInt*) CLG_MALLOC(newsize * sizeof(UInt));
     for(i=0;i<current_fn_active.size;i++)
       new[i] = current_fn_active.array[i];
     while(i<newsize)
@@ -520,7 +667,7 @@ static void resize_fn_array()
     VG_(free)(current_fn_active.array);
     current_fn_active.size = newsize;
     current_fn_active.array = new;
-    SK_(stat).fn_array_resizes++;
+    CLG_(stat).fn_array_resizes++;
 }
 
 
