@@ -1,32 +1,61 @@
 /*--------------------------------------------------------------------*/
 /*--- Callgrind                                                    ---*/
-/*---                                                 ct_include.h ---*/
-/*--- (C) 2004, Josef Weidendorfer                                 ---*/
+/*---                                                     global.h ---*/
+/*--- (C) 2004, 2005 Josef Weidendorfer                            ---*/
 /*--------------------------------------------------------------------*/
 
+#ifndef CLG_GLOBAL
+#define CLG_GLOBAL
 
-#ifndef CT_INCLUDE
-#define CT_INCLUDE
+#include <pub_tool_basics.h>
+#include <pub_tool_debuginfo.h>
+#include <pub_tool_hashtable.h>
+#include <pub_tool_libcbase.h>
+#include <pub_tool_libcassert.h>
+#include <pub_tool_libcfile.h>
+#include <pub_tool_libcprint.h>
+#include <pub_tool_libcproc.h>
+#include <pub_tool_machine.h>
+#include <pub_tool_mallocfree.h>
+#include <pub_tool_options.h>
+#include <pub_tool_profile.h>
+#include <pub_tool_tooliface.h>
+
+#if VG_CORE_INTERFACE_VERSION > 8
+#include <pub_tool_clientstate.h>
+#endif
+
+#include "events.h" // defines CLG_ macro
+#include "costs.h"
+#include "hash.h"
+
+
+/* regparm is not used on ppc32, and gcc shouts at us a lot.
+ * So make it quiet.
+ */
+#if defined(VGA_ppc32)
+#  define CLG_REGPARM(n) /* */
+#else
+#  define CLG_REGPARM(n) __attribute__ ((regparm (n)))
+#endif
 
-#include "vg_skin.h"
-#include "events.h"
 
 /*------------------------------------------------------------*/
 /*--- Calltree compile options                            --- */
 /*------------------------------------------------------------*/
 
 /* Enable debug output */
-#define CT_ENABLE_DEBUG 1
+#define CLG_ENABLE_DEBUG 1
 
 /* Enable experimental features? */
-#define CT_EXPERIMENTAL 0
+#define CLG_EXPERIMENTAL 0
 
 /* Enable data collection ? */
-#define CT_DATA 1
+#define CLG_DATA 0
 
 /* Syscall Timing in microseconds? 
  * (define to 0 if you get compile errors) */
-#define CT_MICROSYSTIME 1
+#define CLG_MICROSYSTIME 0
 
 /* Set to 1 if you want full sanity checks for JCC */
 #define JCC_CHECK 0
@@ -78,8 +107,9 @@ struct _CommandLineOptions {
 
   /* Instrument options */
   Bool instrument_atstart;  /* Instrument at start? */
+  Bool simulate_cache;      /* Call into cache simulator ? */
 
-#if CT_ENABLE_DEBUG
+#if CLG_ENABLE_DEBUG
   Int   verbose;
   ULong verbose_start;
 #endif
@@ -195,15 +225,15 @@ typedef ULong* FullCost; /* Simulator + User */
  * After updating, <last> is set to current event counters. Thus,
  * events are not counted twice for recursive calls (TODO: True?)
  */
-
-#define JmpNone -1 
-#define JmpCond -2
+#define JmpNone (Ijk_Boring+30)
+#define JmpCond (Ijk_Boring+31)
 
 struct _jCC {
   Int  jmpkind;     /* JmpCall, JmpBoring, JmpCond */
   jCC* next_hash;   /* for hash entry chain */
   jCC* next_from;   /* next JCC from a BBCC */
   BBCC *from, *to;  /* call arc from/to this BBCC */
+  UInt jmp;         /* jump no. in source */
 
   ULong call_counter; /* no wraparound with 64 bit */
 
@@ -216,14 +246,23 @@ struct _jCC {
  */
 typedef struct _InstrInfo InstrInfo;
 struct _InstrInfo {
-  Int instr_offset;
-  Int instr_size;
-  Int data_size;
-  Int cost_offset;
+  UInt instr_offset;
+  UInt instr_size;
+  UInt data_size;
+  UInt cost_offset;
   EventSet* eventset;
 };
 
 
+/*
+ * Info for a conditional jump in a basic block
+ */
+typedef struct _CJmpInfo CJmpInfo;
+struct _CJmpInfo {
+    UInt instr; /* instruction index in this basic block */
+    Bool skip;   /* Cond.Jumps to next instruction should be ignored */
+};
+
 
 /**
  * An instrumented basic block (BB).
@@ -243,23 +282,27 @@ struct _InstrInfo {
  */
 struct _BB {
   obj_node*  obj;         /* ELF object of BB */
-  UInt       offset;      /* file offset of BB in object file */
+  OffT       offset;      /* offset of BB in ELF object file */
+  BB*        next;       /* chaining for a hash entry */
 
   VgSectKind sect_kind;  /* section of this BB, e.g. PLT */
   UInt       instr_count;
   
-  /* filled by SK_(get_fn_node) if debug info is available */
+  /* filled by CLG_(get_fn_node) if debug info is available */
   fn_node*   fn;          /* debug info for this BB */
-  Int        line;
+  UInt       line;
   Bool       is_entry;    /* True if this BB is a function entry */
         
   BBCC*      bbcc_list;  /* BBCCs for same BB (see next_bbcc in BBCC) */
   BBCC*      last_bbcc;  /* Temporary: Cached for faster access (LRU) */
 
-  BB*        next;       /* chaining for a hash entry */
+  /* filled by CLG_(instrument) if not seen before */
+  UInt       cjmp_count;  /* number of conditional exits */
+  CJmpInfo*  jmp;         /* array of info for condition jumps,
+			   * allocated directly after this struct */
+  Int        jmpkind;    /* remember jump kind of final exit */
+  Bool       cjmp_inverted; /* condition of last cond.jump can be inverted by VEX */
 
-  /* filled by SK_(instrument) if not seen before */
-  UInt       jmp_offset; /* offset to 1st jmp instr */
   UInt       instr_len;
   UInt       cost_count;
   InstrInfo  instr[0];   /* info on instruction sizes and costs */
@@ -286,11 +329,21 @@ struct _Context {
     UInt size;        // number of function dependencies
     UInt base_number; // for context compression & dump array
     Context* next;    // entry chaining for hash
-    UInt hash;        // for faster lookup...
+    UWord hash;       // for faster lookup...
     fn_node* fn[0];
 };
 
 
+/*
+ * Info for a conditional jump in a basic block
+ */
+typedef struct _JmpData JmpData;
+struct _JmpData {
+    ULong ecounter; /* number of times the BB was left at this exit */
+    jCC*  jcc_list;  /* JCCs for Cond.Jumps from this exit */
+};
+
+
 /*
  * Basic Block Cost Center
  *
@@ -307,27 +360,28 @@ struct _Context {
  * They are distinguishable by their tag field.
  */
 struct _BBCC {
-  BB*      bb;           /* BB for this cost center */
+    BB*      bb;           /* BB for this cost center */
 
-  Context* cxt;          /* execution context of this BBCC */
-  ThreadId tid;          /* only for assertion check purpose */
-  UInt     rec_index;    /* Recursion index in rec->bbcc for this bbcc */
-  BBCC**   rec_array;    /* Variable sized array of pointers to 
-			  * recursion BBCCs. Shared. */
-  ULong    exe_counter;  /* execution counter for BB in this context */
-  ULong    ret_counter;  /* how often returned from jccs of this bbcc */
-
-  BBCC*    next_bbcc;    /* Chain of BBCCs for same BB */
-  BBCC*    lru_next_bbcc; /* BBCC executed next the last time */
+    Context* cxt;          /* execution context of this BBCC */
+    ThreadId tid;          /* only for assertion check purpose */
+    UInt     rec_index;    /* Recursion index in rec->bbcc for this bbcc */
+    BBCC**   rec_array;    /* Variable sized array of pointers to 
+			    * recursion BBCCs. Shared. */
+    ULong    ret_counter;  /* how often returned from jccs of this bbcc;
+			    * used to check if a dump for this BBCC is needed */
     
-  jCC*     lru_from_jcc; /* Temporary: Cached for faster access (LRU) */
-  jCC*     lru_to_jcc;   /* Temporary: Cached for faster access (LRU) */
-  jCC*     jcc_list;     /* list of arcs called from jmp_addr */
-  FullCost skipped;      /* cost for skipped functions called from 
-			  * jmp_addr. Allocated lazy */
-
-  BBCC*    next;         /* entry chain in hash */
-  ULong*   cost;         /* start of 64bit costs for this BBCC */
+    BBCC*    next_bbcc;    /* Chain of BBCCs for same BB */
+    BBCC*    lru_next_bbcc; /* BBCC executed next the last time */
+    
+    jCC*     lru_from_jcc; /* Temporary: Cached for faster access (LRU) */
+    jCC*     lru_to_jcc;   /* Temporary: Cached for faster access (LRU) */
+    FullCost skipped;      /* cost for skipped functions called from 
+			    * jmp_addr. Allocated lazy */
+    
+    BBCC*    next;         /* entry chain in hash */
+    ULong*   cost;         /* start of 64bit costs for this BBCC */
+    ULong    ecounter_sum; /* execution counter for first instruction of BB */
+    JmpData  jmp[0];
 };
 
 
@@ -348,6 +402,7 @@ struct _fn_node {
   Bool zero_before :1;
   Bool toggle_collect :1;
   Bool skip :1;
+  Bool pop_on_jump : 1;
 
   Bool is_malloc :1;
   Bool is_realloc :1;
@@ -358,7 +413,7 @@ struct _fn_node {
   Int  group;
   Int  separate_callers;
   Int  separate_recursions;
-#if CT_ENABLE_DEBUG
+#if CLG_ENABLE_DEBUG
   Int  verbosity; /* Stores old verbosity level while in function */
 #endif
 };
@@ -384,13 +439,11 @@ struct _file_node {
  */
 struct _obj_node {
    Char*      name;
-   UInt       size;
-#if CT_ENABLE_DEBUG
    UInt       last_slash_pos;
-#endif
 
    Addr       start;  /* Start address of text segment mapping */
-   UInt       offset; /* Offset between symbol address and file offset */
+   SizeT      size;   /* Length of mapping */
+   OffT       offset; /* Offset between symbol address and file offset */
 
    file_node* files[N_FILE_ENTRIES];
    UInt       number;
@@ -407,7 +460,9 @@ struct _obj_node {
 struct _call_entry {
     jCC* jcc;           /* jCC for this call */
     FullCost enter_cost; /* cost event counters at entering frame */
-    Addr esp;           /* ESP at call time */
+    Addr sp;            /* stack pointer directly after call */
+    Addr ret_addr;      /* address to which to return to
+			 * is 0 on a simulated call */
     BBCC* nonskipped;   /* see above */
     Context* cxt;       /* context before call */
     Int fn_sp;          /* function stack index before call */
@@ -439,8 +494,7 @@ struct _exec_state {
   Bool     collect;
   Context* cxt;
   
-  Int   jmpkind;   /* kind of jump from last BB executed
-		    * -1 is "unset", -2 is "conditional jump" */
+  Int   jmps_passed; /* number of conditional jumps passed in last BB */
   BBCC* bbcc;      /* last BB executed */
   BBCC* nonskipped;
 
@@ -450,13 +504,13 @@ struct _exec_state {
 /* Global state structures */
 typedef struct _bb_hash bb_hash;
 struct _bb_hash {
-  Int size, entries;
+  UInt size, entries;
   BB** table;
 };
 
 typedef struct _cxt_hash cxt_hash;
 struct _cxt_hash {
-  Int size, entries;
+  UInt size, entries;
   Context** table;
 };  
 
@@ -466,32 +520,33 @@ struct _cxt_hash {
  */
 typedef struct _bbcc_hash bbcc_hash;
 struct _bbcc_hash {
-  Int size, entries;
+  UInt size, entries;
   BBCC** table;
 };
 
 typedef struct _jcc_hash jcc_hash;
 struct _jcc_hash {
-  Int size, entries;
+  UInt size, entries;
   jCC** table;
   jCC* spontaneous;
 };
 
 typedef struct _fn_array fn_array;
 struct _fn_array {
-  Int size;
+  UInt size;
   UInt* array;
 };
 
 typedef struct _call_stack call_stack;
 struct _call_stack {
-  Int size, sp;
+  UInt size;
+  Int sp;
   call_entry* entry;
 };
 
 typedef struct _fn_stack fn_stack;
 struct _fn_stack {
-  Int size;
+  UInt size;
   fn_node **bottom, **top;
 };
 
@@ -541,10 +596,10 @@ struct _thread_info {
  */
 typedef struct _AddrPos AddrPos;
 struct _AddrPos {
-    UInt addr;
+    Addr addr;
     Addr bb_addr;
     file_node* file;
-    int line;
+    UInt line;
 };
 
 /* a simulator cost entity that can be written out in one line */
@@ -569,24 +624,32 @@ struct _FnPos {
 /*--- Cache simulator interface                            ---*/
 /*------------------------------------------------------------*/
 
-#define CT_REGPARM(n) __attribute__ ((regparm (n)))
-
 struct cachesim_if
 {
-  void (*print_opts)(void);
-  Bool (*parse_opt)(Char* arg);
-  void (*post_clo_init)(void);
-  void (*clear)(void);
-  void (*getdesc)(Char* buf);
-  void (*printstat)(void);  
-  void (*add_icost)(SimCost, BBCC*, InstrInfo*);
-  void (*after_bbsetup)(void);
-  void (*finish)(void);
-  
-  CT_REGPARM(1) void (*log_0D)(InstrInfo* ii);
-  CT_REGPARM(2) void (*log_1Dr)(InstrInfo* ii, Addr data);
-  CT_REGPARM(2) void (*log_1Dw)(InstrInfo* ii, Addr data);
-  CT_REGPARM(3) void (*log_2D)(InstrInfo* ii, Addr data1, Addr data2);
+    void (*print_opts)(void);
+    Bool (*parse_opt)(Char* arg);
+    void (*post_clo_init)(void);
+    void (*clear)(void);
+    void (*getdesc)(Char* buf);
+    void (*printstat)(void);  
+    void (*add_icost)(SimCost, BBCC*, InstrInfo*, ULong);
+    void (*after_bbsetup)(void);
+    void (*finish)(void);
+    
+    void (*log_1I0D)(InstrInfo*) CLG_REGPARM(1);
+
+    void (*log_1I1Dr)(InstrInfo*, Addr) CLG_REGPARM(2);
+    void (*log_1I1Dw)(InstrInfo*, Addr) CLG_REGPARM(2);
+    void (*log_1I2D)(InstrInfo*, Addr, Addr) CLG_REGPARM(3);
+
+    void (*log_0I1Dr)(InstrInfo*, Addr) CLG_REGPARM(2);
+    void (*log_0I1Dw)(InstrInfo*, Addr) CLG_REGPARM(2);
+    void (*log_0I2D)(InstrInfo*, Addr, Addr) CLG_REGPARM(3);
+
+    // function names of helpers (for debugging generated code)
+    Char *log_1I0D_name;
+    Char *log_1I1Dr_name, *log_1I1Dw_name, *log_1I2D_name;
+    Char *log_0I1Dr_name, *log_0I1Dw_name, *log_0I2D_name;
 };
 
 
@@ -596,9 +659,11 @@ struct cachesim_if
 
 /* from clo.c */
 
-void SK_(set_clo_defaults)();
-void SK_(update_fn_config)(fn_node*);
-
+void CLG_(set_clo_defaults)();
+void CLG_(update_fn_config)(fn_node*);
+Bool CLG_(process_cmd_line_option)(Char*);
+void CLG_(print_usage)(void);
+void CLG_(print_debug_usage)(void);
 
 /* from sim.c */
 struct event_sets {
@@ -613,191 +678,203 @@ struct event_sets {
   Int off_full_user, off_full_alloc, off_full_systime;
 };
 
-extern struct event_sets SK_(sets);
-extern struct cachesim_if SK_(cachesim);
+extern struct event_sets CLG_(sets);
+extern struct cachesim_if CLG_(cachesim);
 
-void SK_(init_eventsets)(Int user);
+void CLG_(init_eventsets)(Int user);
 
 /* from main.c */
-Bool SK_(get_debug_info)(Addr, Char filename[FILENAME_LEN],
-			 Char fn_name[FN_NAME_LEN], Int*, SegInfo**);
-void SK_(set_instrument_state)(Char*,Bool);
-void SK_(dump_profile)(Char* trigger,Bool only_current_thread);
-void SK_(zero_all_cost)(Bool only_current_thread);
-Int SK_(get_dump_counter)(void);
+Bool CLG_(get_debug_info)(Addr, Char filename[FILENAME_LEN],
+			 Char fn_name[FN_NAME_LEN], UInt*, SegInfo**);
+void CLG_(collectBlockInfo)(IRBB* bbIn, UInt*, UInt*, Bool*);
+void CLG_(set_instrument_state)(Char*,Bool);
+void CLG_(dump_profile)(Char* trigger,Bool only_current_thread);
+void CLG_(zero_all_cost)(Bool only_current_thread);
+Int CLG_(get_dump_counter)(void);
+void CLG_(fini)(Int exitcode);
 
 /* from command.c */
-void SK_(init_command)(Char* dir, Char* dumps);
-void SK_(check_command)(void);
-void SK_(finish_command)(void);
+void CLG_(init_command)(Char* dir, Char* dumps);
+void CLG_(check_command)(void);
+void CLG_(finish_command)(void);
 
 /* from bb.c */
-void SK_(init_bb_hash)();
-bb_hash* SK_(get_bb_hash)();
-BB*  SK_(get_bb)(Addr addr, UCodeBlock* cb_in, Bool *seen_before);
+void CLG_(init_bb_hash)();
+bb_hash* CLG_(get_bb_hash)();
+BB*  CLG_(get_bb)(Addr addr, IRBB* bb_in, Bool *seen_before);
+void CLG_(delete_bb)(Addr addr);
 
 static __inline__ Addr bb_addr(BB* bb)
  { return bb->offset + bb->obj->offset; }
 static __inline__ Addr bb_jmpaddr(BB* bb)
- { return bb->jmp_offset + bb->offset + bb->obj->offset; }
+ { return bb->instr[bb->instr_count-1].instr_offset + bb->offset + bb->obj->offset; }
 
 /* from fn.c */
-void SK_(init_fn_array)(fn_array*);
-void SK_(copy_current_fn_array)(fn_array* dst);
-fn_array* SK_(get_current_fn_array)();
-void SK_(set_current_fn_array)(fn_array*);
-UInt* SK_(get_fn_entry)(Int n);
+void CLG_(init_fn_array)(fn_array*);
+void CLG_(copy_current_fn_array)(fn_array* dst);
+fn_array* CLG_(get_current_fn_array)();
+void CLG_(set_current_fn_array)(fn_array*);
+UInt* CLG_(get_fn_entry)(Int n);
 
-void      SK_(init_obj_table)();
-obj_node* SK_(get_obj_node)(SegInfo* si);
-file_node* SK_(get_file_node)(obj_node*, Char* filename);
-fn_node*  SK_(get_fn_node)(BB* bb);
+void      CLG_(init_obj_table)();
+obj_node* CLG_(get_obj_node)(SegInfo* si);
+file_node* CLG_(get_file_node)(obj_node*, Char* filename);
+fn_node*  CLG_(get_fn_node)(BB* bb);
 
 /* from bbcc.c */
-void SK_(init_bbcc_hash)(bbcc_hash* bbccs);
-void SK_(copy_current_bbcc_hash)(bbcc_hash* dst);
-bbcc_hash* SK_(get_current_bbcc_hash)();
-void SK_(set_current_bbcc_hash)(bbcc_hash*);
-void SK_(forall_bbccs)(void (*func)(BBCC*));
-void SK_(zero_bbcc)(BBCC* bbcc);
-BBCC* SK_(get_bbcc)(BB* bb);
-BBCC* SK_(clone_bbcc)(BBCC* orig, Context* cxt, Int rec_index);
-__attribute__ ((regparm (1))) void SK_(setup_bbcc)(BB* bb);
+void CLG_(init_bbcc_hash)(bbcc_hash* bbccs);
+void CLG_(copy_current_bbcc_hash)(bbcc_hash* dst);
+bbcc_hash* CLG_(get_current_bbcc_hash)();
+void CLG_(set_current_bbcc_hash)(bbcc_hash*);
+void CLG_(forall_bbccs)(void (*func)(BBCC*));
+void CLG_(zero_bbcc)(BBCC* bbcc);
+BBCC* CLG_(get_bbcc)(BB* bb);
+BBCC* CLG_(clone_bbcc)(BBCC* orig, Context* cxt, Int rec_index);
+void CLG_(setup_bbcc)(BB* bb) CLG_REGPARM(1);
 
 
 /* from jumps.c */
-void SK_(init_jcc_hash)(jcc_hash*);
-void SK_(copy_current_jcc_hash)(jcc_hash* dst);
-jcc_hash* SK_(get_current_jcc_hash)();
-void SK_(set_current_jcc_hash)(jcc_hash*);
-jCC* SK_(get_jcc)(BBCC* from, BBCC* to);
+void CLG_(init_jcc_hash)(jcc_hash*);
+void CLG_(copy_current_jcc_hash)(jcc_hash* dst);
+jcc_hash* CLG_(get_current_jcc_hash)();
+void CLG_(set_current_jcc_hash)(jcc_hash*);
+jCC* CLG_(get_jcc)(BBCC* from, UInt, BBCC* to);
 
 /* from callstack.c */
-void SK_(init_call_stack)(call_stack*);
-void SK_(copy_current_call_stack)(call_stack* dst);
-void SK_(set_current_call_stack)(call_stack*);
-call_entry* SK_(get_call_entry)(Int n);
+void CLG_(init_call_stack)(call_stack*);
+void CLG_(copy_current_call_stack)(call_stack* dst);
+void CLG_(set_current_call_stack)(call_stack*);
+call_entry* CLG_(get_call_entry)(Int n);
 
-void SK_(push_call_stack)(BBCC* from, BBCC* to, Addr esp, Bool skip);
-void SK_(pop_call_stack)();
-Int SK_(unwind_call_stack)(Addr esp);
+void CLG_(push_call_stack)(BBCC* from, UInt jmp, BBCC* to, Addr sp, Bool skip);
+void CLG_(pop_call_stack)();
+void CLG_(unwind_call_stack)(Addr sp, Int);
 
 /* from context.c */
-void SK_(init_fn_stack)(fn_stack*);
-void SK_(copy_current_fn_stack)(fn_stack*);
-fn_stack* SK_(get_current_fn_stack)();
-void SK_(set_current_fn_stack)(fn_stack*);
+void CLG_(init_fn_stack)(fn_stack*);
+void CLG_(copy_current_fn_stack)(fn_stack*);
+fn_stack* CLG_(get_current_fn_stack)();
+void CLG_(set_current_fn_stack)(fn_stack*);
 
-void SK_(init_cxt_table)();
-cxt_hash* SK_(get_cxt_hash)();
-Context* SK_(get_cxt)(fn_node** fn);
-void SK_(push_cxt)(fn_node* fn);
+void CLG_(init_cxt_table)();
+cxt_hash* CLG_(get_cxt_hash)();
+Context* CLG_(get_cxt)(fn_node** fn);
+void CLG_(push_cxt)(fn_node* fn);
 
 /* from threads.c */
-void SK_(init_threads)();
-thread_info** SK_(get_threads)();
-thread_info* SK_(get_current_thread)();
-void SK_(switch_thread)(ThreadId tid);
-void SK_(forall_threads)(void (*func)(thread_info*));
-void SK_(run_thread)(ThreadId tid);
-
-void SK_(init_exec_state)(exec_state* es);
-void SK_(init_exec_stack)(exec_stack*);
-void SK_(copy_current_exec_stack)(exec_stack*);
-void SK_(set_current_exec_stack)(exec_stack*);
-void SK_(pre_signal)(ThreadId tid, Int sigNum, Bool alt_stack);
-void SK_(post_signal)(ThreadId tid, Int sigNum);
+void CLG_(init_threads)();
+thread_info** CLG_(get_threads)();
+thread_info* CLG_(get_current_thread)();
+void CLG_(switch_thread)(ThreadId tid);
+void CLG_(forall_threads)(void (*func)(thread_info*));
+void CLG_(run_thread)(ThreadId tid);
 
+void CLG_(init_exec_state)(exec_state* es);
+void CLG_(init_exec_stack)(exec_stack*);
+void CLG_(copy_current_exec_stack)(exec_stack*);
+void CLG_(set_current_exec_stack)(exec_stack*);
+void CLG_(pre_signal)(ThreadId tid, Int sigNum, Bool alt_stack);
+void CLG_(post_signal)(ThreadId tid, Int sigNum);
+void CLG_(run_post_signal_on_call_stack_bottom)();
 
 /* from dump.c */
-extern FullCost SK_(total_cost);
-void SK_(init_files)(Char** dir, Char** file);
-Char* SK_(get_dump_file_base)();
+extern FullCost CLG_(total_cost);
+void CLG_(init_files)(Char** dir, Char** file);
+Char* CLG_(get_dump_file_base)();
 
 
 /* from static.c */
-void SK_(dump_obj)(obj_node*);
+void CLG_(dump_obj)(obj_node*);
 
 
 /* from data.c */
-void SK_(init_data)();
-void SK_(finish_data)();
-void SK_(handle_malloc)(UInt start, UInt size);
-Int  SK_(handle_realloc)(UInt newstart, UInt oldstart, UInt size);
-Int  SK_(handle_free)(UInt addr);
-void SK_(handle_constructor)(UInt addr, Char* fullname);
-void SK_(handle_destructor)(UInt addr, Char* fullname);
-void SK_(handle_read)(UInt addr, UInt size, int sim);
-void SK_(handle_write)(UInt addr, UInt size, int sim);
+void CLG_(init_data)();
+void CLG_(finish_data)();
+void CLG_(handle_malloc)(Addr start, SizeT size);
+Int  CLG_(handle_realloc)(Addr newstart, Addr oldstart, SizeT size);
+Int  CLG_(handle_free)(Addr addr);
+void CLG_(handle_constructor)(Addr addr, Char* fullname);
+void CLG_(handle_destructor)(Addr addr, Char* fullname);
+void CLG_(handle_read)(Addr addr, UInt size, int sim);
+void CLG_(handle_write)(Addr addr, UInt size, int sim);
 
-#if VG_CORE_INTERFACE_MAJOR_VERSION < 7
-typedef UInt SizeT;
-#endif
-
-void SK_(my_new_mem_startup)(Addr a, SizeT len, Bool rr, Bool ww, Bool xx);
-void SK_(my_new_mem_stack_signal)(Addr a, SizeT len);
-void SK_(my_new_mem_brk)(Addr a, SizeT len);
-void SK_(my_new_mem_mmap)(Addr a, SizeT len, Bool rr, Bool ww, Bool xx);
+void CLG_(my_new_mem_startup)(Addr a, SizeT len, Bool rr, Bool ww, Bool xx);
+void CLG_(my_new_mem_stack_signal)(Addr a, SizeT len);
+void CLG_(my_new_mem_brk)(Addr a, SizeT len);
+void CLG_(my_new_mem_mmap)(Addr a, SizeT len, Bool rr, Bool ww, Bool xx);
 
 /*------------------------------------------------------------*/
 /*--- Exported global variables                            ---*/
 /*------------------------------------------------------------*/
 
-extern CommandLineOptions SK_(clo);
-extern Statistics SK_(stat);
-extern EventMapping* SK_(dumpmap);
+extern CommandLineOptions CLG_(clo);
+extern Statistics CLG_(stat);
+extern EventMapping* CLG_(dumpmap);
 
 /* Function active counter array, indexed by function number */
-extern UInt* SK_(fn_active_array);
-extern Bool SK_(instrument_state);
+extern UInt* CLG_(fn_active_array);
+extern Bool CLG_(instrument_state);
+
+extern call_stack CLG_(current_call_stack);
+extern fn_stack   CLG_(current_fn_stack);
+extern exec_state CLG_(current_state);
+extern ThreadId   CLG_(current_tid);
 
-extern call_stack SK_(current_call_stack);
-extern fn_stack   SK_(current_fn_stack);
-extern exec_state SK_(current_state);
-extern ThreadId   SK_(current_tid);
 
 /*------------------------------------------------------------*/
-/*--- Debugging                                            ---*/
+/*--- Debug output                                         ---*/
 /*------------------------------------------------------------*/
 
-#if CT_ENABLE_DEBUG
+#if CLG_ENABLE_DEBUG
 
-#define CT_DEBUGIF(x) \
-  if ( (SK_(clo).verbose >x) && \
-       (SK_(stat).bb_executions >= SK_(clo).verbose_start))
+#define CLG_DEBUGIF(x) \
+  if ( (CLG_(clo).verbose >x) && \
+       (CLG_(stat).bb_executions >= CLG_(clo).verbose_start))
 
-#define CT_DEBUG(x,format,args...)    \
-    CT_DEBUGIF(x) {                   \
-      SK_(print_bbno)();	      \
+#define CLG_DEBUG(x,format,args...)   \
+    CLG_DEBUGIF(x) {                  \
+      CLG_(print_bbno)();	      \
       VG_(printf)(format,##args);     \
     }
 
-#define CT_ASSERT(cond)               \
+#define CLG_ASSERT(cond)              \
     if (!(cond)) {                    \
-      SK_(print_context)();           \
-      SK_(print_bbno)();	      \
-      sk_assert(cond);                \
+      CLG_(print_context)();          \
+      CLG_(print_bbno)();	      \
+      tl_assert(cond);                \
      }
 
 #else
-#define CT_DEBUGIF(x) if (0)
-#define CT_DEBUG(x...) {}
-#define CT_ASSERT(cond) sk_assert(cond);
+#define CLG_DEBUGIF(x) if (0)
+#define CLG_DEBUG(x...) {}
+#define CLG_ASSERT(cond) tl_assert(cond);
 #endif
 
-/* from ct_debug.c */
-void SK_(print_bbno)(void);
-void SK_(print_context)(void);
-void SK_(print_jcc)(int s, jCC* jcc);
-void SK_(print_bbcc)(int s, BBCC* bbcc, Bool);
-void SK_(print_bbcc_fn)(BBCC* bbcc);
-void SK_(print_eventset)(int s, EventSet* es);
-void SK_(print_cost)(int s, EventSet*, ULong* cost);
-void SK_(print_bb)(int s, BB* bb);
-void SK_(print_cxt)(int s, Context* cxt, int rec_index);
-void SK_(print_short_jcc)(jCC* jcc);
-void SK_(print_stackentry)(int s, int sp);
-void SK_(print_addr)(Addr addr);
-void SK_(print_addr_ln)(Addr addr);
+/* from debug.c */
+void CLG_(print_bbno)(void);
+void CLG_(print_context)(void);
+void CLG_(print_jcc)(int s, jCC* jcc);
+void CLG_(print_bbcc)(int s, BBCC* bbcc, Bool);
+void CLG_(print_bbcc_fn)(BBCC* bbcc);
+void CLG_(print_execstate)(int s, exec_state* es);
+void CLG_(print_eventset)(int s, EventSet* es);
+void CLG_(print_cost)(int s, EventSet*, ULong* cost);
+void CLG_(print_bb)(int s, BB* bb);
+void CLG_(print_bbcc_cost)(int s, BBCC*);
+void CLG_(print_cxt)(int s, Context* cxt, int rec_index);
+void CLG_(print_short_jcc)(jCC* jcc);
+void CLG_(print_stackentry)(int s, int sp);
+void CLG_(print_addr)(Addr addr);
+void CLG_(print_addr_ln)(Addr addr);
 
-#endif /* CT_INCLUDE */
+#if 0
+void* CLG_(malloc)(UWord s, char* f);
+void* CLG_(free)(void* p, char* f);
+#define CLG_MALLOC(x) CLG_(malloc)(x,__FUNCTION__)
+#define CLG_FREE(p)   CLG_(free)(p,__FUNCTION__)
+#else
+#define CLG_MALLOC(x) VG_(malloc)(x)
+#define CLG_FREE(p)   VG_(free)(p)
+#endif
+
+#endif /* CLG_GLOBAL */
